CREATE TABLE cst_customer
(
  cst_id INT NOT NULL,
  cst_last_name VARCHAR(30) NOT NULL,
  cst_first_name VARCHAR(30),
  adr_id INT NOT NULL,
  CONSTRAINT PK_cst_customer PRIMARY KEY (cst_id)
);



CREATE TABLE car_car
(
  car_license_plate VARCHAR(9) NOT NULL,
  clr_rgb LONGINT NOT NULL,
  mdl_name VARCHAR(20) NOT NULL,
  cst_id INT NOT NULL,
  CONSTRAINT PK_car_car PRIMARY KEY (car_license_plate)
);



CREATE TABLE mdl_model
(
  mdl_name VARCHAR(20) NOT NULL,
  mdl_manufacturer VARCHAR(20) NOT NULL,
  CONSTRAINT PK_mdl_model PRIMARY KEY (mdl_name)
);



CREATE TABLE stp_service_type
(
  stp_code CHAR(4) NOT NULL,
  stp_description VARCHAR(30) NOT NULL,
  CONSTRAINT PK_stp_service_type PRIMARY KEY (stp_code)
);



CREATE TABLE adr_address
(
  adr_house_name_num VARCHAR(30) NOT NULL,
  adr_street VARCHAR(30) NOT NULL,
  adr_postal_code VARCHAR(30) NOT NULL,
  adr_city VARCHAR(30),
  adr_province VARCHAR(30),
  adr_id INT NOT NULL,
  CONSTRAINT PK_adr_address PRIMARY KEY (adr_id)
);



CREATE TABLE emp_employee
(
  emp_id INT NOT NULL,
  emp_last_name VARCHAR(30) NOT NULL,
  emp_first_name VARCHAR(30) NOT NULL,
  emp_mch_labour_rate LONGINT,
  CONSTRAINT PK_emp_employee PRIMARY KEY (emp_id)
);



CREATE TABLE clr_colour
(
  clr_rgb LONGINT NOT NULL,
  clr_red SMALLINT NOT NULL,
  clr_green SMALLINT NOT NULL,
  clr_blue SMALLINT NOT NULL,
  clr_description VARCHAR(30) NOT NULL,
  CONSTRAINT PK_clr_colour PRIMARY KEY (clr_rgb)
);



CREATE TABLE svc_service
(
  car_license_plate VARCHAR(9) NOT NULL,
  svc_booked_in DATE NOT NULL,
  svc_mileage INT,
  svc_picked_up DATE,
  stp_code CHAR(4) NOT NULL,
  mdl_name VARCHAR(20),
  mdl_mileage INT,
  svc_lbr_adjustments LONGINT,
  svc_cim_adjustments LONGINT,
  CONSTRAINT PK_svc_service PRIMARY KEY (car_license_plate, svc_booked_in)
);



CREATE TABLE rgs_regular_service
(
  mdl_name VARCHAR(20) NOT NULL,
  mdl_mileage INT NOT NULL,
  CONSTRAINT PK_rgs_regular_service PRIMARY KEY (mdl_name, mdl_mileage)
);



CREATE TABLE lbr_labour
(
  lbr_hours INT NOT NULL,
  lbr_notes VARCHAR(255),
  car_license_plate VARCHAR(9) NOT NULL,
  svc_booked_in DATE NOT NULL,
  emp_id INT NOT NULL,
  CONSTRAINT PK_lbr_labour PRIMARY KEY (car_license_plate, svc_booked_in, emp_id)
);



CREATE TABLE cim_chargeable_item
(
  car_license_plate VARCHAR(9) NOT NULL,
  svc_booked_in DATE NOT NULL,
  prt_num INT NOT NULL,
  cim_num_items INT NOT NULL,
  CONSTRAINT PK_cim_chargeable_item PRIMARY KEY (car_license_plate, svc_booked_in, prt_num)
);



CREATE TABLE prt_part
(
  prt_num INT NOT NULL,
  prt_description VARCHAR(30) NOT NULL,
  prt_cost LONGINT NOT NULL,
  CONSTRAINT PK_prt_part PRIMARY KEY (prt_num)
);



CREATE TABLE rsp_reg_svc_part
(
  mdl_name VARCHAR(20) NOT NULL,
  mdl_mileage INT NOT NULL,
  prt_num INT NOT NULL,
  CONSTRAINT PK_rsp_reg_svc_part PRIMARY KEY (mdl_name, mdl_mileage, prt_num)
);



ALTER TABLE cst_customer ADD CONSTRAINT cst_adr FOREIGN KEY (
  adr_id
)
REFERENCES adr_address (
  adr_id
);


ALTER TABLE car_car ADD CONSTRAINT clr_car FOREIGN KEY (
  clr_rgb
)
REFERENCES clr_colour (
  clr_rgb
);


ALTER TABLE car_car ADD CONSTRAINT car_mdl FOREIGN KEY (
  mdl_name
)
REFERENCES mdl_model (
  mdl_name
);


ALTER TABLE car_car ADD CONSTRAINT cst_car FOREIGN KEY (
  cst_id
)
REFERENCES cst_customer (
  cst_id
);


ALTER TABLE svc_service ADD CONSTRAINT car_svc FOREIGN KEY (
  car_license_plate
)
REFERENCES car_car (
  car_license_plate
);


ALTER TABLE svc_service ADD CONSTRAINT stp_svc FOREIGN KEY (
  stp_code
)
REFERENCES stp_service_type (
  stp_code
);


ALTER TABLE svc_service ADD CONSTRAINT svc_rgs FOREIGN KEY (
  mdl_name,
  mdl_mileage
)
REFERENCES rgs_regular_service (
  mdl_name,
  mdl_mileage
);


ALTER TABLE rgs_regular_service ADD CONSTRAINT mdl_rgs FOREIGN KEY (
  mdl_name
)
REFERENCES mdl_model (
  mdl_name
);


ALTER TABLE lbr_labour ADD CONSTRAINT svc_lbr FOREIGN KEY (
  car_license_plate,
  svc_booked_in
)
REFERENCES svc_service (
  car_license_plate,
  svc_booked_in
);


ALTER TABLE lbr_labour ADD CONSTRAINT emp_lbr FOREIGN KEY (
  emp_id
)
REFERENCES emp_employee (
  emp_id
);


ALTER TABLE cim_chargeable_item ADD CONSTRAINT svc_cim FOREIGN KEY (
  car_license_plate,
  svc_booked_in
)
REFERENCES svc_service (
  car_license_plate,
  svc_booked_in
);


ALTER TABLE cim_chargeable_item ADD CONSTRAINT prt_cim FOREIGN KEY (
  prt_num
)
REFERENCES prt_part (
  prt_num
);


ALTER TABLE rsp_reg_svc_part ADD CONSTRAINT rgs_rsp FOREIGN KEY (
  mdl_name,
  mdl_mileage
)
REFERENCES rgs_regular_service (
  mdl_name,
  mdl_mileage
);


ALTER TABLE rsp_reg_svc_part ADD CONSTRAINT prt_rsp FOREIGN KEY (
  prt_num
)
REFERENCES prt_part (
  prt_num
);


