package com.bettersoftwarefaster.carserv.app;

import java.util.*;
import com.bettersoftwarefaster.carserv.infra.*;
import javax.swing.*;
import java.awt.event.ActionEvent;

/**
 * Adds mnemonic property to AbstractAction.
 * Generally the application should use a subclass of AppActionCommand, which
 * invokes the action through the CommandManager.  However, this class exists
 * to support actions that are not to be routed through the
 * CommandManager, that is, do not require undo and redo.
 * <p>An alternative (and recommended approach) <i>is</i> to subclass
 * AppActionCommand but to return Command.ExecReturn.IGNORE.
 */
public abstract class AppAction extends AbstractAction {
    public AppAction(String name) {
        super(name);
    }

    public AppAction(String name, Icon icon) {
        super(name, icon);
    }

    public AppAction(String name, char mnemonic) {
        super(name);
        setMnemonic(mnemonic);
    }

    public AppAction(String name, Icon icon, char mnemonic) {
        super(name, icon);
        setMnemonic(mnemonic);
    }

    private void setMnemonic(char mnemonic) {
        this.putValue("mnemonic", new Character(mnemonic));
    }

    public char getMnemonic() {
        return ((Character)getValue("mnemonic")).charValue();
    }

    public String getName() { return ((String)getValue(Action.NAME)); }

    public CarServApp getApp() {
        return AppSingletonManager.getInstance().getApp();
    }

    public String toString() {
        return "AppAction: " + this.getValue(Action.NAME);
    }
}
