package com.bettersoftwarefaster.carserv.app;

import java.util.*;
import com.bettersoftwarefaster.carserv.infra.*;
import javax.swing.*;
import java.awt.event.ActionEvent;

/**
 * Async. equivalent of AppAction.
 * <p>Listener of own completion, as a convenience.
 */
public abstract class AppAsyncAction extends AppAction implements
ActionCompletionListener {
    public AppAsyncAction(String name) {
        this(name, (char)0);
    }

    public AppAsyncAction(String name, char mnemonic) {
        super(name);
        initialize(mnemonic);
    }

    public AppAsyncAction(String name, Icon icon) {
        this(name, icon, (char)0);
    }

    public AppAsyncAction(String name, Icon icon, char mnemonic) {
        super(name, icon);
        initialize(mnemonic);
    }

    private void initialize(char mnemonic) {
        if (mnemonic != (char)0) {
            setMnemonic(mnemonic);
        }
        addActionCompletionListener(this);
    }

    private void setMnemonic(char mnemonic) {
        this.putValue("mnemonic", new Character(mnemonic));
    }

    public char getMnemonic() {
        return ((Character)getValue("mnemonic")).charValue();
    }

    public String getName() { return ((String)getValue(Action.NAME)); }

    public CarServApp getApp() {
        return AppSingletonManager.getInstance().getApp();
    }

    public void addActionCompletionListener(ActionCompletionListener acl) {
        listeners.add(acl);
    }

    public void removeActionCompletionListener(ActionCompletionListener acl) {
        listeners.remove(acl);
    }

    public void actionPerformed(final ActionEvent ev) {
        if (isExecuting()) {
            return;
        }
        executing = true;
        new Thread() {
            public void run() {
                doActionPerformed(ev);
                for (Iterator iter = listeners.iterator(); iter.hasNext(); ) {
                    ((ActionCompletionListener)iter.next()).
                    actionCompleted(AppAsyncAction.this);
                }
            }
        }
        .start();
        executing = false;
    }

    /** Implementation of the actual action */
    protected abstract void doActionPerformed(ActionEvent ev);

    public boolean isExecuting() { return executing; }

    private boolean executing = false;

    public void actionCompleted(Action action) { }

    /**
     * @link aggregation
     *   @associates <{ActionCompletionListener}>
     * @clientCardinality 1
     * @supplierCardinality 0..*
     * @constraint {ordered}
     */
    private List listeners = new LinkedList();

    public String toString() {
        return "AppAsyncAction: " + this.getValue(Action.NAME);
    }
}
