package com.bettersoftwarefaster.carserv.app;

import java.util.*;
import java.awt.*;
import javax.swing.*;
import com.bettersoftwarefaster.carserv.domain.*;
import com.bettersoftwarefaster.carserv.infra.*;

/**
 * Represents the logical request to book in a car for its service.  The
 * necessary information is provided as an instance of BookInCommand.Arg.
 */
public abstract class BookInCommand extends AppActionCommand {
    public BookInCommand() {
        super("Book in");
    }

    public Command.ExecReturn doExecute(Object argAsObject)
    throws CommandException {
        Arg arg = (Arg)argAsObject;
        try {
            arg.getService().bookInCar(arg.getMileage());
        }
        catch (DomainException ex) {
            throw new CommandException("Unable to book in car.", ex);
        }
        return Command.ExecReturn.UNDOABLE;
    }

    public Command.UndoReturn doUndo(Object argAsObject)
    throws CommandException {
        Arg arg = (Arg)argAsObject;
        Service service = arg.getService();
        Car car;
        try {
            car = service.getCar();
            // first cancel the service booked in
            car.cancelService(service);
            // now re-schedule
            Service newService;
            if (service instanceof RegularService) {
                RegularService regularService = (RegularService)service;
                newService = car.scheduleRegularService(regularService.getBookedIn(),
                regularService.getServiceMileage());
            }
            else {
                newService = car.scheduleAdhocService(service.getBookedIn());
            }
            arg.setService(newService);
            return Command.UndoReturn.REDOABLE;
        }
        catch (DomainException ex) {
            throw new CommandException(ex);
        }
    }

    public static class Arg {
        public Arg(Service service, int mileage) {
            this.mileage = mileage;
            setService(service);
        }

        public int getMileage() { return mileage; }

        public void setService(Service service) { this.service = service; }

        public Service getService() { return service; }

        private int mileage;
        private Service service;
    }
}
