package com.bettersoftwarefaster.carserv.app;

import java.util.*;
import java.awt.*;
import javax.swing.*;
import com.bettersoftwarefaster.carserv.domain.*;
import com.bettersoftwarefaster.carserv.infra.*;
import com.bettersoftwarefaster.carserv.infra.Command;

/**
 * Represents the logical request to cancel a service.  The necessary
 * information is provided as an instance of CancelCommand.Arg.
 */
public abstract class CancelCommand extends AppActionCommand {
    public CancelCommand() {
        super("Cancel");
    }

    public Command.ExecReturn doExecute(Object argAsObject)
    throws CommandException {
        Arg arg = (Arg)argAsObject;
        Service service = arg.getService();
        try {
            Car car = service.getCar();
            car.cancelService(service);
        }
        catch (DomainException ex) {
            throw new CommandException("unable to cancel service", ex);
        }
        return Command.ExecReturn.UNDOABLE;
    }

    public Command.UndoReturn doUndo(Object argAsObject)
    throws CommandException {
        Arg arg = (Arg)argAsObject;
        Service service = arg.getService();
        Car car;
        try {
            car = service.getCar();
            // re-schedule
            Service newService;
            if (service instanceof RegularService) {
                RegularService regularService = (RegularService)service;
                newService = car.scheduleRegularService(regularService.getBookedIn(),
                regularService.getServiceMileage());
            }
            else {
                newService = car.scheduleAdhocService(service.getBookedIn());
            }
            arg.setService(newService);
            return Command.UndoReturn.REDOABLE;
        }
        catch (DomainException ex) {
            throw new CommandException(ex);
        }
    }

    public static class Arg {
        public Arg(Service service) {
            setService(service);
        }

        public void setService(Service service) { this.service = service; }

        public Service getService() { return service; }

        private Service service;
    }
}
