package com.bettersoftwarefaster.carserv.app;

import java.util.*;
import java.awt.*;
import javax.swing.*;
import com.bettersoftwarefaster.carserv.domain.*;
import com.bettersoftwarefaster.carserv.infra.*;
import com.bettersoftwarefaster.carserv.infra.Command;

/**
 * Represents the logical request to schedule an adhoc service for a car.  The
 * necessary information is provided as an instance of
 * ScheduleAdhocServiceCommand.Arg.
 */
public abstract class ScheduleAdhocServiceCommand extends AppActionCommand {
    public ScheduleAdhocServiceCommand(char scheduleServiceMnemonic) {
        super("Schedule", scheduleServiceMnemonic);
    }

    public Command.ExecReturn doExecute(Object argAsObject)
    throws CommandException {
        Arg arg = (Arg)argAsObject;
        Car car = arg.getCar();
        try {
            Service newService = car.scheduleAdhocService(arg.getBookedIn());
            arg.setService(newService);
        }
        catch (DomainException ex) {
            throw new CommandException("Unable to schedule adhoc service", ex);
        }
        return Command.ExecReturn.UNDOABLE;
    }

    public Command.UndoReturn doUndo(Object argAsObject)
    throws CommandException {
        Arg arg = (Arg)argAsObject;
        Car car = arg.getCar();
        Service service = arg.getService();
        try {
            car.cancelService(service);
            return Command.UndoReturn.REDOABLE;
        }
        catch (DomainException ex) {
            throw new CommandException(ex);
        }
    }

    public static class Arg {
        public Arg(Car car, java.sql.Date bookedIn) {
            this.car = car;
            this.bookedIn = bookedIn;
        }

        public Car getCar() { return car; }

        public java.sql.Date getBookedIn() { return bookedIn; }

        public void setService(Service service) { this.service = service; }

        public Service getService() { return service; }

        private Car car;
        private java.sql.Date bookedIn;
        private Service service;
    }
}
