package com.bettersoftwarefaster.carserv.app;

import java.util.*;
import java.awt.*;
import javax.swing.*;
import com.bettersoftwarefaster.carserv.domain.*;
import com.bettersoftwarefaster.carserv.infra.*;
import com.bettersoftwarefaster.carserv.infra.Command;

/**
 * Represents the logical request to schedule an regular service for a car.
 *  The necessary information is provided as an instance of
 * ScheduleRegularServiceCommand.Arg.
 */
public abstract class ScheduleRegularServiceCommand extends AppActionCommand {
    public ScheduleRegularServiceCommand(char scheduleServiceMnemonic) {
        super("Schedule", scheduleServiceMnemonic);
    }

    public Command.ExecReturn doExecute(Object argAsObject)
    throws CommandException {
        Arg arg = (Arg)argAsObject;
        Car car = arg.getCar();
        try {
            Service newService = car.scheduleRegularService(arg.getBookedIn(),
            arg.getServiceMileage());
            arg.setService(newService);
        }
        catch (DomainException ex) {
            throw new CommandException("Unable to schedule regular service", ex);
        }
        return Command.ExecReturn.UNDOABLE;
    }

    public Command.UndoReturn doUndo(Object argAsObject)
    throws CommandException {
        Arg arg = (Arg)argAsObject;
        Car car = arg.getCar();
        Service service = arg.getService();
        try {
            car.cancelService(service);
            return Command.UndoReturn.REDOABLE;
        }
        catch (DomainException ex) {
            throw new CommandException(ex);
        }
    }

    public static class Arg {
        public Arg(Car car, java.sql.Date bookedIn, int serviceMileage) {
            this.car = car;
            this.bookedIn = bookedIn;
            this.serviceMileage = serviceMileage;
        }

        public Car getCar() { return car; }

        public java.sql.Date getBookedIn() { return bookedIn; }

        public int getServiceMileage() { return serviceMileage; }

        public void setService(Service service) { this.service = service; }

        public Service getService() { return service; }

        private Car car;
        private java.sql.Date bookedIn;
        private int serviceMileage;
        private Service service;

        public String toString() {
            return "car = " + car.getLicensePlate() + ", bookedIn = " +
            getBookedIn() + ", service mileage = " + getServiceMileage();
        }
    }
}
