package com.bettersoftwarefaster.carserv.app;

import java.util.*;
import java.awt.event.*;
import com.bettersoftwarefaster.carserv.infra.*;

/**
 * Represents the logical request for an employee to sign on.  The necessary
 * information is provided as an instance of SignOnCommand.Arg.
 */
public abstract class SignOnCommand extends AppActionAsyncCommand {
    public SignOnCommand() {
        super("OK", Command.ExecReturn.UNDOABLE);
        this.addCommandCompletionListener(this);
    }

    /**
     * Note that the return status is ignored by the superclass.  The actual
     * return code has been specified in the call to the super() in
     * the constructor.
     */
    protected Command.ExecReturn doExecute(Object argAsObject)
    throws CommandException {
        Arg arg = (Arg)argAsObject;
        try {
            getApp().signOn(arg.getLastName(), arg.getFirstName());
        }
        catch (AppException ex) {
            throw new CommandException(ex);
        }
        return Command.ExecReturn.UNDOABLE;
    }

    public Command.UndoReturn doUndo(Object arg) {
        getApp().logout();

        /**
         * orig requirement (can redo undone signon).
         * return Command.UndoReturn.REDOABLE;
         */

        /**
         * new requirement (redo not allowed for undone signon). //
         * return Command.UndoReturn.NOT_REDOABLE;
         */
        return Command.UndoReturn.REDOABLE;
    }

    public static class Arg {
        public Arg(String lastName, String firstName) {
            this.lastName = lastName;
            this.firstName = firstName;
        }

        public String getLastName() { return lastName; }

        public String getFirstName() { return firstName; }

        private String lastName;
        private String firstName;
    }
}
