package com.bettersoftwarefaster.carserv.datamgmt;

import java.util.*;

/**
 * Immutable value object to represent an Address.  The house name and number,
 * street and postal codes are mandatory.
 * <p>In the future, could link up to central postal office to automatically
 * look up street, city and province from just the postal code and number?
 * @stereotype valueObject
 */
public class AddressData extends ValueObject {
    public final static long serialVersionUID = 1;

    /** Can only be instantiated from datamgmt package */
    AddressData(int id, String houseNameNumber, String street,
    String postalCode, String city, String province) {
        this.id = id;
        this.houseNameNumber = houseNameNumber;
        this.street = street;
        this.postalCode = postalCode;
        this.city = city;
        this.province = province;
    }

    public int getId() { return id; }

    public String getHouseNameNumber() { return houseNameNumber; }

    public String getStreet() { return street; }

    public String getPostalCode() { return postalCode; }

    public String getCity() { return city; }

    public String getProvince() { return province; }

    public boolean equals(Object other) {
        if (!(other instanceof AddressData))
            return false;
        AddressData otherAddress = (AddressData)other;
        return (equalsPK(otherAddress) &&
        compareStrings(houseNameNumber, otherAddress.houseNameNumber) &&
        compareStrings(street, otherAddress.street) &&
        compareStrings(postalCode, otherAddress.postalCode) &&
        compareStrings(city, otherAddress.city) &&
        compareStrings(province, otherAddress.province));
    }

    public boolean equalsPK(AddressData otherAddress) {
        return (otherAddress == null) ? false : (id == otherAddress.id);
    }

    public int hashCode() {
        return id;
    }

    public String toString() {
        return getClass().getName() + ": " +
        valueOf("houseNameNumber", getHouseNameNumber()) +
        valueOf("street         ", getStreet()) +
        valueOf("postalCode     ", getPostalCode()) +
        valueOf("city           ", getCity()) +
        valueOf("province       ", getProvince());
    }

    private final int id;
    private final String houseNameNumber;
    private final String street;
    private final String postalCode;
    private final String city;
    private final String province;
}
