package com.bettersoftwarefaster.carserv.datamgmt;

import java.sql.*;
import java.util.*;
import java.awt.Color;
import test.com.bettersoftwarefaster.carserv.TestHelper;

/**
 * Car Data Manager.  Look up Car from a license plate, and schedule Services
 * of a Car (schedule, cancel etc.)
 */
public class CarDM extends DataManager {
    public CarDM() {
        this.serviceDataHelper = new ServiceDataHelper();
    }

    public CarData getCar(String licensePlate) throws DataMgmtException {
        CarData car = null;
        int rowcount = 0;
        try {
            getGetCarPS().clearParameters();
            getGetCarPS().setString(1, licensePlate);
            ResultSet resultSet = getGetCarPS().executeQuery();
            while (resultSet.next()) {
                int red = resultSet.getInt(2);
                int green = resultSet.getInt(3);
                int blue = resultSet.getInt(4);
                car = new CarData(resultSet.getString(1),
                new Color(red, green, blue), resultSet.getString(5),
                resultSet.getString(6), resultSet.getInt(7));
                rowcount++;
            }
            resultSet.close();
        }
        catch (SQLException ex) {
            throw new DataMgmtException("Failed to get Car", ex);
        }
        switch (rowcount) {
            case 0:
            case 1:
                return car;
            default:
                throw new DataMgmtException("More than one car with same license plate found.");
        }
    }

    public RegularServiceData[] listRegularServices(CarData car)
    throws DataMgmtException {
        List regularServices = new ArrayList();
        List parts = new ArrayList();
        RegularServiceData eachRegularService;
        RegularServiceData lastRegularService = null;
        ResultSet resultSet = null;
        try {
            getListRegularServicesPS().clearParameters();
            getListRegularServicesPS().setString(1, car.getModel());
            resultSet = getListRegularServicesPS().executeQuery();
            while (resultSet.next()) {
                eachRegularService = new
                RegularServiceData(car.getModel(), resultSet.getInt(1), null);
                if (lastRegularService == null) {
                    lastRegularService = eachRegularService;
                }
                if (!(eachRegularService.equalsPK(lastRegularService))) {
                    regularServices.add(updateParts(lastRegularService,
                    parts));
                    parts = new ArrayList();
                    lastRegularService = eachRegularService;
                }
                parts.add(
                new PartData(resultSet.getInt(2), resultSet.getString(3),
                resultSet.getLong(4)));
            }
            regularServices.add(updateParts(lastRegularService, parts));
            resultSet.close();
        }
        catch (SQLException ex) {
            throw new DataMgmtException("Failed to listRegularServices", ex);
        }
        return (RegularServiceData[]) regularServices.toArray(
        new RegularServiceData[] { });
    }

    private RegularServiceData
    updateParts(RegularServiceData regularService, List parts) {
        return new RegularServiceData(regularService.getModel(),
        regularService.getMileage(), parts);
    }

    public ServiceData[] listServices(CarData car) throws DataMgmtException {
        List services = new ArrayList();
        ResultSet resultSet = null;
        try {
            getListServicesPS().clearParameters();
            getListServicesPS().setString(1, car.getLicensePlate());
            resultSet = getListServicesPS().executeQuery();
            while (resultSet.next()) {
                services.add(serviceDataHelper.fetchServiceData(resultSet));
            }
            resultSet.close();
        }
        catch (SQLException ex) {
            throw new DataMgmtException("Failed to list Services", ex);
        }
        return (ServiceData[]) services.toArray(
        new ServiceData[] { });
    }

    public ServiceData scheduleRegularService(CarData car,
    java.sql.Date bookedIn, int serviceMileage) throws DataMgmtException {
        return scheduleService(car, bookedIn, "RGLR", serviceMileage);
    }

    public ServiceData scheduleAdhocService(CarData car, java.sql.Date bookedIn)
    throws DataMgmtException {
        return scheduleService(car, bookedIn, "ADHC", 0);
    }

    private ServiceData scheduleService(CarData car, java.sql.Date bookedIn,
    String serviceTypeCode, int serviceMileage) throws DataMgmtException {
        int numRowsUpdated;
        try {
            getScheduleServicePS().clearParameters();
            getScheduleServicePS().setString(1, car.getLicensePlate());
            getScheduleServicePS().setDate(2, bookedIn);
            if (serviceTypeCode.equals("RGLR")) {
                getScheduleServicePS().setString(3, "RGLR");
                getScheduleServicePS().setString(4, car.getModel());
                getScheduleServicePS().setInt(5, serviceMileage);
            }
            else {
                getScheduleServicePS().setString(3, "ADHC");
                getScheduleServicePS().setNull(4, Types.VARCHAR);
                getScheduleServicePS().setNull(5, Types.INTEGER);
            }
            beginTransaction();
            numRowsUpdated = getScheduleServicePS().executeUpdate();
        }
        catch (Exception ex) {
            rollbackTransaction();
            throw new DataMgmtException("Failed to schedule service", ex);
        }
        if (numRowsUpdated != 1) {
            rollbackTransaction();
            throw new DataMgmtException("Failed to schedule service, " +
            numRowsUpdated + " rows updated.");
        }
        else {
            commitTransaction();
        }
        return new ServiceData(car.getLicensePlate(),
        bookedIn, serviceMileage, 0, null, 0, 0, null, null);
    }

    public void cancelService(ServiceData service) throws DataMgmtException {
        int numRowsUpdated;
        try {
            getCancelServicePS().clearParameters();
            getCancelServicePS().setString(1, service.getLicensePlate());
            getCancelServicePS().setDate(2, service.getBookedIn());
            beginTransaction();
            numRowsUpdated = getCancelServicePS().executeUpdate();
        }
        catch (Exception ex) {
            rollbackTransaction();
            throw new DataMgmtException("Failed to cancel service", ex);
        }
        if (numRowsUpdated > 1) {
            rollbackTransaction();
            throw new DataMgmtException("Failed to cancel service, " +
            numRowsUpdated + " rows updated.");
        }
        else {
            commitTransaction();
        }
        return;
    }

/*
	public CarData addCar(String licensePlate, Color colour, String model,
		CustomerData owner)
						throws DataMgmtException {
		throw new DataMgmtException("Feature not yet implemented.");
	}

	public CarData updateCar(CarData car, Color colour)
						throws DataMgmtException {
		throw new DataMgmtException("Feature not yet implemented.");
	}

	public CarData changeCarOwner(CarData car, int ownerId)
						throws DataMgmtException {
		throw new DataMgmtException("Feature not yet implemented.");
	}

	public CarData removeCar(CarData car) throws DataMgmtException {
		throw new DataMgmtException("Feature not yet implemented.");
	}
*/

    protected void hookSessionAvailable() { }

    protected void hookSessionUnavailable() {
        getCarPS = null;
        listRegularServicesPS = null;
        listServicesPS = null;
        scheduleServicePS = null;
        cancelServicePS = null;
    }

    private PreparedStatement getGetCarPS() throws DataMgmtException {
        if (getCarPS == null) {
            StringBuffer buf = new StringBuffer();
            buf.append("\n SELECT car.car_license_plate,");
            buf.append("\n        clr.clr_red, clr.clr_green, clr.clr_blue,");
            buf.append("\n        car.mdl_name, mdl.mdl_manufacturer, car.cst_id");
            buf.append("\n FROM   car_car car");
            buf.append("\n   JOIN mdl_model mdl");
            buf.append("\n   ON   car.mdl_name = mdl.mdl_name");
            buf.append("\n   JOIN clr_colour clr");
            buf.append("\n   ON   car.clr_rgb = clr.clr_rgb");
            buf.append("\n WHERE  car.car_license_plate = ?");
            getCarPS = getPreparedStatement(buf.toString());
        }
        return getCarPS;
    }

    private PreparedStatement getListRegularServicesPS()
    throws DataMgmtException {
        if (listRegularServicesPS == null) {
            StringBuffer buf = new StringBuffer();
            buf.append("\n SELECT rsp.mdl_mileage,");
            buf.append("\n        rsp.prt_num, prt.prt_description, prt.prt_cost");
            buf.append("\n FROM   rsp_reg_svc_part rsp");
            buf.append("\n   JOIN prt_part prt");
            buf.append("\n     ON rsp.prt_num = prt.prt_num");
            buf.append("\n WHERE  rsp.mdl_name = ?");
            buf.append("\n ORDER BY rsp.mdl_mileage, rsp.prt_num");
            listRegularServicesPS = getPreparedStatement(buf.toString());
        }
        return listRegularServicesPS;
    }

    private PreparedStatement getListServicesPS() throws DataMgmtException {
        if (listServicesPS == null) {
            StringBuffer buf = new StringBuffer();
            buf.append("\n SELECT svc.car_license_plate, svc.svc_booked_in,");
            buf.append("\n        svc.mdl_mileage, svc.svc_mileage, svc.svc_picked_up,");
            buf.append("\n        svc_lbr_adjustments, svc_cim_adjustments");
            buf.append("\n FROM   svc_service svc");
            buf.append("\n WHERE  svc.car_license_plate = ?");
            buf.append("\n ORDER BY svc.svc_booked_in DESC");
            listServicesPS = getPreparedStatement(buf.toString());
        }
        return listServicesPS;
    }

    private PreparedStatement getScheduleServicePS() throws DataMgmtException {
        if (scheduleServicePS == null) {
            StringBuffer buf = new StringBuffer();
            buf.append("\n INSERT INTO svc_service(");
            buf.append("\n        car_license_plate, svc_booked_in, stp_code,");
            buf.append("\n        mdl_name, mdl_mileage                     )");
            buf.append("\n VALUES(?, ?, ?, ?, ?)");
            scheduleServicePS = getPreparedStatement(buf.toString());
        }
        return scheduleServicePS;
    }

    private PreparedStatement getCancelServicePS() throws DataMgmtException {
        if (cancelServicePS == null) {
            StringBuffer buf = new StringBuffer();
            buf.append("\n DELETE FROM svc_service");
            buf.append("\n WHERE  car_license_plate = ?");
            buf.append("\n AND    svc_booked_in = ?");
            cancelServicePS = getPreparedStatement(buf.toString());
        }
        return cancelServicePS;
    }

    private PreparedStatement getCarPS;
    private PreparedStatement listRegularServicesPS;
    private PreparedStatement listServicesPS;
    private PreparedStatement scheduleServicePS;
    private PreparedStatement cancelServicePS;

    /**
     * @clientCardinality 1
     * @supplierCardinality 1
     * @link aggregationByValue
     */
    private final ServiceDataHelper serviceDataHelper;
}
