package com.bettersoftwarefaster.carserv.datamgmt;

import java.awt.Color;
import java.util.*;

/**
 * Immutable value object to represent a Car.
 * @stereotype valueObject
 */
public class CarData extends ValueObject {
    public final static long serialVersionUID = 1;

    /** Can only be instantiated by the datamgmt package */
    CarData(String licensePlate, Color colour, String model,
    String manufacturer, int ownerId) {
        this.licensePlate = licensePlate;
        this.colour = colour;
        this.model = model;
        this.manufacturer = manufacturer;
        this.ownerId = ownerId;
    }

    public String getLicensePlate() { return licensePlate; }

    public Color getColour() { return colour; }

    public String getModel() { return model; }

    public String getManufacturer() { return manufacturer; }

    public int getOwnerId() { return ownerId; }

    /** @returns Collection of RegularServiceData elements */
    public Collection getRegularServices() {
        return Collections.unmodifiableCollection(regularServices.values());
    }

    public RegularServiceData getRegularService(int mileage) {
        return (RegularServiceData)regularServices.get(new Integer(mileage));
    }

    public boolean equals(Object other) {
        if (!(other instanceof CarData))
            return false;
        CarData otherCar = (CarData)other;
        return (equalsPK(otherCar) && colour.equals(otherCar.colour) &&
        compareStrings(model, otherCar.model) &&
        compareStrings(manufacturer, otherCar.manufacturer) &&
        ownerId == otherCar.ownerId);
    }

    public boolean equalsPK(CarData otherCar) {
        return (otherCar == null) ? false :
        compareStrings(licensePlate, otherCar.licensePlate);
    }

    public int hashCode() {
        return licensePlate.hashCode();
    }

    public String toString() {
        return getClass().getName() + ": " +
        valueOf("licensePlate ", getLicensePlate()) +
        valueOf("colour       ", getColour()) +
        valueOf("model        ", getModel()) +
        valueOf("manufacturer ", getManufacturer()) +
        valueOf("owner's Id   ", getOwnerId());
    }

    private final String licensePlate;
    private final Color colour;
    private final String model;
    private final String manufacturer;
    private final int ownerId;

    /**
     * @link aggregationByValue
     *   @associates <{com.bettersoftwarefaster.carserv.datamgmt.RegularServiceData}>
     *   @clientQualifier mileage
     * @clientCardinality 1
     * @supplierCardinality 0..*
     * @supplierRole regularServices
     * @constraint {set}
     */
    private Map regularServices = new HashMap();
}
