package com.bettersoftwarefaster.carserv.datamgmt;

import java.util.*;
import java.sql.*;

/**
 * Database connection factory.  Supports local (embedded) or remote
 * (client/server) connections to a cloudscape database.
 * @stereotype factory
 */
class ConnectionFactory {
    public final static String JDBC_DRIVER_REMOTE =
    "COM.cloudscape.core.RmiJdbcDriver";
    public final static String JDBC_URL_REMOTE =
    "jdbc:cloudscape:rmi://localhost:1099/CarServDB";
    public final static String JDBC_DRIVER_LOCAL =
    "COM.cloudscape.core.JDBCDriver";
    public final static String JDBC_URL_LOCAL =
    "jdbc:cloudscape:c:/cloudscape_3.6/CarServDB";

    ConnectionFactory(String jdbcDriver, String jdbcUrl) {
        this.jdbcDriver = jdbcDriver;
        this.jdbcUrl = jdbcUrl;
    }

    private final String jdbcDriver;
    private final String jdbcUrl;

    static ConnectionFactory getConnectionFactory(String localOrRemote) {
        if (localOrRemote.equals("local")) {
            return new ConnectionFactory(JDBC_DRIVER_LOCAL, JDBC_URL_LOCAL);
        }
        else {
            return new ConnectionFactory(JDBC_DRIVER_REMOTE, JDBC_URL_REMOTE);
        }
    }

    Connection getConnection() throws DataMgmtException {
        Connection connection;
        try {
            Class.forName(this.jdbcDriver);
            connection = DriverManager.getConnection(this.jdbcUrl);
            connection.setAutoCommit(true);
            connection.setTransactionIsolation(Connection.TRANSACTION_READ_COMMITTED);
        }
        catch (ClassNotFoundException ex) {
            throw new DataMgmtException("Unable to load JDBC driver", ex);
        }
        catch (SQLException ex) {
            throw new DataMgmtException("Problem obtaining / configuring connection.", ex);
        }
        return connection;
    }
}
