package com.bettersoftwarefaster.carserv.datamgmt;

import java.awt.Color;
import java.sql.*;
import java.util.*;

/**
 * Customer data manager.  Locate customers by name, address etc, and find the
 * cars that they own.
 */
public class CustDM extends DataManager {
    public CustDM() {
    }

    public CustomerData[] findCustomers(String lastName, String firstName,
    String addressStreet, String addressPostalCode) throws DataMgmtException {
        List customers = new ArrayList();
        ResultSet resultSet = null;
        try {
            getFindCustomersPS().clearParameters();
            getFindCustomersPS().setString(1, wildCard(lastName));
            getFindCustomersPS().setString(2, wildCard(firstName));
            getFindCustomersPS().setString(3, wildCard2(addressStreet));
            getFindCustomersPS().setString(4, wildCard(addressPostalCode));
            resultSet = getFindCustomersPS().executeQuery();
            while (resultSet.next()) {
                AddressData address = new
                AddressData(resultSet.getInt(4), resultSet.getString(5),
                resultSet.getString(6), resultSet.getString(7), resultSet.getString(8),
                resultSet.getString(9));
                customers.add(
                new CustomerData(resultSet.getInt(1), resultSet.getString(2),
                resultSet.getString(3), address));
            }
            resultSet.close();
        }
        catch (SQLException ex) {
            throw new DataMgmtException("Failed to findCustomers", ex);
        }
        return (CustomerData[]) customers.toArray(
        new CustomerData[] { });
    }

    public CustomerData findCustomer(int id) throws DataMgmtException {
        CustomerData customer = null;
        ResultSet resultSet = null;
        try {
            getFindCustomerPS().clearParameters();
            getFindCustomerPS().setInt(1, id);
            resultSet = getFindCustomerPS().executeQuery();
            while (resultSet.next()) {
                AddressData address = new
                AddressData(resultSet.getInt(4), resultSet.getString(5),
                resultSet.getString(6), resultSet.getString(7), resultSet.getString(8),
                resultSet.getString(9));
                customer = new CustomerData(resultSet.getInt(1), resultSet.getString(2),
                resultSet.getString(3), address);
            }
            resultSet.close();
        }
        catch (SQLException ex) {
            throw new DataMgmtException("Failed to findCustomer", ex);
        }
        return customer;
    }

    public CarData[] listCars(CustomerData owner) throws DataMgmtException {
        List cars = new ArrayList();
        ResultSet resultSet = null;
        try {
            getListCarsPS().clearParameters();
            getListCarsPS().setInt(1, owner.getId());
            resultSet = getListCarsPS().executeQuery();
            while (resultSet.next()) {
                int red = resultSet.getInt(2);
                int green = resultSet.getInt(3);
                int blue = resultSet.getInt(4);
                cars.add(
                new CarData(resultSet.getString(1),
                new Color(red, green, blue), resultSet.getString(5),
                resultSet.getString(6), owner.getId()));
            }
            resultSet.close();
        }
        catch (SQLException ex) {
            throw new DataMgmtException("Failed to listCars", ex);
        }
        return (CarData[]) cars.toArray(
        new CarData[] { });
    }

    public CustomerData addCustomer(String lastName, String firstName,
    String houseNameNumber, String street, String postalCode, String city,
    String province) throws DataMgmtException {
        throw new DataMgmtException("Feature not yet implemented.");
    }

    public CustomerData updateCustomer(CustomerData customer,
    String newLastName, String newFirstName) throws DataMgmtException {
        int numRowsUpdated;
        try {
            getUpdateCustomerPS().clearParameters();
            getUpdateCustomerPS().setString(1, newLastName);
            getUpdateCustomerPS().setString(2, newFirstName);
            getUpdateCustomerPS().setInt(3, customer.getId());
            beginTransaction();
            numRowsUpdated = getUpdateCustomerPS().executeUpdate();
        }
        catch (Exception ex) {
            rollbackTransaction();
            throw new DataMgmtException("Failed to updateCustomer", ex);
        }
        if (numRowsUpdated != 1) {
            rollbackTransaction();
            throw new DataMgmtException("Failed to updateCustomer, " +
            numRowsUpdated + " rows updated.");
        }
        else {
            commitTransaction();
        }
        return new CustomerData(customer.getId(),
        newLastName, newFirstName, customer.getAddress());
    }

    /**
     * Will locate or create an address for the customer, and associate the
     * customer with this address.
     */

/*
	public CustomerData changeAddress(CustomerData customer,
		String houseNameNumber, String street, String postalCode,
		String city, String province)
						throws DataMgmtException {
		throw new DataMgmtException("Feature not yet implemented.");
	}

	public void removeCustomer(CustomerData customer)
						throws DataMgmtException {
		throw new DataMgmtException("Feature not yet implemented.");
	}
*/

    protected void hookSessionAvailable() { }

    protected void hookSessionUnavailable() {
        findCustomersPS = null;
        findCustomerPS = null;
        updateCustomerPS = null;
        listCarsPS = null;
    }

    private PreparedStatement getFindCustomersPS() throws DataMgmtException {
        if (findCustomersPS == null) {
            StringBuffer buf = new StringBuffer();
            buf.append("\n SELECT cst.cst_id, cst.cst_last_name, cst.cst_first_name,");
            buf.append("\n        cst.adr_id,");
            buf.append("\n        adr.adr_house_name_num, adr.adr_street,");
            buf.append("\n        adr.adr_postal_code, adr.adr_city, adr.adr_province");
            buf.append("\n FROM   cst_customer cst");
            buf.append("\n   JOIN adr_address adr");
            buf.append("\n   ON   cst.adr_id = adr.adr_id");
            buf.append("\n WHERE  cst.cst_last_name LIKE ?");
            buf.append("\n AND    cst.cst_first_name LIKE ?");
            buf.append("\n AND    adr.adr_street LIKE ?");
            buf.append("\n AND    adr.adr_postal_code LIKE ?");
            buf.append("\n ORDER BY cst.cst_last_name, cst.cst_first_name");
            findCustomersPS = getPreparedStatement(buf.toString());
        }
        return findCustomersPS;
    }

    private PreparedStatement getFindCustomerPS() throws DataMgmtException {
        if (findCustomerPS == null) {
            StringBuffer buf = new StringBuffer();
            buf.append("\n SELECT cst.cst_id, cst.cst_last_name, cst.cst_first_name,");
            buf.append("\n        cst.adr_id,");
            buf.append("\n        adr.adr_house_name_num, adr.adr_street,");
            buf.append("\n        adr.adr_postal_code, adr.adr_city, adr.adr_province");
            buf.append("\n FROM   cst_customer cst");
            buf.append("\n   JOIN adr_address adr");
            buf.append("\n   ON   cst.adr_id = adr.adr_id");
            buf.append("\n WHERE  cst.cst_id = ?");
            findCustomerPS = getPreparedStatement(buf.toString());
        }
        return findCustomerPS;
    }

    private PreparedStatement getUpdateCustomerPS() throws DataMgmtException {
        if (updateCustomerPS == null) {
            StringBuffer buf = new StringBuffer();
            buf.append("\n UPDATE cst_customer");
            buf.append("\n SET    cst_last_name = ?");
            buf.append("\n       ,cst_first_name = ?");
            buf.append("\n WHERE  cst_id = ?");
            updateCustomerPS = getPreparedStatement(buf.toString());
        }
        return updateCustomerPS;
    }

    private PreparedStatement getListCarsPS() throws DataMgmtException {
        if (listCarsPS == null) {
            StringBuffer buf = new StringBuffer();
            buf.append("\n SELECT car.car_license_plate,");
            buf.append("\n        clr.clr_red, clr.clr_green, clr.clr_blue,");
            buf.append("\n        car.mdl_name, mdl.mdl_manufacturer");
            buf.append("\n FROM   car_car car");
            buf.append("\n   JOIN mdl_model mdl");
            buf.append("\n   ON   car.mdl_name = mdl.mdl_name");
            buf.append("\n   JOIN clr_colour clr");
            buf.append("\n   ON   car.clr_rgb = clr.clr_rgb");
            buf.append("\n WHERE  car.cst_id = ?");
            listCarsPS = getPreparedStatement(buf.toString());
        }
        return listCarsPS;
    }

    private PreparedStatement findCustomersPS;
    private PreparedStatement findCustomerPS;
    private PreparedStatement updateCustomerPS;
    private PreparedStatement listCarsPS;
}
