package com.bettersoftwarefaster.carserv.datamgmt;

import java.util.*;

/**
 * Immutable value object to represent a Customer.
 * @stereotype valueObject
 */
public class CustomerData extends ValueObject {
    public final static long serialVersionUID = 1;

    /** Can only be instantiated by the datamgmt package */
    CustomerData(int id, String lastName, String firstName,
    AddressData address) {
        this.id = id;
        this.name = new NameData(lastName, firstName);
        this.address = address;
    }

    public int getId() { return id; }

    public String getFirstName() { return name.getFirstName(); }

    public String getLastName() { return name.getLastName(); }

    public AddressData getAddress() { return address; }

    public boolean equals(Object other) {
        if (!(other instanceof CustomerData))
            return false;
        CustomerData otherCustomer = (CustomerData)other;
        return (equalsPK(otherCustomer) && name.equals(otherCustomer.name) &&
        address.equals(otherCustomer.address));
    }

    public boolean equalsPK(CustomerData otherCustomer) {
        return (otherCustomer == null) ? false : (id == otherCustomer.id);
    }

    public int hashCode() {
        return id;
    }

    public String toString() {
        return getClass().getName() + ": " + valueOf("id       ", getId()) +
        valueOf("lastName ", getLastName()) +
        valueOf("firstName", getLastName()) +
        valueOf("Embedded address:\n", getAddress());
    }

    private final int id;
    private final NameData name;
    private final AddressData address;
}
