package com.bettersoftwarefaster.carserv.datamgmt;

import java.util.*;

/**
 * Concrete implementation of DatabaseSessionProvider.  Designed
 * as a superclass.
 */
public class DatabaseSessionProviderAdapter implements
DatabaseSessionProvider {
    public DatabaseSessionProviderAdapter() {
        this.listeners = new ArrayList();
    }

    public void addListener(DatabaseSessionListener listener) {
        listeners.add(listener);
    }

    public void removeListener(DatabaseSessionListener listener) {
        listeners.remove(listener);
    }

    public void fireSessionAvailable(DatabaseSession session) {
        DatabaseSessionEvent ev = new DatabaseSessionEvent(this, session);
        for (Iterator iter = listeners.iterator(); iter.hasNext(); ) {
            DatabaseSessionListener listener =
            (DatabaseSessionListener)iter.next();
            listener.sessionAvailable(ev);
        }
        this.currentDatabaseSession = session;
    }

    public void fireSessionUnavailable(DatabaseSession session) {
        DatabaseSessionEvent ev = new DatabaseSessionEvent(this, session);
        for (Iterator iter = listeners.iterator(); iter.hasNext(); ) {
            ((DatabaseSessionListener)iter.next()).sessionUnavailable(ev);
        }
        this.currentDatabaseSession = null;
    }

    public boolean isSessionAvailable() {
        return getCurrentDatabaseSession() != null;
    }

    public DatabaseSession getCurrentDatabaseSession() {
        return currentDatabaseSession;
    }

    /**
     * @supplierRole current
     * @supplierCardinality 0..1
     * @label creates
     */
    private DatabaseSession currentDatabaseSession;

    /**
     * @link aggregation
     *   @associates <{com.bettersoftwarefaster.carserv.datamgmt.DatabaseSessionListener}>
     * @clientCardinality 1
     * @supplierCardinality 0..*
     * @supplierRole listeners
     * @constraint {ordered}
     */
    private final List listeners;
}
