package com.bettersoftwarefaster.carserv.datamgmt;

import java.util.*;

/**
 * Immutable value object to represent an Employee.
 * @stereotype valueObject
 */
public class EmployeeData extends ValueObject {
    public final static long serialVersionUID = 1;

    EmployeeData(int id, String lastName, String firstName,
    long mechanicLabourRate) {
        this.id = id;
        this.mechanicLabourRate = mechanicLabourRate;
        this.name = new NameData(lastName, firstName);
    }

    public int getId() { return id; }

    public String getFirstName() { return name.getFirstName(); }

    public String getLastName() { return name.getLastName(); }

    public long getMechanicLabourRate() { return mechanicLabourRate; }

    public boolean equals(Object other) {
        if (!(other instanceof EmployeeData))
            return false;
        EmployeeData otherEmployee = (EmployeeData)other;
        return equalsPK(otherEmployee) && name.equals(otherEmployee.name) &&
        mechanicLabourRate == otherEmployee.mechanicLabourRate;
    }

    public boolean equalsPK(EmployeeData otherEmployee) {
        return (otherEmployee == null) ? false : (id == otherEmployee.id);
    }

    public int hashCode() {
        return id;
    }

    public String toString() {
        return getClass().getName() + ": " +
        valueOf("id                ", getId()) +
        valueOf("lastName          ", getLastName()) +
        valueOf("firstName         ", getFirstName()) +
        valueOf("mechanicLabourRate", getMechanicLabourRate());
    }

    private final int id;
    private final NameData name;
    private final long mechanicLabourRate;
}
