package com.bettersoftwarefaster.carserv.datamgmt;

import java.util.*;

/**
 * Immutable value object to represent a Part used to repair a car (eg
 * windscreen wiper).
 */
public class PartData extends ValueObject {
    public final static long serialVersionUID = 1;

    /** can only be instantiated by datamgmt */
    PartData(int number, String description, long cost) {
        this.number = number;
        this.description = description;
        this.cost = cost;
    }

    public int getNumber() { return number; }

    public String getDescription() { return description; }

    public long getCost() { return cost; }

    public boolean equals(Object other) {
        if (!(other instanceof PartData))
            return false;
        PartData otherPart = (PartData)other;
        return (equalsPK(otherPart) &&
        compareStrings(description, otherPart.description) &&
        cost == otherPart.cost);
    }

    public boolean equalsPK(PartData otherPart) {
        return (otherPart == null) ? false : number == otherPart.number;
    }

    public int hashCode() {
        return number;
    }

    public String toString() {
        return getClass().getName() + ": " +
        valueOf("number     ", getNumber()) +
        valueOf("description", getDescription()) +
        valueOf("cost       ", getCost());
    }

    private int number;
    private String description;
    private long cost;
}
