package com.bettersoftwarefaster.carserv.datamgmt;

import java.util.*;

/**
 * Abstract class for value objects.  Provide methods to assist in
 * implementation of equals(), hashCode() and toString().
 */
public abstract class ValueObject implements java.io.Serializable {
    protected boolean compareStrings(String firstString, String secondString) {
        if ((firstString == null) || (secondString == null))
            return (firstString == null) && (secondString == null);
        return firstString.equals(secondString);
    }

    protected boolean compareDates(Date firstDate, Date secondDate) {
        if ((firstDate == null) || (secondDate == null))
            return (firstDate == null) && (secondDate == null);
        return firstDate.equals(secondDate);
    }

    protected boolean compareCollections
    (Collection firstCollection, Collection secondCollection) {
        if ((firstCollection == null) || (secondCollection == null))
            return (firstCollection == null) && (secondCollection == null);
        return firstCollection.containsAll(secondCollection) &&
        secondCollection.containsAll(firstCollection);
    }

    protected boolean compareMaps(Map firstMap, Map secondMap) {
        if ((firstMap == null) || (secondMap == null))
            return (firstMap == null) && (secondMap == null);
        return compareCollections(firstMap.values(), secondMap.values());
    }

    protected String angle(String value) {
        return "<" + value + ">";
    }

    protected String valueOf(String attributeName, String attributeValue) {
        return "\n  " + attributeName + ": " + angle(attributeValue);
    }

    protected String valueOf(String attributeName, long attributeValue) {
        return valueOf(attributeName, "" + attributeValue);
    }

    protected String valueOf(String attributeName, double attributeValue) {
        return valueOf(attributeName, "" + attributeValue);
    }

    protected String valueOf(String attributeName, char attributeValue) {
        return valueOf(attributeName, "" + attributeValue);
    }

    protected String valueOf(String attributeName, boolean attributeValue) {
        return valueOf(attributeName, "" + attributeValue);
    }

    protected String valueOf(String attributeName, Object attributeValue) {
        return valueOf(attributeName,
        (attributeValue != null) ? attributeValue.toString() : "NULL");
    }
}
