package com.bettersoftwarefaster.carserv.domain;

import com.bettersoftwarefaster.carserv.datamgmt.*;
import java.util.*;

/**
 * Implementation class that maintains a Map of Cars, to ensure that reference
 * object semantics of Car is maintained.  <p>Trusts clients, in the sense that
 * clients can provide Car details to the CarFactory and the factory will
 * remember them if that Car is previously unknown.  (This design is to reduce
 * to-ing and fro-ing to the back-end database).
 * @stereotype factory
 * @see Car
 */
public class CarFactory extends FactoryAdapter {
    CarFactory() {
    }

    public Car getCar(String licensePlate) throws DomainException {
        Car car = (Car)getMap().get(licensePlate);
        if (car == null) {
            try {
                CarData carData = getCarDM().getCar(licensePlate);
                car = new Car(carData);
                getMap().put(licensePlate, car);
            }
            catch (DatabaseNotAvailableException ex) {
                throw new DomainException("Not signed on.");
            }
            catch (DataMgmtException ex) {
                throw new DomainException("Problem locating car.", ex);
            }
        }
        else {
        }
        if (car == null) {
            throw new DomainException("No such car.");
        }
        return car;
    }

    /**
     * @param name - name of car to fetch
     * @param data - underlying data if car not previously known
     */
    public Car getCar(String licensePlate, CarData data)
    throws DomainException {
        Car car = (Car)getMap().get(licensePlate);
        if (car == null) {
            car = new Car(data);
            getMap().put(licensePlate, car);
        }
        else {
        }
        return car;
    }

    /**
     * @param name - name of car to fetch
     * @param owner - underlying owner if car not previously known
     * @param data - underlying data if car not previously known
     */
    public Car getCar(String licensePlate, Customer owner, CarData data)
    throws DomainException {
        Car car = (Car)getMap().get(licensePlate);
        if (car == null) {
            car = new Car(owner, data);
            getMap().put(licensePlate, car);
        }
        else {
        }
        return car;
    }

    private CarDM getCarDM() {
        return DomainSingletonManager.getInstance().getCarDM();
    }
}
