package com.bettersoftwarefaster.carserv.domain;

import java.util.*;
import com.bettersoftwarefaster.carserv.datamgmt.*;

/**
 * Represents an item (or a number of items) of some Part used to complete a
 * service. <p>Implementation: just a thin wrapper around underlying data
 * object; instead of PartData, uses Part.
 * @stereotype mi-detail
 * @stereotype valueObject
 * @represents specification
 */
public class ChargeableItem {
    ChargeableItem(Part part, int numberOfItems) {
        this.part = part;
        this.numberOfItems = numberOfItems;
    }

    public Part getPart() {
        return this.part;
    }

    public int getNumberOfItems() {
        return numberOfItems;
    }

    private int numberOfItems;

    /**
     * @clientCardinality 0..*
     * @supplierCardinality 1
     */
    private Part part;

    public boolean equals(Object other) {
        if (!(other instanceof ChargeableItem))
            return false;
        ChargeableItem otherChargeableItem = (ChargeableItem)other;
        return part.equals(otherChargeableItem.part) &&
        numberOfItems == otherChargeableItem.numberOfItems;
    }

    public int hashCode() {
        return part.hashCode() * numberOfItems;
    }
}
