package com.bettersoftwarefaster.carserv.domain;

import java.util.*;
import com.bettersoftwarefaster.carserv.datamgmt.*;

/**
 * Reference object representing some Customer of the garage.
 * @stereotype party
 * @see CustomerFactory
 */
public class Customer extends Person {
    Customer(CustomerData data) {
        super(data.getLastName(), data.getFirstName(), data.getAddress());
        reset(data);
        this.ownerRole = new CarOwnerAdapter();
        addRole(ownerRole);
    }

    public CarOwner getOwnerRole() { return ownerRole; }

    /**
     * @supplierCardinality 1
     * @link aggregationByValue
     * @clientCardinality 1
     */
    private CarOwner ownerRole;

    void reset(CustomerData data) {
        this.data = data;
        setLastName(data.getLastName());
        setFirstName(data.getFirstName());
    }

    /////////////////////////////////////////////////////
    public int getOwnerId() { return getCustomerId(); }

    public int getCustomerId() { return data.getId(); }

    public void update(String newLastName, String newFirstName)
    throws DomainException {
        try {
            CustomerData data = getCustDM().updateCustomer(getData(),
            newLastName, newFirstName);
            reset(data);
        }
        catch (DatabaseNotAvailableException ex) {
            throw new DomainException("Not signed on.", ex);
        }
        catch (DataMgmtException ex) {
            throw new DomainException("Unable to update", ex);
        }
    }

    public List cars() throws DomainException {
        if (cars == null) {
            fetchCars();
        }
        return Collections.unmodifiableList(cars);
    }

    private void fetchCars() throws DomainException {
        refreshCars();
    }

    public void refreshCars() throws DomainException {
        List cars = new ArrayList();
        CarData[] data;
        try {
            data = getCustDM().listCars(getData());
        }
        catch (DatabaseNotAvailableException ex) {
            throw new DomainException("Not signed on.", ex);
        }
        catch (DataMgmtException ex) {
            throw new DomainException("Problem when attempting to list cars", ex);
        }
        for (int i = 0; i < data.length; i++) {
            Car car = getCarFactory().getCar(data[i].getLicensePlate(),
            this, data[i]);
            cars.add(car);
        }
        this.cars = cars;
    }

    /////////////////////////////////////////////////////
    private CarFactory getCarFactory() throws DomainException {
        return DomainSingletonManager.getInstance().getCarFactory();
    }

    private CustDM getCustDM() throws DomainException {
        return DomainSingletonManager.getInstance().getCustDM();
    }

    private CustomerData getData() {
        return data;
    }

    public String toString() {
        return getCustomerId() + ": " + getFirstName() + " " + getLastName();
    }
    /////////////////////////////////////////////////////

    /**
     * @associates <{com.bettersoftwarefaster.carserv.domain.Car}>
     * @clientRole owner
     * @supplierCardinality 0..*
     * @constraint {set}
     * @bidirectional <{com.bettersoftwarefaster.carserv.domain.Car#owner}>
     */
    private List cars;

    private CustomerData data;

    public class CarOwnerAdapter implements CarOwner {
        public List cars() throws DomainException {
            return Customer.this.cars();
        }

        public int getOwnerId() {
            return Customer.this.getOwnerId();
        }

        public void refreshCars() throws DomainException {
            Customer.this.refreshCars();
        }
    }
}
