package com.bettersoftwarefaster.carserv.domain;

import java.util.*;
import com.bettersoftwarefaster.carserv.datamgmt.*;

/**
 * Implementation class that maintains a Map of Customers, to
 * ensure that reference
 * object semantics of Customer is maintained.  <p>Trusts clients, in the sense
 * that clients can provide Customer details to the CustomerFactory and the
 * factory will remember them if that Customer is previously unknown.  (This
 * design is to reduce to-ing and fro-ing to the back-end database).
 * @stereotype factory
 * @see Customer
 */
public class CustomerFactory extends FactoryAdapter {
    CustomerFactory() {
    }

    /**
     * @link dependency
     *   @associates <{Customer}>
     */
    public List listCustomers(String lastName, String firstName,
    String addressStreet, String addressPostalCode) throws DomainException {
        List customerList = new ArrayList();
        CustomerData[] data;
        try {
            data = getCustDM().findCustomers(lastName, firstName,
            addressStreet, addressPostalCode);
        }
        catch (DatabaseNotAvailableException ex) {
            throw new DomainException("Not signed on.");
        }
        catch (DataMgmtException ex) {
            throw new DomainException("Problem when attempting to find matching customers", ex);
        }
        for (int i = 0; i < data.length; i++) {
            customerList.add(getCustomer(data[i]));
        }
        return customerList;
    }

    public Customer getCustomer(int id) throws DomainException {
        CustomerData data;
        try {
            data = getCustDM().findCustomer(id);
        }
        catch (DataMgmtException ex) {
            throw new DomainException("Problem when attempting to find customer", ex);
        }
        return getCustomer(data);
    }

    private Customer getCustomer(CustomerData data) {
        int id = data.getId();
        Customer customer = (Customer)getMap().get(new Integer(id));
        if (customer == null) {
            customer = new Customer(data);
            getMap().put(new Integer(id), customer);
        }
        return customer;
    }

    private CustDM getCustDM() {
        return DomainSingletonManager.getInstance().getCustDM();
    }
}
