package com.bettersoftwarefaster.carserv.domain;

import java.util.*;
import java.sql.*;
import com.bettersoftwarefaster.carserv.datamgmt.*;
import com.bettersoftwarefaster.carserv.infra.SingletonRegister;

/**
 * Domain wrapper over superclass, to notify data managers in the datamgmt
 * package when a session is available (user signed on) or not (user logged
 * out).  Also, converts datamgmt concept of user (EmployeeData) into domain
 * concept (Employee).
 */
public class DomainSessionManager extends DatabaseSessionProviderAdapter {
    public DomainSessionManager() {
        addListener(getCarDM());
        addListener(getCustDM());
        addListener(getServiceDM());
    }

    public Employee signOn(String lastName, String firstName)
    throws DomainException {
        if (getCurrentDatabaseSession() != null) {
            logout();
        }
        EmployeeData employeeData;
        try {
            DatabaseSession session = new DatabaseSession();
            employeeData = session.signOn(lastName, firstName);
            fireSessionAvailable(session);
        }
        catch (DataMgmtException ex) {
            throw new DomainException("Unable to sign on.", ex);
        }
        this.currentUser = getEmployeeFactory().getEmployee(employeeData.getId(),
        employeeData);
        return getCurrentUser();
    }

    public void logout() {
        if (getCurrentUser() == null)
            return;
        DatabaseSession session = getCurrentDatabaseSession();
        fireSessionUnavailable(null);
        session.logout();
        this.currentUser = null;
    }

    public boolean isLoggedIn() {
        return getCurrentUser() != null;
    }

    public Employee getCurrentUser() {
        return currentUser;
    }

    public boolean isDatabaseConnectionValid() {
        return isSessionAvailable() &&
        getCurrentDatabaseSession().isDatabaseConnectionValid();
    }

    private CarDM getCarDM() {
        return DomainSingletonManager.getInstance().getCarDM();
    }

    private CustDM getCustDM() {
        return DomainSingletonManager.getInstance().getCustDM();
    }

    private ServiceDM getServiceDM() {
        return DomainSingletonManager.getInstance().getServiceDM();
    }

    private EmployeeFactory getEmployeeFactory() {
        return DomainSingletonManager.getInstance().getEmployeeFactory();
    }

    /**
     * @clientCardinality 1
     * @supplierCardinality 0..1
     * @supplierRole currentUser
     * @label creates / looks up
     */
    private Employee currentUser;
}
