package com.bettersoftwarefaster.carserv.domain;

import java.util.*;
import com.bettersoftwarefaster.carserv.datamgmt.*;

/**
 * Reference object representing an Employee of the garage.  Currently a
 * subclass of Person, along with Customer.  In future, might dispense with
 * such subclasses, and rely entirely on PersonRoles.  This might be one way
 * to provide 10% discounts to Customers who happen to be
 * Employees, for example.
 * @stereotype party
 * @see EmployeeFactory
 */
public class Employee extends Person {
    public Employee(EmployeeData data, Worker worker) {
        super(data.getLastName(), data.getFirstName());
        this.worker = worker;
        this.data = data;
        addRole(worker);
    }

    public boolean isMechanic() {
        return worker.isMechanic();
    }

    public boolean isManager() {
        return worker.isManager();
    }

    public Worker getWorkerRole() { return worker; }

    EmployeeData getData() {
        return data;
    }

    public int getEmployeeId() {
        return getData().getId();
    }

    public int getWorkerId() { return getEmployeeId(); }

    /**
     * @clientCardinality 1
     * @supplierCardinality 1
     * @link aggregationByValue
     */
    private Worker worker;
    private EmployeeData data;
}
