package com.bettersoftwarefaster.carserv.domain;

import java.util.*;
import com.bettersoftwarefaster.carserv.datamgmt.*;

/**
 * Implementation class that maintains a Map of Employees, to
 * ensure that reference
 * object semantics of Employee is maintained.  The appropriate role is also
 * created, depending upon whether there is a non-zero labour rate (implies
 * mechanic, not manager).  <p>Trusts clients, in the sense
 * that clients can provide Employee details to the EmployeeFactory and the
 * factory will remember them if that Employee is previously unknown.  (This
 * design is to reduce to-ing and fro-ing to the back-end database).
 * @stereotype factory
 * @see Employee
 */
public class EmployeeFactory extends FactoryAdapter {
    EmployeeFactory() {
    }

    /**
     * @param id Id of the employee being requested
     * @param data underlying data if not known by factory
     */
    public Employee getEmployee(int id, EmployeeData data) {
        Employee employee = getEmployee(id);
        if (employee == null) {
            Worker worker;
            if (data.getMechanicLabourRate() == 0L) {
                worker = new ManagerAdapter(data.getId());
            }
            else {
                worker = new MechanicAdapter(data.getId(),
                data.getMechanicLabourRate());
            }
            employee = new Employee(data, worker);
            getMap().put(new Integer(employee.getEmployeeId()), employee);
        }
        return employee;
    }

    public Employee getEmployee(int id) {
        return (Employee)getMap().get(new Integer(id));
    }
}
