package com.bettersoftwarefaster.carserv.domain;

import java.util.*;
import com.bettersoftwarefaster.carserv.datamgmt.*;

/**
 * Reference object for a Model, such as a Ford Focus or Subaru Impreza.  A
 * model defines a service schedule of regular services, defined by mileage
 * (7500, 15000, 22500 and so on); any Car of that model type would be
 * expected to be serviced at the appropriate mileages.
 * @stereotype description
 * @represents specification
 * @see ModelFactory
 */
public class Model {
    Model(String name, RegularServiceData[] data) {
        this.name = name;
        this.serviceSchedule = new ServiceSchedule();
        for (int i = 0; i < data.length; i++) {
            int mileage = data[i].getMileage();
            List parts = data[i].getParts();
            serviceSchedule.put(mileage,
            new ModelRegularService(mileage, parts));
        }
    }

    public java.util.List getServiceMileages() {
        return serviceSchedule.mileages();
    }

    public String getName() { return name; }

    private String name;

    public boolean isValidServiceMileage(int serviceMileage) {
        return serviceSchedule.isValidServiceMileage(serviceMileage);
    }

    public List parts(int serviceMileage) throws DomainException {
        return serviceSchedule.parts(serviceMileage);
    }

    public String toString() {
        return getName();
    }

    /**
     * @supplierCardinality 1
     * @link aggregationByValue
     */
    private final ServiceSchedule serviceSchedule;
}
