package com.bettersoftwarefaster.carserv.domain;

import java.util.*;
import com.bettersoftwarefaster.carserv.datamgmt.*;

/**
 * Represents a regular service (at some mileage) for a Model.  The regular
 * service defines a list of parts that should be replaced.  For example, at a
 * 40000 mile service, the oil and brake pads might need replacing.
 * @stereotype description
 */
public class ModelRegularService {
    ModelRegularService(int mileage, List parts) {
        this.mileage = mileage;
        this.parts = new ArrayList();
        for (Iterator iter = parts.iterator(); iter.hasNext(); ) {
            PartData data = (PartData)iter.next();
            Part part = getPartFactory().getPart(data.getNumber(), data);
            this.parts.add(part);
        }
    }

    public int getMileage() {
        return mileage;
    }

    public List parts() {
        return Collections.unmodifiableList(parts);
    }

    ///////////////////////////////////////////////
    private PartFactory getPartFactory() {
        return DomainSingletonManager.getInstance().getPartFactory();
    }
    ///////////////////////////////////////////////

    /**
     * @link aggregation
     *   @associates <{Part}>
     * @clientCardinality 0..*
     * @supplierCardinality 0..*
     * @constraint {ordered}
     */
    private List parts;
    private int mileage;
}
