package com.bettersoftwarefaster.carserv.domain;

import java.util.*;
import com.bettersoftwarefaster.carserv.datamgmt.*;

/**
 * Implementation class that maintains a Map of Parts, to ensure that reference
 * object semantics of Part is maintained.  <p>Trusts clients,
 * in the sense that
 * clients can provide Part details to the PartFactory and the factory will
 * remember them if that Part is previously unknown.  (This design is to
 * reduce to-ing and fro-ing to the back-end database).  In fact, totally
 * relies on clients to populate its map, so any design that uses ModelFactory
 * must ensure that it will have been correctly populated previously
 * before calling.
 * @stereotype factory
 * @see Part
 */
public class PartFactory extends FactoryAdapter {
    PartFactory() {
    }

    public Part getPart(int partNumber) throws DomainException {
        Part part = (Part)getMap().get(new Integer(partNumber));
        if (part == null) {
            throw new DomainException("No such part.");
        }
        return part;
    }

    /**
     * @param partNumber - part number
     * @param data       - underlying data if not previously known
     */
    public Part getPart(int partNumber, PartData data) {
        Part part = (Part)getMap().get(new Integer(partNumber));
        if (part == null) {
            part = new Part(data);
            getMap().put(new Integer(partNumber), part);
        }
        return part;
    }
}
