package com.bettersoftwarefaster.carserv.domain;

import java.util.*;
import com.bettersoftwarefaster.carserv.datamgmt.*;

/**
 * Abstraction of Customer and Employee.  Factors out common functionality,
 * but otherwise is not used.  The idea in the future is that we might provide
 * some special functionality for Customers who are Employees, for example,
 * giving them a 10% discount.
 * @stereotype party
 */
public class Person {
    Person(String lastName, String firstName) {
        this.lastName = lastName;
        this.firstName = firstName;
    }

    Person(String lastName, String firstName, AddressData address) {
        this(lastName, firstName);
        this.address = new Address(address);
    }

    public String getLastName() {
        return lastName;
    }

    protected void setLastName(String lastName) {
        this.lastName = lastName;
    }

    public String getFirstName() {
        return firstName;
    }

    protected void setFirstName(String firstName) {
        this.firstName = firstName;
    }

    public Address getAddress() {
        return address;
    }

    public void addRole(PersonRole role) {
        personRoles.add(role);
    }

    public void removeRole(PersonRole role) {
        personRoles.remove(role);
    }

    public String toString() {
        return firstName + " " + lastName;
    }

    private String firstName;
    private String lastName;

    /**
     * @link aggregation
     *   @associates <{PersonRole}>
     * @clientCardinality 1
     * @supplierCardinality 0..*
     * @supplierRole personRoles
     * @constraint {ordered}
     */
    private List personRoles = new ArrayList();

    /**
     * @clientCardinality 1
     * @supplierCardinality 0..1
     */
    private Address address;
}
