package com.bettersoftwarefaster.carserv.infra;

import java.util.*;
import java.io.*;

/**
 * Superclass for all exceptions - typically each dependent package will
 * define its own package-level exception, with this class as its superclass.
 * <p>Provides ability to wrap up a single underlying exception.  Also,
 * toString() implementation prints out the stack trace of the
 * underlying exception.
 * @stereotype exception
 */
public class CarServException extends Exception {
    public CarServException(String message) {
        this(message, null);
    }

    public CarServException(String message, Exception ex) {
        super(message);
        this.ex = ex;
    }

    public CarServException(Exception ex) {
        this("", ex);
    }

    public Exception getException() {
        return this.ex;
    }

    public String toString() {
        return getMessage() + unravelled(ex);
    }

    private Exception ex;

    private String unravelled(Exception ex) {
        if (ex == null) {
            return " (no embedded exception).";
        }
        else {
            StringWriter writer = new StringWriter();
            ex.printStackTrace(new PrintWriter(writer));
            return "\n>> embedded exception: " + ex.getMessage() +
            "\n>> stack trace: " + writer.toString();
        }
    }
}
