package com.bettersoftwarefaster.carserv.infra;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

/**
 * Enhanced version of JList, created with a DefaultListModel and a
 * mnemonic'd label.  Only single selection is provided for.
 * <p>When adding to a panel, the LabelledList's scrollpane
 * should be added rather than the list itself.
 */
public class LabelledList extends JList {
    public LabelledList(String labelString, char mnemonic) {
        super(new DefaultListModel());
        this.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        label = new JLabel(labelString);
        label.setDisplayedMnemonic(mnemonic);
        label.setLabelFor(this);
        scrollPane = new JScrollPane();
        scrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
        scrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
        scrollPane.setViewportView(this);
    }

    public void setEnabled(boolean enabled) {
        super.setEnabled(enabled);
        label.setEnabled(enabled);
        scrollPane.setEnabled(enabled);
        this.enabled = enabled;
    }

    public boolean getEnabled() {
        return enabled;
    }

    public DefaultListModel getDefaultListModel() {
        return (DefaultListModel)getModel();
    }

    /** Need to hold, no getEnabled() method provided for list, label etc. */
    private boolean enabled;
    private final JLabel label;
    private final JScrollPane scrollPane;

    public JLabel getLabel() { return label; }

    public JList getList() { return this; }

    public JScrollPane getScrollPane() { return scrollPane; }
}
