package com.bettersoftwarefaster.carserv.infra;

/**
 * Decorates a Command with a logging command.  Hence, can simply trace which
 * commands are being executed/undone.  <p>If no Logger specified, uses a
 * PrintLogger. <p>Note: it is currently the
 * responsibility of the application to provide traceCommand instances.  An
 * enhancement might be to enable/disable tracing within the CommandManager;
 * if tracing were enabled, it would wrap any executed commands with an
 * instance of TraceCommand.
 * @stereotype description
 * @see PrintLogger
 * @see TraceCommandInvocation
 */
public class TraceCommand extends CommandAdapter {
    public TraceCommand(Command command) {
        this(command, new PrintLogger());
    }

    public TraceCommand(Command command, Logger logger) {
        this.command = command;
        this.logger = logger;
    }

    public Logger getLogger() { return logger; }

    public Command getCommand() { return command; }

    public Command.ExecReturn execute(Object arg) throws CommandException {
        getLogger().log("executing: " + getCommand() + ", arg: <" + arg + ">");
        return getCommand().execute(arg);
    }

    public Command.UndoReturn undo(Object arg) throws CommandException {
        getLogger().log("undoing: " + getCommand() + ", arg: <" + arg + ">");
        return getCommand().undo(arg);
    }

    public String toString() {
        return "TraceCommand; logger = " + logger + ", command = " + command;
    }

    final private Logger logger;
    final private Command command;
}
