package com.bettersoftwarefaster.carserv.ui;

import com.bettersoftwarefaster.carserv.infra.*;
import com.bettersoftwarefaster.carserv.domain.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

/** Displays list of all services for a car. */
public class CarServicePanel extends JPanel implements CarSelectionListener,
SignOnListener, CarServiceListener {
    CarServicePanel(char servicesMnemonic) {
        setLayout(new BorderLayout());
        serviceTable = new ServiceTable();
        add(serviceTable.getScrollPane(), BorderLayout.CENTER);
    }

    public void addServiceSelectionListener(ServiceSelectionListener ssl) {
        listeners.add(ssl);
    }

    public void removeServiceSelectionListener(ServiceSelectionListener ssl) {
        listeners.remove(ssl);
    }

    public void notifyListeners() {
        for (Iterator iter = listeners.iterator(); iter.hasNext(); ) {
            ((ServiceSelectionListener)iter.next()).serviceSelectionChanged(
            new ServiceSelectionEvent(this,
            serviceTable.getSelectedService()));
        }
    }

    public void userSignedOn(SignOnEvent ev) {
        serviceTable.setEnabled(true);
    }

    public void userLoggedOut(SignOnEvent ev) {
        serviceTable.setEnabled(false);
    }

    public void carSelectionChanged(CarSelectionEvent ev) {
        refresh(ev.getCar());
    }

    public void carServiceChanged(CarServiceEvent ev) {
        if (ev.getCar() == null || ev.getCar().equals(this.car)) {
            refresh(this.car);
        }
    }

    private void refresh(Car car) {
        java.util.List services = car.services();
        serviceTable.clear();
        for (Iterator iter = services.iterator(); iter.hasNext(); ) {
            Service service = (Service)iter.next();
            serviceTable.addService(service);
        }
        this.car = car;
    }

    private Car car;

    /**
     * @clientCardinality 1
     * @supplierCardinality 1
     * @link aggregationByValue
     */
    private final ServiceTable serviceTable;
    private java.util.List listeners = new LinkedList();

    private class ServiceTable extends HeadedTable {
        ServiceTable() {
            addColumn(new HeadedTableColumn(0, 120, "Booked in", false));
            addColumn(new HeadedTableColumn(1, 70, "Type", false));
            addColumn(
            new HeadedTableColumn(2, 70, "Svc mileage", false, Number.class));
            addColumn(
            new HeadedTableColumn(3, 70, "Act mileage", false, Number.class));
            addColumn(new HeadedTableColumn(4, 120, "Picked up", false));
            addColumn(new HeadedTableColumn(5, "Hidden"));
            addListSelectionListener(
            new SelectionListener() {
                public void doValueChanged(ListSelectionEvent ev) {
                    selectedService = (Service)getHiddenValue();
                    notifyListeners();
                }
            });
            this.setPreferredScrollableViewportSize(new Dimension(300, 200));
        }

        private Service selectedService;

        public Service getSelectedService() { return selectedService; }

        public void insertService(int row, Service service) {
            String type;
            int serviceMileage;
            if (service instanceof RegularService) {
                type = "Regular";
                serviceMileage = ((RegularService)service).getServiceMileage();
            }
            else {
                type = "Adhoc";
                serviceMileage = 0;
            }
            getDefaultTableModel().insertRow(row,
            new Object[] {
                service.getBookedIn(), type, new Integer(serviceMileage),
                new Integer(service.getActualMileage()), service.getPickedUp(), service
            });
        }

        public void clear() {
            selectedService = null;
            super.clear();
        }

        public void addService(Service service) {
            insertService(getDefaultTableModel().getRowCount(), service);
        }
    }
}
