package com.bettersoftwarefaster.carserv.ui;

import com.bettersoftwarefaster.carserv.infra.*;
import com.bettersoftwarefaster.carserv.app.*;
import com.bettersoftwarefaster.carserv.domain.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

/**
 * Allows a customer to be looked up (using LookupCustomerDialog) and then
 * displays cars for that customer.  Allows car to be selected.
 * @see LookupCustomerDialog
 */
public class CustomerCarPanel extends GridBagPanel implements SignOnListener {
    CustomerCarPanel(char lookupButtonMnemonic, char carMnemonic) {
        customerLTF = new LabelledTextField("Customer:", 12);
        customerLTF.setEditable(false);
        customerLookupButton = new
        MnemonicButton(
        new AppAction("Lookup...", lookupButtonMnemonic) {
            public void actionPerformed(ActionEvent ev) {
                LookupCustomerDialog dialog =
                new LookupCustomerDialog(CustomerCarPanel.this);
                dialog.show();
                if (dialog.getCustomer() != null) {
                    setCustomer(dialog.getCustomer());
                }
            }
        });
        carCB = new CarComboBox(carMnemonic);
        add(customerLTF.getLabel(),
        new GridBagPanel.Constraints(0, 0, 1, 1, 13, 0, 0, 0, 5, 0));
        add(customerLTF,
        new GridBagPanel.Constraints(1, 0, 2, 1, 1.0, 0.0, 17, 2, 0, 0,
        0, 5, 5, 0));
        add(customerLookupButton,
        new GridBagPanel.Constraints(3, 0, 1, 1, 10, 0, 0, 10, 5, 0));
        add(carCB.getLabel(),
        new GridBagPanel.Constraints(0, 1, 1, 1, 13, 0, 0, 0, 0, 0));
        add(carCB,
        new GridBagPanel.Constraints(1, 1, 2, 1, 1.0, 0.0, 10, 2, 0, 0,
        0, 5, 0, 0));
    }

    private class CarComboBox extends LabelledComboBox {
        CarComboBox(char carMnemonic) {
            super("Car:", carMnemonic);
            addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent ev) {
                    Object item = getSelectedItem();
                    if (item != null) {
                        selectedCar = ((CarUi)item).getCar();
                        notifyListeners(selectedCar);
                    }
                }
            });
        }

        public void clear() {
            selectedCar = null;
            super.clear();
        }

        void addCar(Car car) {
            getDefaultComboBoxModel().addElement(new CarUi(car));
        }

        Car getSelectedCar() { return selectedCar; }

        private Car selectedCar;

        /** Thin presentation layer over the car */
        private class CarUi {
            CarUi(Car car) {
                this.car = car;
            }

            public String toString() {
                return car.getLicensePlate();
            }

            public Car getCar() {
                return car;
            }

            private Car car;
        }
    }

    public void setCustomer(Customer customer) {
        this.customer = customer;
        customerLTF.setText(customer.getLastName() + ", " +
        customer.getFirstName());
        try {
            java.util.List cars = customer.cars();
            carCB.clear();
            for (Iterator iter = cars.iterator(); iter.hasNext(); ) {
                Car car = (Car)iter.next();
                carCB.addCar(car);
            }
        }
        catch (DomainException ex) {
            System.err.println("Unable to fetch cars for customer");
        }
        if (carCB.getDefaultComboBoxModel().getSize() > 0) {
            carCB.setSelectedIndex(0);
        }
    }

    public Customer getCustomer() {
        return customer;
    }

    public Car getCar() {
        return car;
    }

    public void addCarSelectionListener(CarSelectionListener ccpl) {
        listeners.add(ccpl);
    }

    public void removeCarSelectionListener(CarSelectionListener ccpl) {
        listeners.remove(ccpl);
    }

    public void notifyListeners(Car car) {
        this.car = car;
        for (Iterator iter = listeners.iterator(); iter.hasNext(); ) {
            ((CarSelectionListener)iter.next()).carSelectionChanged(
            new CarSelectionEvent(this, car));
        }
    }

    private Customer customer;
    private Car car;

    public void userSignedOn(SignOnEvent ev) {
        customerLTF.setEnabled(true);
        customerLookupButton.setEnabled(true);
        carCB.setEnabled(true);
    }

    public void userLoggedOut(SignOnEvent ev) {
        customerLTF.setEnabled(false);
        customerLookupButton.setEnabled(false);
        carCB.setEnabled(false);
    }

    /**
     * @clientCardinality 1
     * @supplierCardinality 1
     * @link aggregationByValue
     */
    private final LabelledTextField customerLTF;

    /**
     * @clientCardinality 1
     * @supplierCardinality 1
     * @link aggregationByValue
     */
    private final MnemonicButton customerLookupButton;

    /**
     * @clientCardinality 1
     * @supplierCardinality 1
     * @link aggregationByValue
     */
    private final CarComboBox carCB;
    private java.util.List listeners = new LinkedList();
}
