package com.bettersoftwarefaster.carserv.ui;

import com.bettersoftwarefaster.carserv.infra.*;
import com.bettersoftwarefaster.carserv.app.*;
import com.bettersoftwarefaster.carserv.domain.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.sql.Date;

/**
 * Represents the diary panel, allowing date to be changed and showing the
 * services scheduled on that date.
 */
public class DiaryPanel extends JPanel implements SignOnListener,
CarServiceListener {
    DiaryPanel(char todayButtonMnemonic, char servicesListMnemonic) {
        setLayout(new BorderLayout());
        JPanel datePanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
        JPanel dateTodayPanel = new JPanel(new GridLayout(2, 1));
        dateTextField = new MutableDateTextField();
        final JButton todayButton =
        new MnemonicButton(
        new DateChangedCommandUI("Today", todayButtonMnemonic) {
            protected java.sql.Date doGetNextDate() {
                dateTextField.setDate(new java.util.Date());
                return new java.sql.Date(dateTextField.getDate().getTime());
            }
        });
        dateTodayPanel.add(dateTextField);
        dateTodayPanel.add(todayButton);
        final JButton backOneDayButton = new JButton(
        new DateChangedCommandUI("<") {
            protected java.sql.Date doGetNextDate() {
                dateTextField.decrementDay();
                return new java.sql.Date(dateTextField.getDate().getTime());
            }
        });
        final JButton forwardOneDayButton = new JButton(
        new DateChangedCommandUI(">") {
            protected java.sql.Date doGetNextDate() {
                dateTextField.incrementDay();
                return new java.sql.Date(dateTextField.getDate().getTime());
            }
        });
        final JButton backOneMonthButton = new JButton(
        new DateChangedCommandUI("<<") {
            protected java.sql.Date doGetNextDate() {
                dateTextField.decrementMonth();
                return new java.sql.Date(dateTextField.getDate().getTime());
            }
        });
        final JButton forwardOneMonthButton = new JButton(
        new DateChangedCommandUI(">>") {
            protected java.sql.Date doGetNextDate() {
                dateTextField.incrementMonth();
                return new java.sql.Date(dateTextField.getDate().getTime());
            }
        });
        JPanel buttonPanel = new JPanel(new GridLayout(2, 2));
        buttonPanel.add(backOneDayButton);
        buttonPanel.add(forwardOneDayButton);
        buttonPanel.add(backOneMonthButton);
        buttonPanel.add(forwardOneMonthButton);
        datePanel.add(dateTodayPanel);
        datePanel.add(buttonPanel);
        servicesPanel = new DiaryServicePanel(CarServApp.getInstance(),
        servicesListMnemonic);
        add(datePanel, BorderLayout.NORTH);
        add(servicesPanel, BorderLayout.CENTER);
        addCarServiceListener(this); // listen to own changes
        addDateSelectionListener(servicesPanel);
    }

    public java.sql.Date getSqlDate() {
        return new java.sql.Date(dateTextField.getDate().getTime());
    }

    public java.util.Date getDate() { return dateTextField.getDate(); }

    public void userSignedOn(SignOnEvent ev) {
        servicesPanel.userSignedOn(ev);
        notifyListeners(); // send out the current date
    }

    public void userLoggedOut(SignOnEvent ev) {
        servicesPanel.userLoggedOut(ev);
    }

    public void addCarServiceListener(CarServiceListener csl) {
        servicesPanel.addCarServiceListener(csl);
    }

    public void removeCarServiceListener(CarServiceListener csl) {
        servicesPanel.removeCarServiceListener(csl);
    }

    public void addDateSelectionListener(DateSelectionListener dsl) {
        listeners.add(dsl);
    }

    public void removeDateSelectionListener(DateSelectionListener dsl) {
        listeners.remove(dsl);
    }

    public void notifyListeners() {
        for (Iterator iter = listeners.iterator(); iter.hasNext(); ) {
            ((DateSelectionListener)iter.next()).dateSelectionChanged(
            new DateSelectionEvent(this, getSqlDate()));
        }
    }

    private java.util.List listeners = new LinkedList();

    public void carServiceChanged(CarServiceEvent ev) {
        servicesPanel.carServiceChanged(ev);
    }

    /**
     * @clientCardinality 1
     * @supplierCardinality 1
     * @link aggregationByValue
     */
    private final MutableDateTextField dateTextField;

    /**
     * @clientCardinality 1
     * @supplierCardinality 1
     * @link aggregationByValue
     * @supplierRole services
     */
    private final DiaryServicePanel servicesPanel;

    private abstract class DateChangedCommandUI extends DateChangedCommand {
        DateChangedCommandUI(String name, char mnemonic) {
            super(name, mnemonic);
        }

        DateChangedCommandUI(String name) {
            super(name);
        }

        protected final Object getArg(ActionEvent ev) {
            return new Arg(getSqlDate());
        }

        public void preExecute(Object argAsObject) {
            Arg arg = (Arg)argAsObject;
        }

        public void postExecute(Object argAsObject) {
            Arg arg = (Arg)argAsObject;
            dateTextField.setDate(doGetNextDate());
            arg.setNextDate(getSqlDate());
            notifyListeners();
        }

        public void executeFailed(CommandException ex) {
            CarServAppUI.getInstance().getStatusBar().setMessage(ex);
        }

        protected abstract java.sql.Date doGetNextDate();

        public void postUndo(Object argAsObject) {
            Arg arg = (Arg)argAsObject;
            dateTextField.setDate(arg.getPreviousDate());
            notifyListeners();
        }
    }
}
