package com.bettersoftwarefaster.carserv.ui;

import com.bettersoftwarefaster.carserv.infra.*;
import com.bettersoftwarefaster.carserv.app.*;
import com.bettersoftwarefaster.carserv.domain.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;

/** List of services scheduled for a certain date. */
public class DiaryServicePanel extends JPanel implements
DateSelectionListener {
    DiaryServicePanel(CarServApp app, char servicesMnemonic) {
        this.app = app;
        setLayout(new BorderLayout());
        serviceTable = new ServiceTable();
        add(serviceTable.getScrollPane(), BorderLayout.CENTER);
        carServiceChangeSupport = new CarServiceChangeSupport();
        final JMenuItem bookInMenuItem = new JMenuItem(
        new PopupBookInCommand(serviceTable) {
            protected Object getArg(ActionEvent ev) {
                if (selectedService == null) {
                    return null;
                }
                String mileageAsString =
                JOptionPane.showInputDialog(DiaryServicePanel.this,
                "Please enter actual mileage", "Book in",
                JOptionPane.QUESTION_MESSAGE);
                int mileage;
                try {
                    mileage = Integer.parseInt(mileageAsString);
                }
                catch (NumberFormatException ex) {
                    return null;
                }
                return new BookInCommand.Arg(selectedService, mileage);
            }
            protected void postExecute(Object arg) {
                CarServAppUI.getInstance().getStatusBar().setMessage("Booked in");
                notifyListeners(selectedService);
            }
            public void executeFailed(CommandException ex) {
                CarServAppUI.getInstance().getStatusBar().setMessage(ex);
            }
            protected void postUndo(Object arg) {
                notifyListeners(selectedService);
            }
            private void notifyListeners(Service service) {
                Car car = null;
                java.sql.Date bookedIn = null;
                try {
                    car = service.getCar();
                    bookedIn = service.getBookedIn();
                }
                catch (DomainException ex) {
                    // continue anyway
                }
                carServiceChangeSupport.notifyListeners(car, bookedIn);
            }
        });
        final JMenuItem cancelMenuItem = new JMenuItem(
        new PopupCancelCommand(serviceTable) {
            protected Object getArg(ActionEvent ev) {
                if (selectedService == null) {
                    return null;
                }
                return new CancelCommand.Arg(selectedService);
            }
            protected void postExecute(Object arg) {
                CarServAppUI.getInstance().getStatusBar().setMessage("Cancelled");
                notifyListeners(selectedService);
            }
            public void executeFailed(CommandException ex) {
                CarServAppUI.getInstance().getStatusBar().setMessage(ex);
            }
            protected void postUndo(Object arg) {
                notifyListeners(selectedService);
            }
            private void notifyListeners(Service service) {
                Car car = null;
                java.sql.Date bookedIn = null;
                try {
                    car = service.getCar();
                    bookedIn = service.getBookedIn();
                }
                catch (DomainException ex) {
                    // continue anyway
                }
                carServiceChangeSupport.notifyListeners(car, bookedIn);
            }
        });
        final JPopupMenu popupMenu = new JPopupMenu();
        popupMenu.add(bookInMenuItem);
        popupMenu.add(cancelMenuItem);
        serviceTable.addMouseListener(
        new MouseAdapter() {
            public void mousePressed(MouseEvent ev) { maybeShowPopup(ev); }
            public void mouseReleased(MouseEvent ev) { maybeShowPopup(ev); }
            private void maybeShowPopup(MouseEvent ev) {
                if (ev.isPopupTrigger()) {
                    Service service = serviceTable.getSelectedService();
                    Component[] components = popupMenu.getComponents();
                    for (int i = 0; i < components.length; i++) {
                        components[i].setEnabled((service != null) &&
                        (service.getActualMileage() == 0));
                    }
                    popupMenu.show(ev.getComponent(), ev.getX(), ev.getY());
                }
            }
        });
    }

    private abstract class PopupBookInCommand extends BookInCommand {
        PopupBookInCommand(ServiceTable serviceTable) {
            serviceTable.addListSelectionListener(serviceTable.new
            SelectionListener() {
                public void doValueChanged(ListSelectionEvent ev) {
                    selectedService = (Service)getHiddenValue();
                }
            });
        }

        protected Service selectedService;
    }

    private abstract class PopupCancelCommand extends CancelCommand {
        PopupCancelCommand(ServiceTable serviceTable) {
            serviceTable.addListSelectionListener(serviceTable.new
            SelectionListener() {
                public void doValueChanged(ListSelectionEvent ev) {
                    selectedService = (Service)getHiddenValue();
                }
            });
        }

        protected Service selectedService;
    }

    private class ServiceTable extends HeadedTable {
        ServiceTable() {
            addColumn(new HeadedTableColumn(0, 160, "Owner", false));
            addColumn(new HeadedTableColumn(1, 120, "License", false));
            addColumn(new HeadedTableColumn(2, 70, "Type", false));
            addColumn(
            new HeadedTableColumn(3, 70, "Svc mileage", false, Number.class));
            TableCellEditor actualMileageEditor = new ActualMileageEditor();
            addColumn(
            new HeadedTableColumn(4, 70, "Act mileage", actualMileageEditor,
            Number.class));
            addColumn(new HeadedTableColumn(5, "Hidden"));
            addListSelectionListener(
            new SelectionListener() {
                public void doValueChanged(ListSelectionEvent ev) {
                    selectedService = (Service)getHiddenValue();
                }
            });
            this.setPreferredScrollableViewportSize(new Dimension(300, 200));
        }

        protected Service selectedService;

        public Service getSelectedService() { return selectedService; }

        public void clear() {
            selectedService = null;
            super.clear();
        }

        public void insertService(int row, Service service) {
            try {
                Customer customer = (Customer)service.getCar().getOwner();
                String type;
                int serviceMileage;
                if (service instanceof RegularService) {
                    type = "Regular";
                    serviceMileage = ((RegularService)service).getServiceMileage();
                }
                else {
                    type = "Adhoc";
                    serviceMileage = 0;
                }
                getDefaultTableModel().insertRow(row,
                new Object[] {
                    customer.getLastName() + ", " + customer.getFirstName(),
                    service.getLicensePlate(), type, new Integer(serviceMileage),
                    new Integer(service.getActualMileage()), service // hidden
                });
            }
            catch (DomainException ex) {
                System.out.println("unable to add service");
            }
        }

        public void addService(Service service) {
            insertService(getDefaultTableModel().getRowCount(), service);
        }

        private class ActualMileageEditor extends DefaultCellEditor {
            ActualMileageEditor() {
                super(new WholeNumberField(0, 7));
                editorField = (WholeNumberField)getComponent();
                bookInCommand = new BookInCommand() {
                    protected Object getArg(ActionEvent ev) {
                        return new BookInCommand.Arg(selectedService,
                        editorField.getValue());
                    }
                    protected void postExecute(Object arg) {
                        notifyListeners(((BookInCommand.Arg)
                        arg).getService());
                    }
                    protected void postUndo(Object arg) {
                        notifyListeners(((BookInCommand.Arg)
                        arg).getService());
                    }
                    private void notifyListeners(Service service) {
                        Car car = null;
                        java.sql.Date bookedIn = null;
                        try {
                            car = service.getCar();
                            bookedIn = service.getBookedIn();
                        }
                        catch (DomainException ex) {
                            // continue anyway
                        }
                        carServiceChangeSupport.notifyListeners(car, bookedIn);
                    }
                };
                addListSelectionListener(ServiceTable.this.new
                SelectionListener() {
                    public void doValueChanged(ListSelectionEvent ev) {
                        selectedService = (Service)getHiddenValue();
                    }
                });
            }

            private WholeNumberField editorField;
            private BookInCommand bookInCommand;
            private Service selectedService;

            public boolean isCellEditable(EventObject anEvent) {
                return (selectedService.getActualMileage() == 0);
            }

            public Object getCellEditorValue() {
                return new Integer(editorField.getValue());
            }

            public boolean stopCellEditing() {
                super.stopCellEditing();
                bookInCommand.executeSelf();
                return true;
            }
        }
    }

    void refresh(java.sql.Date date) {
        serviceTable.clear();
        try {
            java.util.List services = app.listServices(date);
            for (Iterator iter = services.iterator(); iter.hasNext(); ) {
                Service eachService = (Service)iter.next();
                serviceTable.addService(eachService);
            }
        }
        catch (AppException ex) {
            System.err.println("Unable to list services, ex = " + ex);
        }
        this.date = date; // cache date in case have to refresh
    }

    public void addCarServiceListener(CarServiceListener csl) {
        carServiceChangeSupport.addCarServiceListener(csl);
    }

    public void removeCarServiceListener(CarServiceListener csl) {
        carServiceChangeSupport.removeCarServiceListener(csl);
    }

    public void carServiceChanged(CarServiceEvent ev) {
        // hack; comparing the dates directly fails due to daylight saving time.
        if (ev.getDate() == null ||
        ev.getDate().toString().equals(this.date.toString())) {
            refresh(this.date);
        }
    }

    public void dateSelectionChanged(DateSelectionEvent ev) {
        if (serviceTable.getEnabled()) {
            refresh(ev.getDate());
        }
    }

    public void userSignedOn(SignOnEvent ev) {
        serviceTable.setEnabled(true);
    }

    public void userLoggedOut(SignOnEvent ev) {
        serviceTable.setEnabled(false);
        serviceTable.clear();
    }

    private final CarServApp app;
    private java.sql.Date date;

    /**
     * @clientCardinality 1
     * @supplierCardinality 1
     * @link aggregationByValue
     */
    private final ServiceTable serviceTable;

    /**
     * @clientCardinality 1
     * @supplierCardinality 1
     * @link aggregationByValue
     */
    private CarServiceChangeSupport carServiceChangeSupport;
}
