package com.bettersoftwarefaster.carserv.ui;

import java.util.*;
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import com.bettersoftwarefaster.carserv.infra.*;
import com.bettersoftwarefaster.carserv.app.*;
import com.bettersoftwarefaster.carserv.domain.*;

/** Modal dialog box to find customer by name, address etc. */
public class LookupCustomerDialog extends JDialog {
    private static JFrame frameFor(JComponent parent) {
        Container topLevel = parent.getTopLevelAncestor();
        if (topLevel == null) {
            return null;
        }
        if (topLevel instanceof JFrame) {
            return (JFrame)topLevel;
        }
        return null;
    }

    public LookupCustomerDialog(final JComponent parent) {
        super(frameFor(parent), "Lookup Customer", true);
        setResizable(false);
        setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
        JPanel panel = new GridBagPanel();
        final LabelledTextField lastNameLTF =
        new LabelledTextField("Last name:", 'N', 15);
        final LabelledTextField streetLTF =
        new LabelledTextField("Street:", 'S', 25);
        final LabelledTextField postcodeLTF =
        new LabelledTextField("Post code:", 'P', 15);
        customerTable = new CustomerTable();
        final MnemonicButton okButton = new MnemonicButton(
        new AppAction("OK", 'O') {
            public void actionPerformed(ActionEvent ev) {
                customer = customerTable.getSelectedCustomer();
                dispose();
            }
        });
        final MnemonicButton lookupButton = new MnemonicButton(
        new AppAction("Lookup", 'k') {
            public void actionPerformed(ActionEvent ev) {
                customerTable.clear();
                okButton.setEnabled(false);
                try {
                    java.util.List customers =
                    getApp().listCustomers(lastNameLTF.getText(),
                    null, streetLTF.getText(), postcodeLTF.getText());
                    for (Iterator iter = customers.iterator();
                    iter.hasNext(); ) {
                        customerTable.addCustomer((Customer)iter.next());
                    }
                }
                catch (AppException ex) {
                    System.err.println("Problem listing customers, ex = " + ex);
                }
            }
        });
        customerTable.addListSelectionListener(customerTable.new
        SelectionListener() {
            public void doValueChanged(ListSelectionEvent ev) {
                okButton.setEnabled(true);
            }
        });
        final MnemonicButton cancelButton = new MnemonicButton(
        new AppAction("Cancel", 'l') {
            public void actionPerformed(ActionEvent ev) {
                customer = null;
                dispose();
            }
        });
        panel.add(lastNameLTF.getLabel(),
        new GridBagPanel.Constraints(0, 0, 1, 1, 13, 0, 15, 15, 0, 5));
        panel.add(lastNameLTF,
        new GridBagPanel.Constraints(1, 0, 1, 1, 17, 0, 15, 0, 0, 15));
        panel.add(streetLTF.getLabel(),
        new GridBagPanel.Constraints(0, 1, 1, 1, 13, 0, 5, 15, 0, 5));
        panel.add(streetLTF,
        new GridBagPanel.Constraints(1, 1, 1, 1, 17, 0, 5, 0, 0, 15));
        panel.add(postcodeLTF.getLabel(),
        new GridBagPanel.Constraints(0, 2, 1, 1, 13, 0, 5, 15, 0, 5));
        panel.add(postcodeLTF,
        new GridBagPanel.Constraints(1, 2, 1, 1, 17, 0, 5, 0, 0, 15));
        panel.add(lookupButton,
        new GridBagPanel.Constraints(0, 3, 1, 1, 17, 0, 15, 15, 0, 15));
        panel.add(customerTable.getScrollPane(),
        new GridBagPanel.Constraints(0, 4, 2, 3, 0.0, 1.0, 10, 1, 0, 0, 15,
        15, 15, 15));
        panel.add(okButton,
        new GridBagPanel.Constraints(0, 7, 1, 1, 17, 0, 15, 15, 30, 15));
        panel.add(cancelButton,
        new GridBagPanel.Constraints(1, 7, 1, 1, 13, 0, 15, 15, 30, 15));
        getContentPane().add(panel);
        pack();
        centreOnScreen();
    }

    public Customer getCustomer() {
        return customer;
    }

    private Customer customer;

    private void centreOnScreen() {
        Dimension preferredSize = this.getPreferredSize();
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        setLocation((screenSize.width - preferredSize.width) / 2,
        (screenSize.height - preferredSize.height) / 2);
    }

    private final CustomerTable customerTable;

    private static class CustomerTable extends HeadedTable {
        CustomerTable() {
            addColumn(new HeadedTableColumn(0, 80, "Last name", false));
            addColumn(new HeadedTableColumn(1, 80, "First name", false));
            addColumn(new HeadedTableColumn(2, 180, "Address", false));
            addColumn(new HeadedTableColumn(3, 60, "Postal code", false));
            addColumn(new HeadedTableColumn(4, "Hidden"));
            addListSelectionListener(
            new SelectionListener() {
                public void doValueChanged(ListSelectionEvent ev) {
                    selectedCustomer = (Customer)getHiddenValue();
                }
            });
            this.setPreferredScrollableViewportSize(new Dimension(200, 200));
        }

        private Customer selectedCustomer;

        public Customer getSelectedCustomer() { return selectedCustomer; }

        public void insertCustomer(int row, Customer customer) {
            getDefaultTableModel().insertRow(row,
            new Object[] {
                customer.getLastName(), customer.getFirstName(),
                customer.getAddress().getStreet(), customer.getAddress().getPostalCode(), customer
            });
        }

        public void addCustomer(Customer customer) {
            insertCustomer(getDefaultTableModel().getRowCount(), customer);
        }
    }
}
