package com.bettersoftwarefaster.carserv.ui;

import com.bettersoftwarefaster.carserv.infra.*;
import com.bettersoftwarefaster.carserv.app.*;
import com.bettersoftwarefaster.carserv.domain.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

/**
 * Panel to schedule a regular or an adhoc service, for currently
 * displayed date and car.
 */
public class ScheduleServicePanel extends GridBagPanel implements CarSelectionListener,
DateSelectionListener, SignOnListener {
    ScheduleServicePanel(char scheduleRegularServiceMnemonic,
    char scheduleAdhocServiceMnemonic, char mileageMnemonic,
    final char scheduleServiceMnemonic) {
        ButtonGroup scheduleServiceGroup = new ButtonGroup();
        final AppActionCommand scheduleRegularServiceCommand =
        new ScheduleRegularServiceCommand(scheduleServiceMnemonic) {
            protected Object getArg(ActionEvent ev) {
                return new ScheduleRegularServiceCommand.Arg(selectedCar,
                selectedDate, regularServiceMileageComboBox.getServiceMileage());
            }
            protected void postExecute(Object arg) {
                CarServAppUI.getInstance().getStatusBar().setMessage("Scheduled regular service.");
                carServiceChangeSupport.notifyListeners(selectedCar,
                selectedDate);
            }
            public void executeFailed(CommandException ex) {
                CarServAppUI.getInstance().getStatusBar().setMessage(ex);
            }
            protected void postUndo(Object arg) {
                carServiceChangeSupport.notifyListeners(selectedCar,
                selectedDate);
            }
        };
        final AppActionCommand scheduleAdhocServiceCommand =
        new ScheduleAdhocServiceCommand(scheduleServiceMnemonic) {
            protected Object getArg(ActionEvent ev) {
                return new ScheduleAdhocServiceCommand.Arg(selectedCar,
                selectedDate);
            }
            protected void postExecute(Object arg) {
                CarServAppUI.getInstance().getStatusBar().setMessage("Scheduled adhoc service.");
                carServiceChangeSupport.notifyListeners(selectedCar,
                selectedDate);
            }
            public void executeFailed(CommandException ex) {
                CarServAppUI.getInstance().getStatusBar().setMessage(ex);
            }
            protected void postUndo(Object arg) {
                carServiceChangeSupport.notifyListeners(selectedCar,
                selectedDate);
            }
        };
        scheduleRegularServiceRadioButton =
        new MnemonicRadioButton(
        new AppAction("Regular Service", scheduleRegularServiceMnemonic) {
            public void actionPerformed(ActionEvent ev) {
                regularServiceMileageComboBox.setEnabled(true);
                scheduleButton.setAction(scheduleRegularServiceCommand);
            }
        }, true);
        scheduleAdhocServiceRadioButton =
        new MnemonicRadioButton(
        new AppAction("Adhoc Service", scheduleAdhocServiceMnemonic) {
            public void actionPerformed(ActionEvent ev) {
                regularServiceMileageComboBox.setEnabled(false);
                scheduleButton.setAction(scheduleAdhocServiceCommand);
            }
        }, true);
        scheduleServiceGroup.add(scheduleRegularServiceRadioButton);
        scheduleServiceGroup.add(scheduleAdhocServiceRadioButton);
        regularServiceMileageComboBox =
        new RegularServiceMileageComboBox(mileageMnemonic);
        scheduleButton = new MnemonicButton(scheduleRegularServiceCommand);
        scheduleRegularServiceRadioButton.setHorizontalAlignment(SwingConstants.LEFT);
        add(scheduleRegularServiceRadioButton,
        new GridBagPanel.Constraints(0, 0, 1, 1, 10, 2, 0, 0, 5, 0));
        add(scheduleAdhocServiceRadioButton,
        new GridBagPanel.Constraints(0, 1, 1, 1, 10, 2, 0, 0, 5, 0));
        add(regularServiceMileageComboBox.getLabel(),
        new GridBagPanel.Constraints(1, 0, 1, 1, 10, 2, 0, 0, 0, 0));
        add(regularServiceMileageComboBox,
        new GridBagPanel.Constraints(2, 0, 1, 1, 10, 2, 0, 5, 0, 0));
        add(scheduleButton,
        new GridBagPanel.Constraints(3, 0, 1, 2, 10, 1, 0, 10, 0, 0));
        carServiceChangeSupport = new CarServiceChangeSupport();
    }

    public void carSelectionChanged(CarSelectionEvent ev) {
        selectedCar = ev.getCar();
        regularServiceMileageComboBox.setCarModel(selectedCar.getModel());
    }

    public void dateSelectionChanged(DateSelectionEvent ev) {
        selectedDate = ev.getDate();
    }

    private Car selectedCar;
    private java.sql.Date selectedDate;

    public void userSignedOn(SignOnEvent ev) {
        scheduleRegularServiceRadioButton.setEnabled(true);
        scheduleAdhocServiceRadioButton.setEnabled(true);
        regularServiceMileageComboBox.setEnabled(true);
        scheduleButton.setEnabled(true);
    }

    public void userLoggedOut(SignOnEvent ev) {
        scheduleRegularServiceRadioButton.setEnabled(false);
        scheduleAdhocServiceRadioButton.setEnabled(false);
        regularServiceMileageComboBox.setEnabled(false);
        scheduleButton.setEnabled(false);
    }

    public void addCarServiceListener(CarServiceListener dsl) {
        carServiceChangeSupport.addCarServiceListener(dsl);
    }

    public void removeCarServiceListener(CarServiceListener dsl) {
        carServiceChangeSupport.removeCarServiceListener(dsl);
    }

    /**
     * @clientCardinality 1
     * @supplierCardinality 1
     * @link aggregationByValue
     */
    private JRadioButton scheduleRegularServiceRadioButton;

    /**
     * @clientCardinality 1
     * @supplierCardinality 1
     * @link aggregationByValue
     */
    private JRadioButton scheduleAdhocServiceRadioButton;

    /**
     * @clientCardinality 1
     * @supplierCardinality 1
     * @link aggregationByValue
     */
    private RegularServiceMileageComboBox regularServiceMileageComboBox;

    /**
     * @clientCardinality 1
     * @supplierCardinality 1
     * @link aggregationByValue
     */
    private MnemonicButton scheduleButton;

    /**
     * @clientCardinality 1
     * @supplierCardinality 1
     * @link aggregationByValue
     */
    private CarServiceChangeSupport carServiceChangeSupport;

    private static class RegularServiceMileageComboBox extends
    LabelledComboBox {
        RegularServiceMileageComboBox(char mnemonic) {
            super("Mileage:", mnemonic);
            setEditable(false);
            addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent ev) {
                    Object item = getSelectedItem();
                    if (item != null) {
                        selectedServiceMileage = ((Integer)item).intValue();
                    }
                }
            });
        }

        public void setCarModel(Model model) {
            this.model = model;
            clear();
            for (Iterator iter = model.getServiceMileages().iterator();
            iter.hasNext(); ) {
                Integer serviceMileage = (Integer)iter.next();
                getDefaultComboBoxModel().addElement(serviceMileage);
            }
            if (getDefaultComboBoxModel().getSize() > 0) {
                setSelectedIndex(0);
            }
        }

        public int getServiceMileage() { return selectedServiceMileage; }

        public void clear() {
            selectedServiceMileage = 0;
            super.clear();
        }

        private int selectedServiceMileage;
        private Model model;
    }
}
