package com.bettersoftwarefaster.carserv.ui;

import com.bettersoftwarefaster.carserv.infra.*;
import com.bettersoftwarefaster.carserv.domain.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

/**
 * Abstract superclass for showing detail of a service - either labour or
 * parts.  Both have an adjustment.
 */
public abstract class ServiceDetailPanel extends JPanel implements SignOnListener,
ServiceSelectionListener {
    ServiceDetailPanel(ServiceDetailTable detailTable,
    char adjustmentsMnemonic) {
        setLayout(new BorderLayout());
        this.detailTable = detailTable;
        adjustmentsPanel = new AdjustmentsPanel(adjustmentsMnemonic);
        add(detailTable.getScrollPane(), BorderLayout.CENTER);
        add(adjustmentsPanel, BorderLayout.SOUTH);
    }

    protected ServiceDetailTable getDetailTable() {
        return detailTable;
    }

    public Service getService() {
        return service;
    }

    private Service service;

    public final void serviceSelectionChanged(ServiceSelectionEvent ev) {
        service = ev.getService();
        try {
            service.fillOut();
        }
        catch (DomainException ex) {
            System.err.println("unable to fill out detail for service");
        }
        detailTable.refresh(service);
        adjustmentsPanel.setAdjustments(doAdjustmentFor(service));
        adjustmentsPanel.setSubTotal(detailTable.getSubTotal());
        notifyListeners();
    }

    protected abstract long doAdjustmentFor(Service service);

    static abstract class ServiceDetailTable extends HeadedTable {
        ServiceDetailTable() {
            setPreferredScrollableViewportSize(new Dimension(220, 150));
        }

        public void clear() {
            this.subTotal = 0L;
            super.clear();
        }

        public void refresh(Service service) {
            clear();
            doRefresh(service);
        }

        protected abstract void doRefresh(Service service);

        private long subTotal;

        long getSubTotal() { return subTotal; }

        public void insertRow(int row, Object object) {
            subTotal += doInsertRow(row, object);
        }

        protected abstract long doInsertRow(int row, Object object);

        public void addRow(Object object) {
            insertRow(getDefaultTableModel().getRowCount(), object);
        }
    }

    public void userSignedOn(SignOnEvent ev) {
        detailTable.setEnabled(true);
        adjustmentsPanel.setEnabled(true);
    }

    public void userLoggedOut(SignOnEvent ev) {
        detailTable.setEnabled(false);
        adjustmentsPanel.setEnabled(false);
    }

    public void addSubTotalListener(TotalChangedListener stl) {
        listeners.add(stl);
    }

    public void removeSubTotalListener(TotalChangedListener stl) {
        listeners.remove(stl);
    }

    public void notifyListeners() {
        for (Iterator iter = listeners.iterator(); iter.hasNext(); ) {
            ((TotalChangedListener)iter.next()).totalChanged(
            new TotalEvent(this, getTotal()));
        }
    }

    public long getAdjustments() {
        return adjustmentsPanel.getAdjustments();
    }

    public long getTotal() {
        return adjustmentsPanel.getTotal();
    }

    /**
     * @clientCardinality 1
     * @supplierCardinality 1
     * @link aggregationByValue
     */
    private final ServiceDetailTable detailTable;

    /**
     * @clientCardinality 1
     * @supplierCardinality 1
     * @link aggregationByValue
     */
    private final AdjustmentsPanel adjustmentsPanel;

    /**
     * @clientCardinality 1
     * @supplierCardinality 1
     * @link aggregationByValue
     */
    private java.util.List listeners = new LinkedList();
}
