package com.bettersoftwarefaster.carserv.ui;

import com.bettersoftwarefaster.carserv.infra.*;
import com.bettersoftwarefaster.carserv.domain.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

/** Shows labour detail and adjustments for a service. */
public class ServiceLabourPanel extends ServiceDetailPanel {
    ServiceLabourPanel(char adjustmentsMnemonic) {
        super(new LabourTable(), adjustmentsMnemonic);
    }

    protected long doAdjustmentFor(Service service) {
        return service.getLabourAdjustments();
    }

    private static class LabourTable extends
    ServiceDetailPanel.ServiceDetailTable {
        LabourTable() {
            addColumn(new HeadedTableColumn(0, 120, "Employee", false));
            addColumn(
            new HeadedTableColumn(1, 70, "Hourly rate", false, Number.class));
            //			addColumn(new HeadedTableColumn(2, 30, "Notes", false));
            addColumn(
            new HeadedTableColumn(2, 70, "Hours", false, Number.class));
            addColumn(
            new HeadedTableColumn(3, 100, "Cost", false, Number.class));
            addColumn(new HeadedTableColumn(4, "Hidden"));
        }

        public void insertLabour(int row, Labour labour) {
            insertRow(row, labour);
        }

        public long doInsertRow(int row, Object object) {
            Labour labour = (Labour)object;
            Employee employee = labour.getMechanic();
            long hourlyRate;
            if (labour.getMechanic().isMechanic()) {
                hourlyRate = ((Mechanic)employee.getWorkerRole()).getHourlyRate(); //
                // in 1/100ths of GBP
            }
            else {
                // manager
                hourlyRate = 0L;
            }
            long tenthsHours = labour.getTenthsHours();
            long cost = tenthsHours * hourlyRate / 10L; // in 1/100ths of GBP
            getDefaultTableModel().insertRow(row,
            new Object[] {
                employee.getLastName() + ", " + employee.getFirstName(),
                new Double(hourlyRate / 10000.0),
                //					labour.getNotes(),
                new Double(tenthsHours / 10.0), new Double(cost / 10000.0), labour // hidden
            });
            return cost;
        }

        public void addLabour(Labour labour) {
            addRow(labour);
        }

        protected void doRefresh(Service service) {
            try {
                java.util.List labourItems = service.labour();
                for (Iterator iter = labourItems.iterator();
                iter.hasNext(); ) {
                    Labour eachLabourItem = (Labour)iter.next();
                    addLabour(eachLabourItem);
                }
            }
            catch (DomainException ex) {
                System.err.println("Unable to list labour items, ex = " + ex);
            }
        }
    }
}
