package com.bettersoftwarefaster.carserv.ui;

import com.bettersoftwarefaster.carserv.infra.*;
import com.bettersoftwarefaster.carserv.domain.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

/** Shows part detail and adjustments for a service. */
public class ServicePartsPanel extends ServiceDetailPanel {
    ServicePartsPanel(char adjustmentsMnemonic) {
        super(new ChargeableItemsTable(), adjustmentsMnemonic);
    }

    protected long doAdjustmentFor(Service service) {
        return service.getChargeableItemAdjustments();
    }

    private static class ChargeableItemsTable extends ServiceDetailTable {
        ChargeableItemsTable() {
            addColumn(new HeadedTableColumn(0, 120, "Part", false));
            addColumn(
            new HeadedTableColumn(1, 70, "Cost per item", false,
            Number.class));
            addColumn(
            new HeadedTableColumn(2, 70, "Num items", false, Number.class));
            addColumn(
            new HeadedTableColumn(3, 100, "Cost", false, Number.class));
            addColumn(new HeadedTableColumn(4, "Hidden"));
        }

        public void insertChargeableItem(int row,
        ChargeableItem chargeableItem) {
            insertRow(row, chargeableItem);
        }

        public long doInsertRow(int row, Object object) {
            ChargeableItem chargeableItem = (ChargeableItem)object;
            int numberOfItems = chargeableItem.getNumberOfItems();
            long costPerItem = chargeableItem.getPart().getCost(); // in 1/100ths of GBP
            long cost = costPerItem * numberOfItems; // in 1/100ths of GBP
            getDefaultTableModel().insertRow(row,
            new Object[] {
                chargeableItem.getPart(), new Double(costPerItem / 10000.0),
                new Integer(numberOfItems), new Double(cost / 10000.0), chargeableItem // hidden
            });
            return cost;
        }

        public void addChargeableItem(ChargeableItem chargeableItem) {
            addRow(chargeableItem);
        }

        protected void doRefresh(Service service) {
            try {
                java.util.List chargeableItems = service.chargeableItems();
                for (Iterator iter = chargeableItems.iterator();
                iter.hasNext(); ) {
                    ChargeableItem eachChargeableItem =
                    (ChargeableItem)iter.next();
                    addChargeableItem(eachChargeableItem);
                }
            }
            catch (DomainException ex) {
                System.err.println("Unable to list chargeable items, ex = " + ex);
            }
        }
    }
}
