package com.bettersoftwarefaster.carserv.ui;

import java.util.*;
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import com.bettersoftwarefaster.carserv.infra.*;
import com.bettersoftwarefaster.carserv.app.*;
import com.bettersoftwarefaster.carserv.domain.*;

/**
 * Sign on dialog.  Executes the SignOnCommand, and notifies CarServAppUI when
 * the sign on is complete.
 */
public class SignOnDialog extends JDialog implements
CommandCompletionListener {
    public SignOnDialog() {
        super(CarServAppUI.getInstance(), "Login", true);
        setResizable(false);
        setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
        JPanel panel = new GridBagPanel();
        final LabelledTextField firstNameLTF =
        new LabelledTextField("First name:", 'F', 20);
        final LabelledTextField lastNameLTF =
        new LabelledTextField("Last name:", 'L', 20);
        SignOnCommand signOnCommand = new SignOnCommand() {
            public Object getArg(ActionEvent ev) {
                return new SignOnCommand.Arg(lastNameLTF.getText(),
                firstNameLTF.getText());
            }
            public void postUndo(Object arg) {
                CarServAppUI.getInstance().userLoggedOut();
            }
            public void executeFailed(CommandException ex) {
                CarServAppUI.getInstance().getStatusBar().setMessage(ex);
            }
        };
        final JButton okButton = new MnemonicButton(signOnCommand, 'O');
        signOnCommand.addCommandCompletionListener(this);
        final JButton cancelButton = new MnemonicButton(
        new AbstractAction("Cancel") {
            public void actionPerformed(ActionEvent ev) {
                dispose();
            }
        }, 'l');
        panel.add(firstNameLTF.getLabel(),
        new GridBagPanel.Constraints(0, 0, 1, 1, 13, 0, 30, 15, 0, 5));
        panel.add(lastNameLTF.getLabel(),
        new GridBagPanel.Constraints(0, 1, 1, 1, 13, 0, 15, 15, 15, 5));
        panel.add(firstNameLTF,
        new GridBagPanel.Constraints(1, 0, 1, 1, 10, 0, 30, 0, 0, 15));
        panel.add(lastNameLTF,
        new GridBagPanel.Constraints(1, 1, 1, 1, 10, 0, 15, 0, 15, 15));
        panel.add(okButton,
        new GridBagPanel.Constraints(0, 2, 1, 1, 17, 0, 15, 15, 15, 0));
        panel.add(cancelButton,
        new GridBagPanel.Constraints(1, 2, 1, 1, 13, 0, 15, 0, 15, 15));
        getContentPane().add(panel);
        pack();
        centreOnScreen();
    }

    public void commandCompleted(Command c, CommandException ex) {
        if (ex != null) {
            JOptionPane.showMessageDialog(CarServAppUI.getInstance(),
            "Unable to sign on.");
            return;
        }
        CarServAppUI.getInstance().userSignedOn();
        dispose();
    }

    private void centreOnScreen() {
        Dimension preferredSize = this.getPreferredSize();
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        setLocation((screenSize.width - preferredSize.width) / 2,
        (screenSize.height - preferredSize.height) / 2);
    }
}
