package test.com.bettersoftwarefaster.carserv.app;

import junit.framework.*;
import junitx.framework.*;
//import junit.util.*;
import java.util.*;
import test.com.bettersoftwarefaster.carserv.*;
import test.com.bettersoftwarefaster.carserv.domain.DomainTestCase;
import com.bettersoftwarefaster.carserv.app.*;
import com.bettersoftwarefaster.carserv.domain.*;
import com.bettersoftwarefaster.carserv.datamgmt.DatabaseSession;

/** @stereotype test */
public class TestCarServApp extends TestCase {

	public TestCarServApp(String aName) {
		super(aName);
		this.testHelper = new TestHelper(DatabaseSession.DB_TYPE);
	}
	protected final TestHelper testHelper;

	public void testListCustomersOneCustomer() throws AppException {

		Employee user = getCarServApp().signOn("Percival", "Lance");
		List customers = getCarServApp().listCustomers("Smith", "Wilbur", null);

		assertEquals(1, customers.size());
	}

	public void testListCustomersTwoCustomers() throws AppException {
		Employee user = getCarServApp().signOn("Percival", "Lance");

		List customers = getCarServApp().listCustomers("Smith", null, null);
		assertEquals(2, customers.size());
	}

	public void testListCustomersNotSignedOn() {
		try {
			List customers = getCarServApp().listCustomers("Smith", "Wilbur", null);
			fail("Expected AppException to be thrown, instead caught nothing.");
		} catch(AppException ex) {
			// expected
		} catch(Exception ex) {
			fail("Expected AppException to be thrown, instead caught ex = " +  ex);
		}
	}

	public void testListServicesNoServices() throws AppException {
		Employee user = getCarServApp().signOn("Percival", "Lance");
		List services = getCarServApp().listServices(
			testHelper.makeDate(1970, 01, 01));
		assertEquals(0, services.size());
	}

	public void testListServicesOneService() throws AppException {
		Employee user = getCarServApp().signOn("Percival", "Lance");

		List services = getCarServApp().listServices(testHelper.makeDate(2001, 02, 01));
		assertEquals(1, services.size());

		Service service = (Service)services.get(0);
		assertEquals(testHelper.makeDate(2001, 02, 01), service.getBookedIn() );
	}

	protected void setUp() throws AppException, TestAccessException {
		this.carServApp = new CarServApp();
	}

	protected void tearDown() throws AppException, TestAccessException {
		this.carServApp.logout();
		this.carServApp = null;
	}

	protected CarServApp getCarServApp() {
		return carServApp;
	}

	private CarServApp carServApp;

	/**
	 * returns a <code>TestSuite</code> instance which contains this
	 * class as only test.
	 * @return  a <code>TestSuite</code>
	 */

	static public Test suite() {
		return new TestSuite(TestCarServApp.class);
	}
}


