package test.com.bettersoftwarefaster.carserv.app;

import junit.framework.*;
import junitx.framework.*;
//import junit.util.*;
import java.util.*;
import java.awt.event.*;
import com.bettersoftwarefaster.carserv.infra.*;
import test.com.bettersoftwarefaster.carserv.*;
import com.bettersoftwarefaster.carserv.app.*;

/** @stereotype test */
public class TestSignOnCommand extends TestCase {
    public TestSignOnCommand(String aName) {
        super(aName);
    }

    //    protected final TestHelper testHelper;
    private class CommandCompletionIndicator implements
    CommandCompletionListener {
        public void commandCompleted(Command command, CommandException ex) {
            complete = true;
        }

        public boolean isComplete() { return complete; }

        private boolean complete = false;
    }

    public void testExecuteCommandCompletionListener() throws AppException,
    InterruptedException {
        signOnLanceCommand.addCommandCompletionListener(commandCompletionIndicator);
        signOnLanceCommand.executeSelf();
        signOnLanceCommand.getThread().join();
        assertTrue(signOnLanceCommand.getMessage(),
        signOnLanceCommand.allAsExpected());
        assertTrue("listener", commandCompletionIndicator.isComplete());
    }

    public void testExecuteCurrentUser() throws AppException,
    InterruptedException {
        signOnLanceCommand.executeSelf();
        signOnLanceCommand.getThread().join();
        assertNotNull("app", getCarServApp());
        assertNotNull("current user", getCarServApp().getCurrentUser());
    }

    public void testExecuteCommandManager() throws AppException,
    InterruptedException {
        signOnLanceCommand.executeSelf();
        signOnLanceCommand.getThread().join();
        assertEquals("undo list", 1, commandManager.getSizeOfUndoList());
        assertEquals("redo list", 0, commandManager.getSizeOfRedoList());
    }

    public void testExecuteFailedBadDetails() throws AppException,
    InterruptedException {
        signOnFailBadDetailsCommand.addCommandCompletionListener(commandCompletionIndicator);
        signOnFailBadDetailsCommand.executeSelf();
        signOnFailBadDetailsCommand.getThread().join();
        assertTrue(signOnFailBadDetailsCommand.getMessage(),
        signOnFailBadDetailsCommand.allAsExpected());
        assertTrue("Listener", commandCompletionIndicator.isComplete());
        assertNotNull("app", getCarServApp());
        assertNull("current user", getCarServApp().getCurrentUser());
    }

    public void testExecuteSignOnWithNoLogout() throws AppException,
    InterruptedException {
        signOnLanceCommand.executeSelf();
        signOnLanceCommand.getThread().join();
        signOnHattieCommand.executeSelf();
        signOnHattieCommand.getThread().join();
        assertTrue(signOnHattieCommand.getMessage(),
        signOnHattieCommand.allAsExpected());
        assertNotNull("app", getCarServApp());
        assertNotNull("current user", getCarServApp().getCurrentUser());
        assertEquals("current user last name", "Jacques",
        getCarServApp().getCurrentUser().getLastName());
    }

    public void testUndo() throws AppException, CommandException,
    InterruptedException {
        signOnLanceCommand.executeSelf();
        signOnLanceCommand.getThread().join();
        commandManager.undo(); // is synchronous
        assertEquals("undo list", 0, commandManager.getSizeOfUndoList());

		/******
		 * orig expected results (can redo undone signon). //
		 * /
		assertEquals("redo list", 1, commandManager.getSizeOfRedoList());
		******/

		/******
		 * new expected results (redo not allowed for undone signon).*/
		//assertEquals("redo list", 0, commandManager.getSizeOfRedoList());
		/******/
		assertEquals("redo list", 1, commandManager.getSizeOfRedoList());

		assertNotNull("app", getCarServApp());
        assertNull("current user", getCarServApp().getCurrentUser());
    }

    public void testRedo() throws AppException, CommandException,
    InterruptedException {
        signOnLanceCommand.executeSelf();
        signOnLanceCommand.getThread().join();
        commandManager.undo(); // is synchronous

        /**
		 *********** orig expected results (can redo undone signon).
		 */
		commandManager.redo(); signOnLanceCommand.getThread().join();
		assertEquals("undo list", 1, commandManager.getSizeOfUndoList());
		assertEquals("redo list", 0, commandManager.getSizeOfRedoList());
		assertNotNull("app", getCarServApp());
		assertNotNull("current user", getCarServApp().getCurrentUser());
		assertEquals("current user last name", "Percival",
		getCarServApp().getCurrentUser().getLastName());
		/********/

        /**
		 *********** new expected results (redo not allowed for undone signon).
		 *
		try {
		    commandManager.redo();
			fail("redo should fail.");
		} catch(CommandException ex) {
		   // expected
		} catch(Exception ex) {
		   fail("expected to catch CommandException");
		}
		/********/

/*
		commandManager.redo(); signOnLanceCommand.getThread().join();
		assertEquals("undo list", 1, commandManager.getSizeOfUndoList());
		assertEquals("redo list", 0, commandManager.getSizeOfRedoList());
		assertNotNull("app", getCarServApp());
		assertNotNull("current user", getCarServApp().getCurrentUser());
		assertEquals("current user last name", "Percival",
		getCarServApp().getCurrentUser().getLastName());
*/
	}

    private abstract class QueryableSignOnCommand extends SignOnCommand {
        private String message;

        public String getMessage() { return message; }

        public boolean allAsExpected() { return (message == null); }

        void unexpected(String message) { this.message = message; }
    }

    protected void setUp() throws AppException, TestAccessException {
        this.carServApp = new CarServApp();
        commandCompletionIndicator = new CommandCompletionIndicator();
        this.signOnLanceCommand = new QueryableSignOnCommand() {
            public Object getArg(ActionEvent ev) {
                return new SignOnCommand.Arg("Percival", "Lance");
            }
            public void postExecute(Object arg) {
                // expected
            }
            public void executeFailed(CommandException ex) {
                unexpected("Did not expect executeFailed() to be called.");
            }
            public void postUndo(Object arg) {
                // expected
            }
            public void commandCompleted(Object arg, CommandException ex) {
                if (ex == null) {
                    unexpected("SignOnCommand completed, exception: " + ex);
                }
            }
        };
        this.signOnFailBadDetailsCommand = new QueryableSignOnCommand() {
            public Object getArg(ActionEvent ev) {
                return new SignOnCommand.Arg("Existent", "None");
            }
            public void postExecute(Object arg) {
                unexpected("Did not expect postExecute() to be called.");
            }
            public void executeFailed(CommandException ex) {
                if (ex == null) {
                    unexpected("executeFailed called, but ex was null");
                }
            }
            public void postUndo(Object arg) {
                unexpected("Did not expect postUndo() to be called.");
            }
            public void commandCompleted(Object arg, CommandException ex) {
                if (ex == null) {
                    unexpected("commandCompleted called, but ex was null");
                }
            }
        };
        this.signOnHattieCommand = new QueryableSignOnCommand() {
            public Object getArg(ActionEvent ev) {
                return new SignOnCommand.Arg("Jacques", "Hattie");
            }
            public void postExecute(Object arg) {
                // expected
            }
            public void executeFailed(CommandException ex) {
                unexpected("Did not expect executeFailed() to be called.");
            }
            public void postUndo(Object arg) {
                // expected
            }
            public void commandCompleted(Object arg, CommandException ex) {
                if (ex == null) {
                    unexpected("SignOnCommand completed, exception: " + ex);
                }
            }
        };
        this.commandManager = carServApp.getAppSingletonManager().getCommandManager();
    }

    protected void tearDown() throws AppException, TestAccessException {
        this.commandManager = null;
        carServApp.logout();
        this.commandCompletionIndicator = null;
        this.carServApp = null;
    }

    protected CarServApp getCarServApp() {
        return carServApp;
    }

    private CarServApp carServApp;
    private QueryableSignOnCommand signOnLanceCommand;
    private QueryableSignOnCommand signOnFailBadDetailsCommand;
    private QueryableSignOnCommand signOnHattieCommand;
    private CommandCompletionIndicator commandCompletionIndicator;
    private CommandManager commandManager;

    /**
     * returns a <code>TestSuite</code> instance which contains this
     * class as only test.
     * @return  a <code>TestSuite</code>
     */
    static public Test suite() {
        return new TestSuite(TestSignOnCommand.class);
    }
}
