package test.com.bettersoftwarefaster.carserv.datamgmt;

import java.util.*;
import java.sql.*;
import junit.framework.*;
import junitx.framework.*;
//import junit.util.*;
import com.bettersoftwarefaster.carserv.datamgmt.*;
import test.com.bettersoftwarefaster.carserv.*;



/** @stereotype test */
public class TestCarDM extends DataMgmtTestCase {
	/**
	 * constructor.
	 * @param    aName     a test name
	 */
	public TestCarDM(String aName) {
		super(aName);
	}

	public void testGetCarNoSuchCar() throws DataMgmtException, TestAccessException {
		CarData actualCar;
		actualCar = carDM.getCar("A 123 BCD");

		assertNull(actualCar);
	}

	public void testGetCarFoundTheCar() throws DataMgmtException, TestAccessException {
		CarData actualCar = carDM.getCar("V 654 NPQ");

		assertEquals(TestData.fredsFocus(), actualCar);
	}

	public void testListRegularServices() throws DataMgmtException, TestAccessException {
		RegularServiceData[] actualRegularServices =
			carDM.listRegularServices(TestData.fredsImpreza());

		assertTrue("Incorrect number of regular services for model",
			actualRegularServices.length == 8);

		RegularServiceData[] imprezaRegularServices =
			new RegularServiceData[] {
			 testHelper.makeRegularServiceData("Impreza", 7500,
				Arrays.asList(TestData.justOil()) ),
			 testHelper.makeRegularServiceData("Impreza", 15000,
				Arrays.asList(TestData.oilAndSparkPlug())),
			 testHelper.makeRegularServiceData("Impreza", 22500,
				Arrays.asList(TestData.justOil()) ),
			 testHelper.makeRegularServiceData("Impreza", 30000,
				Arrays.asList(TestData.oilSparkPlugAndBrakePads()) ),
			 testHelper.makeRegularServiceData("Impreza", 37500,
				Arrays.asList(TestData.justOil()) ),
			 testHelper.makeRegularServiceData("Impreza", 45000,
				Arrays.asList(TestData.oilAndSparkPlug()) ),
			 testHelper.makeRegularServiceData("Impreza", 52500,
				Arrays.asList(TestData.justOil()) ),
			 testHelper.makeRegularServiceData("Impreza", 60000,
				Arrays.asList(TestData.oilSparkPlugAndBrakePads()) )  };

		testHelper.assertContainSame(
			"regular services", Arrays.asList(actualRegularServices),
			Arrays.asList( imprezaRegularServices ) );

	}

	public void testListServices() throws DataMgmtException, TestAccessException {
		ServiceData[] actualServices = carDM.listServices(TestData.fredsFocus());
		ServiceData[] expectedServices = new ServiceData[] {
			testHelper.makeServiceData(TestData.fredsFocus().getLicensePlate(),
				testHelper.makeDate(2000,  4,  4), 0, 10232,
				testHelper.makeDate(2000,  4,  6), 0L, 0L, null, null),
			testHelper.makeServiceData(TestData.fredsFocus().getLicensePlate(),
				testHelper.makeDate(2000,  2, 24), 9000,  8992,
				testHelper.makeDate(2000,  2, 24), 0L, 0L, null, null),
			testHelper.makeServiceData(TestData.fredsFocus().getLicensePlate(),
				testHelper.makeDate(2000,  9, 12), 18000, 18023,
				testHelper.makeDate(2000,  9, 12), 0L, 0L, null, null),
			testHelper.makeServiceData(TestData.fredsFocus().getLicensePlate(),
				testHelper.makeDate(2001,  4, 1), 27000,      0,
				null,                 0L, 0L, null, null),
			 };

		testHelper.assertContainSame("services",
			Arrays.asList(expectedServices), Arrays.asList(actualServices) );
	}

	public void testScheduleRegularService() throws DataMgmtException, TestAccessException {

		ServiceData actualScheduledService = carDM.scheduleRegularService(
			TestData.wilbursLegacy(), testHelper.makeDate(2001, 8, 1), 18000);

		ServiceData expectedScheduledService = testHelper.makeServiceData(
			TestData.wilbursLegacy().getLicensePlate(),
			testHelper.makeDate(2001, 8, 1), 18000, 0,
			null, 0L, 0L, null, null);

		assertEquals("Scheduled (not completed) regular service different",
			expectedScheduledService,
			actualScheduledService);
	}

	public void testScheduleAdhocService() throws DataMgmtException, TestAccessException {

		ServiceData actualAdhocService = carDM.scheduleAdhocService(
			TestData.wilbursLegacy(), testHelper.makeDate(2001, 6, 1));

		ServiceData expectedAdhocService = testHelper.makeServiceData(
			TestData.wilbursLegacy().getLicensePlate(),
			testHelper.makeDate(2001, 6, 1), 0, 0, null, 0L, 0L, null, null);

		assertEquals(expectedAdhocService, actualAdhocService);
	}

	public void testCancelService() throws DataMgmtException, TestAccessException {

		// create a service; tested elsewhere
		ServiceData scheduledService = carDM.scheduleRegularService(
			TestData.wilbursLegacy(), testHelper.makeDate(2001, 8, 1), 18000);

		// under test
		carDM.cancelService(scheduledService);

		// list services now available; tested elsewhere
		ServiceData[] scheduledServices = carDM.listServices(TestData.wilbursLegacy());

		assertTrue("Scheduled service not cancelled.",
			   !(Arrays.asList(scheduledServices).contains(
			   			Arrays.asList(new ServiceData[] { scheduledService } )
					) ) );
	}

	/**
	 * prepares the environment and ressource to run the tests
	 * contained in this test case.
	 */
	protected void setUp() throws DataMgmtException {
		this.session = new DatabaseSession();
		this.carDM = new CarDM();
		this.carDM.sessionAvailable(new DatabaseSessionEvent(this, session));
	}

	/**
	 * resets the environment and releases the ressources used by the tests
	 * contained in this test case.
	 */
	protected void tearDown() throws DataMgmtException, TestAccessException {

		// clean up any stale data
//		Connection connection = session.getConnection();
		Connection connection = (Connection)invoke(session, "getConnection", new Object[] { } );

		testHelper.removeService(
			connection, TestData.wilbursLegacy().getLicensePlate(), testHelper.makeDate(2001, 8, 1));
		testHelper.removeService(
			connection, TestData.wilbursLegacy().getLicensePlate(), testHelper.makeDate(2001, 6, 1));

		this.carDM = null;
		this.session.logout();
		this.session = null;
	}


	private DatabaseSession session;
	private CarDM  carDM;

	/**
	 * returns a <code>TestSuite</code> instance which contains this
	 * class as only test.
	 * @return  a <code>TestSuite</code>
	 */
	static public Test suite() {
		return new TestSuite(TestCarDM.class);
	}

}


