package test.com.bettersoftwarefaster.carserv.datamgmt;

import junit.framework.*;
import junitx.framework.*;
//import junit.util.*;
import java.util.*;
import java.sql.*;
import java.awt.Color;
import com.bettersoftwarefaster.carserv.datamgmt.*;
import test.com.bettersoftwarefaster.carserv.*;


/** @stereotype test */
public class TestCustDM extends DataMgmtTestCase {

	/**
	 * constructor.
	 * @param    aName     a test name
	 */
	public TestCustDM(String aName) {
		super(aName);
	}


	public void testFindCustomersNoCustomers() throws DataMgmtException {
		CustomerData[] actualCustomers =
			custDM.findCustomers("Existent", "None", null);
		assertTrue("Should be no customer called None Existent, actually found " +
			actualCustomers.length, actualCustomers.length == 0 );
	}

	public void testFindCustomersOneCustomer() throws DataMgmtException, TestAccessException {
		CustomerData[] actualCustomers =
			custDM.findCustomers("Smith", "Wilbur", null);
		assertTrue("Should be just one customer called Wilbur Smith, actually found " +
				actualCustomers.length, (actualCustomers.length == 1) );

		testHelper.assertContainSame("customers",
			Arrays.asList(new CustomerData[] { TestData.wilburSmith()} ) ,
			Arrays.asList(actualCustomers) );
	}

	public void testFindCustomersMultipleCustomers() throws DataMgmtException, TestAccessException {
		CustomerData[] actualCustomers =
			custDM.findCustomers("Smith", null, null);
		assertTrue("Should be two customers called Smith, actually found " +
				actualCustomers.length, actualCustomers.length == 2);

		// note that marge and wilbur have same address
		testHelper.assertContainSame("customers",
			Arrays.asList(
				new CustomerData[] { TestData.margeSmith(), TestData.wilburSmith() } ),
			Arrays.asList(actualCustomers) );
	}

	public void testFindCustomersViaAddressPostalCode() throws DataMgmtException, TestAccessException {
		CustomerData[] actualCustomers =
			custDM.findCustomers(null, null, "L23 9TT");

		assertEquals(TestData.ireneMurdoch(), actualCustomers[0]);
	}

	public void testFindCustomer() throws DataMgmtException, TestAccessException {
		CustomerData actualCustomer = custDM.findCustomer(1);

		assertEquals(TestData.wilburSmith(), actualCustomer );
	}


	public void testListCarsNoCars() throws DataMgmtException, TestAccessException {
		CarData[] actualCars = custDM.listCars(TestData.ireneMurdoch() );
		assertTrue("Irene Murdoch should own no cars, actually found " +
				actualCars.length, actualCars.length == 0);
	}

	public void testListCarsOneCar() throws DataMgmtException, TestAccessException {
		CarData[] actualCars = custDM.listCars(TestData.wilburSmith() );

		testHelper.assertContainSame("cars",
			Arrays.asList(new CarData[] { TestData.wilbursLegacy() } ),
			Arrays.asList(actualCars) );
	}

	public void testListCarsTwoCar() throws DataMgmtException, TestAccessException {
		CarData[] actualCars = custDM.listCars(TestData.frederickForsyth() );

		testHelper.assertContainSame(
			"cars", Arrays.asList( new CarData[] {
				 TestData.fredsImpreza(), TestData.fredsFocus() } ),
					Arrays.asList(actualCars) );
	}

	public void testUpdateCustomer() throws DataMgmtException, TestAccessException {

		CustomerData fredDeedPollActual =
			custDM.updateCustomer(TestData.frederickForsyth(),
			"Forsooth", "Fred" );

		CustomerData fredDeedPollExpected =
			testHelper.makeCustomerData(TestData.frederickForsyth().getId(),
			"Forsooth", "Fred", TestData.frederickForsyth().getAddress());

		assertEquals("Unable to update customer last and first names",
			fredDeedPollActual, fredDeedPollExpected);

		CustomerData[] fredCommitted =
			custDM.findCustomers("Forsooth", "Fred", null);

		assertEquals("Commit change of update customer failed",
			fredCommitted[0], fredDeedPollExpected);
	}

	/**
	 * prepares the environment and ressource to run the tests
	 * contained in this test case.
	 */
	protected void setUp() throws DataMgmtException {
		this.session = new DatabaseSession();

		this.custDM = new CustDM();
		this.custDM.sessionAvailable(new DatabaseSessionEvent(this, session));
	}

	/**
	 * resets the environment and releases the ressources used by the tests
	 * contained in this test case.
	 */
	protected void tearDown() throws DataMgmtException {

		// clean up
		try {
			custDM.updateCustomer(TestData.frederickForsyth(),
				TestData.frederickForsyth().getLastName(),
				TestData.frederickForsyth().getFirstName() );
		} catch(Exception ex) { }

		this.custDM = null;
		this.session.logout();
		this.session = null;
	}

	private DatabaseSession session;
    private CustDM  custDM;

	/**
	 * returns a <code>TestSuite</code> instance which contains this
	 * class as only test.
	 * @return  a <code>TestSuite</code>
	 */
	static public Test suite() {
		return new TestSuite(TestCustDM.class);
	}
}


