package test.com.bettersoftwarefaster.carserv.datamgmt;

import junit.framework.*;
import junitx.framework.*;
//import junit.util.*;
import com.bettersoftwarefaster.carserv.datamgmt.*;
import java.sql.*;
import test.com.bettersoftwarefaster.carserv.*;


/** @stereotype test */
public class TestDatabaseSession extends DataMgmtTestCase{

	/**
	 * constructor.
	 * @param    aName     a test name
	 */
	public TestDatabaseSession(String aName) {
		super(aName);
	}

	public void testSignOn() throws DataMgmtException, TestAccessException {
		EmployeeData actualUser = session.signOn("Hawtrey", "Charles");
		EmployeeData expectedUser =
			testHelper.makeEmployeeData(2, "Hawtrey", "Charles", 330000);
		assertEquals(expectedUser, actualUser);
	}

	public void testLogout() throws DataMgmtException {
		EmployeeData notUsed = session.signOn("Hawtrey", "Charles");
		session.logout();
		try {
			session.getUser();
			fail("After logout, getUser() should have thrown an DatabaseNotAvailableException, instead nothing");
		} catch(DatabaseNotAvailableException ex) {
		}
	}

	public void testGetConnection() throws DataMgmtException, TestAccessException {
		EmployeeData actualUser = session.signOn("Hawtrey", "Charles");

//		Connection conn = session.getConnection();
		Connection conn = (Connection)invoke(session, "getConnection", new Object[] { } );

		assertNotNull(conn);
	}

	public void testGetUser() throws DataMgmtException {
		EmployeeData userFromSignOn = session.signOn("Hawtrey", "Charles");
		EmployeeData userFromGetUser = session.getUser();
		assertSame("User from signing on and user from getUser() should be one and the same",
			userFromSignOn, userFromGetUser);
	}

	/**
	 * prepares the environment and resource to run the tests
	 * contained in this test case.
	 *
	 * Obviously, do not expect this setUp to fail...
	 */
	protected void setUp() throws DataMgmtException {
		this.session = new DatabaseSession();
	}

	/**
	 * resets the environment and releases the ressources used by the tests
	 * contained in this test case.
	 */
	protected void tearDown() {
		this.session = null;
	}

	/**
	 * returns a <code>TestSuite</code> instance which contains this
	 * class as only test.
	 * @return  a <code>TestSuite</code>
	 */
	static public Test suite() {
		return new TestSuite(TestDatabaseSession.class);
	}

	/**
	 * @clientCardinality 1
	 * @supplierCardinality 0..1
	 */
	private DatabaseSession session;
}


