package test.com.bettersoftwarefaster.carserv.datamgmt;

import junit.framework.*;
import junitx.framework.*;
//import junit.util.*;
import java.util.*;
import java.sql.*;
import com.bettersoftwarefaster.carserv.datamgmt.*;
import test.com.bettersoftwarefaster.carserv.*;


/** @stereotype test */
public class TestServiceDM extends DataMgmtTestCase {
	/**
	 * constructor.
	 * @param    aName     a test name
	 */
	public TestServiceDM(String aName) {
		super(aName);
	}

	public void testListServicesNoService() throws DataMgmtException, TestAccessException {
		ServiceData[] data = serviceDM.listServices(testHelper.makeDate(1970, 01, 01));

		assertEquals(0, data.length);
	}

	public void testListServicesOneService() throws DataMgmtException, TestAccessException {
		ServiceData scheduledService =
			testHelper.makeServiceData(TestData.fredsFocus().getLicensePlate(),
				testHelper.makeDate(2001,  4, 1), 27000,      0,
				null,                 0L, 0L, null, null);

		ServiceData[] data = serviceDM.listServices(testHelper.makeDate(2001, 04, 01));

		assertEquals(1, data.length);
		assertEquals(scheduledService, data[0]);
	}

	public void testGetServiceOnlyScheduled() throws DataMgmtException, TestAccessException {
		ServiceData scheduledService =
			testHelper.makeServiceData(TestData.fredsFocus().getLicensePlate(),
				testHelper.makeDate(2001,  4, 1), 27000,      0,
				null,                 0L, 0L, null, null);

		ServiceData actualScheduledServiceWithDetail = serviceDM.getService(scheduledService);
		ServiceData expectedScheduledServiceWithDetail =
			testHelper.makeServiceData(TestData.fredsFocus().getLicensePlate(),
				testHelper.makeDate(2001,  4, 1), 27000,      0,
				null,                 0L, 0L, null, null );

		assertEquals(expectedScheduledServiceWithDetail, actualScheduledServiceWithDetail);
	}

	public void testGetServiceCompletedLabourOnly() throws DataMgmtException, TestAccessException {

		ServiceData adhocService =
			testHelper.makeServiceData(TestData.fredsImpreza().getLicensePlate(),
				testHelper.makeDate(1998,  7, 1), 0, 204,
				testHelper.makeDate(1998, 7, 2), 30000L, 0L, null, null);

		ServiceData actualAdhocServiceWithDetail = serviceDM.getService(adhocService);
		ServiceData expectedAdhocServiceWithDetail =
			testHelper.makeServiceData(TestData.fredsImpreza().getLicensePlate(),
				testHelper.makeDate(1998,  7, 1), 0, 204,
				testHelper.makeDate(1998, 7, 2), 300000L, 0L,
				testHelper.labourMapByMechanic(
					new ServiceData.Labour[] {
						testHelper.makeServiceDataLabour(TestData.charlesHawtrey(), 5, "Sorted out a rattle")
					}
			 	), null);

		assertEquals(expectedAdhocServiceWithDetail, actualAdhocServiceWithDetail);
	}

	public void testGetServiceCompletedLabourAndParts() throws DataMgmtException, TestAccessException {

		ServiceData impreza7500MileService =
			testHelper.makeServiceData(TestData.fredsImpreza().getLicensePlate(),
				testHelper.makeDate(1998, 11, 23), 	7500, 7598,
				testHelper.makeDate(1998, 11, 23), 0L, 0L, null, null);

		ServiceData actualImpreza7500ServiceWithDetail =
						serviceDM.getService(impreza7500MileService);
		ServiceData expectedImpreza7500ServiceWithDetail =
			testHelper.makeServiceData(TestData.fredsImpreza().getLicensePlate(),
				testHelper.makeDate(1998, 11, 23), 7500, 7598,
				testHelper.makeDate(1998, 11, 23), 0L, 0L,
			testHelper.labourMapByMechanic(
				new ServiceData.Labour[] {
					testHelper.makeServiceDataLabour(TestData.charlesHawtrey(), 10, null)
				}
		 	),
			 testHelper.chargeableItemsMapByPart(
				 new ServiceData.ChargeableItems[] {
					 testHelper.makeServiceDataChargeableItems(TestData.oil(), 1),
					 testHelper.makeServiceDataChargeableItems(TestData.sparkPlug(), 1),
				 }
			 )
			);

		assertEquals(expectedImpreza7500ServiceWithDetail, actualImpreza7500ServiceWithDetail);
	}

	public void testBookInCar() throws DataMgmtException, TestAccessException {

		// create a service; tested elsewhere
		ServiceData scheduledService = carDM.scheduleRegularService(
			TestData.wilbursLegacy(), testHelper.makeDate(2001, 8, 1), 18000);

		ServiceData actualBookedInService =
			serviceDM.bookInCar(scheduledService, 18012);

		ServiceData expectedBookedInService = testHelper.makeServiceData(
			TestData.wilbursLegacy().getLicensePlate(),
				testHelper.makeDate(2001, 8, 1), 18000, 18012,
				null, 0L, 0L, null, null);

		assertEquals(expectedBookedInService, actualBookedInService);
	}

	public void testPickUpCar() throws DataMgmtException, TestAccessException {

		// create a service and book in; tested elsewhere
		ServiceData scheduledService = carDM.scheduleRegularService(
			TestData.wilbursLegacy(), testHelper.makeDate(2001, 8, 1), 18000);
		scheduledService = serviceDM.bookInCar(scheduledService, 18012);

		ServiceData actualPickedUpService =
			serviceDM.pickUpCar(scheduledService, testHelper.makeDate(2001, 8, 2));
		ServiceData expectedPickedUpService = testHelper.makeServiceData(
			TestData.wilbursLegacy().getLicensePlate(),
			testHelper.makeDate(2001, 8, 1), 18000, 18012,
			testHelper.makeDate(2001, 8, 2), 0L, 0L, null, null);

		assertEquals(expectedPickedUpService, actualPickedUpService);
	}

	public void testUpdateLabourAddLabour() throws DataMgmtException, TestAccessException {

		// create a service and book in; tested elsewhere
		ServiceData scheduledService = carDM.scheduleAdhocService(
			TestData.wilbursLegacy(), testHelper.makeDate(2001, 6, 1));
		scheduledService = serviceDM.bookInCar(scheduledService, 18012);

		ServiceData actualServiceWithLabour =
			serviceDM.updateLabour(scheduledService,
				TestData.charlesHawtrey(), 15, "Sorted out a leak under the dashboard");
		ServiceData expectedServiceWithLabour = testHelper.makeServiceData(
			TestData.wilbursLegacy().getLicensePlate(),
			testHelper.makeDate(2001, 6, 1), 0, 18012,
			null, 0L, 0L,
			testHelper.labourMapByMechanic(new ServiceData.Labour[] {
			 	testHelper.makeServiceDataLabour(TestData.charlesHawtrey(), 15,
				"Sorted out a leak under the dashboard") } )
			, null);

		assertEquals(expectedServiceWithLabour, actualServiceWithLabour);
	}

	public void testUpdateLabourMoreLabourSameMechanic() throws DataMgmtException, TestAccessException {

		// create a service, book in and add some labour; tested elsewhere
		ServiceData scheduledService = carDM.scheduleAdhocService(
			TestData.wilbursLegacy(), testHelper.makeDate(2001, 6, 1));
		scheduledService = serviceDM.bookInCar(scheduledService, 18012);

		ServiceData scheduledServiceWithFirstSetOfLabour =
			serviceDM.updateLabour(scheduledService,
				TestData.charlesHawtrey(), 15, "Sorted out a leak under the dashboard");

		ServiceData actualServiceWithMoreLabour =
			serviceDM.updateLabour(scheduledServiceWithFirstSetOfLabour,
				TestData.charlesHawtrey(), 25, "Sorted out a leak under the dashboard; one more hours' work");

		ServiceData expectedServiceWithMoreLabour = testHelper.makeServiceData(
			TestData.wilbursLegacy().getLicensePlate(),
			testHelper.makeDate(2001, 6, 1), 0, 18012,
			null, 0L, 0L,
			testHelper.labourMapByMechanic(new ServiceData.Labour[] {
			 	testHelper.makeServiceDataLabour(TestData.charlesHawtrey(), 25,
				"Sorted out a leak under the dashboard; one more hours' work") } )
			, null);

		assertEquals(expectedServiceWithMoreLabour, actualServiceWithMoreLabour);

	}

	public void testUpdateLabourMoreLabourTwoMechanics() throws DataMgmtException, TestAccessException {

		// create a service, book in and add some labour; tested elsewhere
		ServiceData scheduledService = carDM.scheduleAdhocService(
			TestData.wilbursLegacy(), testHelper.makeDate(2001, 6, 1));
		scheduledService = serviceDM.bookInCar(scheduledService, 18012);

		ServiceData scheduledServiceWithFirstMechanic =
			serviceDM.updateLabour(scheduledService,
				TestData.charlesHawtrey(), 15, "Sorted out a leak under the dashboard");

		ServiceData actualServiceWithTwoMechanics =
			serviceDM.updateLabour(scheduledServiceWithFirstMechanic,
				TestData.hattieJacques(), 5, "Gave the engine a steam clean");

		ServiceData expectedServiceWithTwoMechanics = testHelper.makeServiceData(
			TestData.wilbursLegacy().getLicensePlate(),
			testHelper.makeDate(2001, 6, 1), 0, 18012, null, 0L, 0L,
			testHelper.labourMapByMechanic(new ServiceData.Labour[] {
			 	testHelper.makeServiceDataLabour(TestData.charlesHawtrey(), 15,
				"Sorted out a leak under the dashboard"),
			 	testHelper.makeServiceDataLabour(TestData.hattieJacques(), 5,
				"Gave the engine a steam clean") } )
			, null);

		assertEquals(expectedServiceWithTwoMechanics, actualServiceWithTwoMechanics);
	}

	public void testAdjustServiceLabour() throws DataMgmtException, TestAccessException {

		// create a service, book in and add two mechanics' labours; tested elsewhere
		ServiceData scheduledService = carDM.scheduleAdhocService(
			TestData.wilbursLegacy(), testHelper.makeDate(2001, 6, 1));
		scheduledService = serviceDM.bookInCar(scheduledService, 18012);

		ServiceData scheduledServiceWithFirstMechanic =
			serviceDM.updateLabour(scheduledService,
				TestData.charlesHawtrey(), 15, "Sorted out a leak under the dashboard");

		ServiceData scheduledServiceWithTwoMechanics =
			serviceDM.updateLabour(scheduledServiceWithFirstMechanic,
				TestData.hattieJacques(), 5, "Gave the engine a steam clean");

		// deduct 20 quid
		ServiceData actualServiceAfterLabourAdjustment =
			serviceDM.adjustServiceLabour(scheduledServiceWithTwoMechanics, 200000L);

		ServiceData expectedServiceAfterLabourAdjustment = testHelper.makeServiceData(
			TestData.wilbursLegacy().getLicensePlate(),
			testHelper.makeDate(2001, 6, 1), 0, 18012, null, 200000L, 0L,
			testHelper.labourMapByMechanic(new ServiceData.Labour[] {
			 	testHelper.makeServiceDataLabour(TestData.charlesHawtrey(), 15,
				"Sorted out a leak under the dashboard"),
			 	testHelper.makeServiceDataLabour(TestData.hattieJacques(), 5,
				"Gave the engine a steam clean") } )
			, null);

		assertEquals(expectedServiceAfterLabourAdjustment, actualServiceAfterLabourAdjustment);
	}

	public void testUpdateLabourDelLabour() throws DataMgmtException, TestAccessException {
		// create a service, book in and add some labour; tested elsewhere
		ServiceData scheduledService = carDM.scheduleAdhocService(
			TestData.wilbursLegacy(), testHelper.makeDate(2001, 6, 1));
		scheduledService = serviceDM.bookInCar(scheduledService, 18012);

		ServiceData scheduledServiceWithFirstMechanic =
			serviceDM.updateLabour(scheduledService,
				TestData.charlesHawtrey(), 15, "Sorted out a leak under the dashboard");
		ServiceData scheduledServiceWithTwoMechanics =
			serviceDM.updateLabour(scheduledServiceWithFirstMechanic,
				TestData.hattieJacques(), 5, "Gave the engine a steam clean");

		ServiceData actualServiceAfterRemoveLabour =
			serviceDM.updateLabour(scheduledServiceWithTwoMechanics,
				TestData.charlesHawtrey(), 0, null);

		ServiceData expectedServiceAfterRemoveLabour = testHelper.makeServiceData(
			TestData.wilbursLegacy().getLicensePlate(),
			testHelper.makeDate(2001, 6, 1), 0, 18012, null, 0L, 0L,
			testHelper.labourMapByMechanic(new ServiceData.Labour[] {
			 	testHelper.makeServiceDataLabour(TestData.hattieJacques(), 5,
				"Gave the engine a steam clean") } )
			, null);

		assertEquals(expectedServiceAfterRemoveLabour, actualServiceAfterRemoveLabour);
	}

	public void testUpdateChargeableItemAddItem() throws DataMgmtException, TestAccessException {

		// create a service and book in; tested elsewhere
		ServiceData scheduledService = carDM.scheduleRegularService(
			TestData.wilbursLegacy(), testHelper.makeDate(2001, 8, 1), 18000);
		scheduledService = serviceDM.bookInCar(scheduledService, 18012);

		ServiceData actualServiceWithChargeableItem =
			serviceDM.updateChargeableItem(scheduledService,
				TestData.oil(), 1);
		ServiceData expectedServiceWithChargeableItem = testHelper.makeServiceData(
			TestData.wilbursLegacy().getLicensePlate(),
			testHelper.makeDate(2001, 8, 1), 18000, 18012, null, 0L, 0L, null,
			testHelper.chargeableItemsMapByPart(new ServiceData.ChargeableItems[] {
			 	testHelper.makeServiceDataChargeableItems(TestData.oil(), 1) } )
			);

		assertEquals(expectedServiceWithChargeableItem, actualServiceWithChargeableItem);
	}

	public void testUpdateChargeableItemMoreItemsSamePart() throws DataMgmtException, TestAccessException {
		// create a service, book in and charge an item; tested elsewhere
		ServiceData scheduledService = carDM.scheduleRegularService(
			TestData.wilbursLegacy(), testHelper.makeDate(2001, 8, 1), 18000);
		scheduledService = serviceDM.bookInCar(scheduledService, 18012);
		ServiceData serviceWithChargeableItem =
			serviceDM.updateChargeableItem(scheduledService,
				TestData.sparkPlug(), 1);

		ServiceData actualServiceWithMoreOfSameChargeableItem =
			serviceDM.updateChargeableItem(scheduledService,
				TestData.sparkPlug(), 3);

		ServiceData expectedServiceWithMoreOfSameChargeableItem = testHelper.makeServiceData(
			TestData.wilbursLegacy().getLicensePlate(),
			testHelper.makeDate(2001, 8, 1), 18000, 18012, null, 0L, 0L, null,
			testHelper.chargeableItemsMapByPart(new ServiceData.ChargeableItems[] {
			 	testHelper.makeServiceDataChargeableItems(TestData.sparkPlug(), 3) } )
			);

		assertEquals(
			expectedServiceWithMoreOfSameChargeableItem,
			actualServiceWithMoreOfSameChargeableItem);
	}

	public void testUpdateChargeableItemMoreItemsTwoParts() throws DataMgmtException, TestAccessException {
		// create a service, book in and charge an item; tested elsewhere
		ServiceData scheduledService = carDM.scheduleRegularService(
			TestData.wilbursLegacy(), testHelper.makeDate(2001, 8, 1), 18000);
		scheduledService = serviceDM.bookInCar(scheduledService, 18012);
		ServiceData serviceWithChargeableItem =
			serviceDM.updateChargeableItem(scheduledService,
				TestData.oil(), 1);

		ServiceData actualServiceWithChargeableItemsTwoParts =
			serviceDM.updateChargeableItem(scheduledService,
				TestData.sparkPlug(), 4);

		ServiceData expectedServiceWithChargeableItemsTwoParts = testHelper.makeServiceData(
			TestData.wilbursLegacy().getLicensePlate(),
			testHelper.makeDate(2001, 8, 1), 18000, 18012, null, 0L, 0L, null,
			testHelper.chargeableItemsMapByPart(new ServiceData.ChargeableItems[] {
			 	testHelper.makeServiceDataChargeableItems(TestData.oil(), 1),
			 	testHelper.makeServiceDataChargeableItems(TestData.sparkPlug(), 4)
		} ) );

		assertEquals(
			expectedServiceWithChargeableItemsTwoParts,
			actualServiceWithChargeableItemsTwoParts);
	}

	public void testAdjustServiceChargeableItems() throws DataMgmtException, TestAccessException {
		// create a service, book in and charge an item; tested elsewhere
		ServiceData scheduledService = carDM.scheduleRegularService(
			TestData.wilbursLegacy(), testHelper.makeDate(2001, 8, 1), 18000);
		scheduledService = serviceDM.bookInCar(scheduledService, 18012);
		ServiceData serviceWithChargeableItem =
			serviceDM.updateChargeableItem(scheduledService,
				TestData.oil(), 1);
		ServiceData serviceWithChargeableItemsTwoParts =
			serviceDM.updateChargeableItem(scheduledService,
				TestData.sparkPlug(), 4);

		// knock off 10 quid for parts
		ServiceData actualServiceAfterChargeableItemAdjustment =
			serviceDM.adjustServiceChargeableItems(serviceWithChargeableItemsTwoParts,
			100000L);

		ServiceData expectedServiceAfterChargeableItemAdjustment = testHelper.makeServiceData(
			TestData.wilbursLegacy().getLicensePlate(),
			testHelper.makeDate(2001, 8, 1), 18000, 18012, null, 0L, 100000L, null,
			testHelper.chargeableItemsMapByPart(new ServiceData.ChargeableItems[] {
			 	testHelper.makeServiceDataChargeableItems(TestData.oil(), 1),
			 	testHelper.makeServiceDataChargeableItems(TestData.sparkPlug(), 4)
		} ) );

		assertEquals(
			expectedServiceAfterChargeableItemAdjustment,
			actualServiceAfterChargeableItemAdjustment);
	}

	public void testUpdateChargeableItemRemoveItem() throws DataMgmtException, TestAccessException {
		// create a service, book in and charge two items; tested elsewhere
		ServiceData scheduledService = carDM.scheduleRegularService(
			TestData.wilbursLegacy(), testHelper.makeDate(2001, 8, 1), 18000);
		scheduledService = serviceDM.bookInCar(scheduledService, 18012);
		ServiceData serviceWithChargeableItem =
			serviceDM.updateChargeableItem(scheduledService,
				TestData.oil(), 1);
		ServiceData serviceWithChargeableItemsTwoParts =
			serviceDM.updateChargeableItem(scheduledService,
				TestData.sparkPlug(), 4);

		ServiceData actualServiceAfterRemoveItem =
			serviceDM.updateChargeableItem(serviceWithChargeableItemsTwoParts,
				TestData.oil(), 0);

		ServiceData expectedServiceAfterRemoveItem = testHelper.makeServiceData(
			TestData.wilbursLegacy().getLicensePlate(),
			testHelper.makeDate(2001, 8, 1), 18000, 18012, null, 0L, 0L, null,
			testHelper.chargeableItemsMapByPart(new ServiceData.ChargeableItems[] {
			 	testHelper.makeServiceDataChargeableItems(TestData.sparkPlug(), 4)
		} ) );

		assertEquals(
			expectedServiceAfterRemoveItem,
			actualServiceAfterRemoveItem);
	}

	public void testUpdateLabourAndChargeableItems() throws DataMgmtException, TestAccessException {
		// create a service, book in and charge two items; tested elsewhere
		ServiceData scheduledService = carDM.scheduleRegularService(
			TestData.wilbursLegacy(), testHelper.makeDate(2001, 8, 1), 18000);
		scheduledService = serviceDM.bookInCar(scheduledService, 18012);

		ServiceData serviceWithFirstMechanic =
			serviceDM.updateLabour(scheduledService,
				TestData.charlesHawtrey(), 30, "18000 mile service");

		ServiceData serviceWithChargeableItem =
			serviceDM.updateChargeableItem(serviceWithFirstMechanic,
				TestData.oil(), 1);
		ServiceData serviceWithChargeableItemsTwoParts =
			serviceDM.updateChargeableItem(scheduledService,
				TestData.sparkPlug(), 4);

		ServiceData actualServiceWithFurtherMechanic =
			serviceDM.updateLabour(scheduledService,
				TestData.sidneyJames(), 5, "Steam clean engine");

		ServiceData expectedServiceWithFurtherMechanic = testHelper.makeServiceData(
			TestData.wilbursLegacy().getLicensePlate(),
			testHelper.makeDate(2001, 8, 1), 18000, 18012, null, 0L, 0L,
			testHelper.labourMapByMechanic(new ServiceData.Labour[] {
			 	testHelper.makeServiceDataLabour(TestData.charlesHawtrey(), 30,
				"18000 mile service"),
			 	testHelper.makeServiceDataLabour(TestData.sidneyJames(), 5,
				"Steam clean engine") } ),
			testHelper.chargeableItemsMapByPart(new ServiceData.ChargeableItems[] {
			 	testHelper.makeServiceDataChargeableItems(TestData.oil(), 1),
			 	testHelper.makeServiceDataChargeableItems(TestData.sparkPlug(), 4) }
			 ) );

		assertEquals(
			expectedServiceWithFurtherMechanic,
			actualServiceWithFurtherMechanic);
	}

	/**
	 * prepares the environment and resource to run the tests
	 * contained in this test case.
	 */
	protected void setUp() throws DataMgmtException {
		this.session   = new DatabaseSession();

		this.carDM     = new CarDM();
		this.carDM.sessionAvailable(new DatabaseSessionEvent(this, session));
		this.serviceDM = new ServiceDM();
		this.serviceDM.sessionAvailable(new DatabaseSessionEvent(this, session));
	}

	/**
	 * resets the environment and releases the ressources used by the tests
	 * contained in this test case.
	 */
	protected void tearDown() throws DataMgmtException, TestAccessException{

		// clean up any stale data
//		Connection connection = session.getConnection();
		Connection connection = (Connection)invoke(session, "getConnection", new Object[] { } );

		testHelper.removeService(
			connection, TestData.wilbursLegacy().getLicensePlate(), testHelper.makeDate(2001, 8, 1));
		testHelper.removeService(
			connection, TestData.wilbursLegacy().getLicensePlate(), testHelper.makeDate(2001, 6, 1));

		this.carDM = null;
		this.serviceDM = null;
		this.session.logout();
		this.session = null;
	}

	private DatabaseSession    session;
	private CarDM      carDM;
	private ServiceDM  serviceDM;

	/**
	 * returns a <code>TestSuite</code> instance which contains this
	 * class as only test.
	 * @return  a <code>TestSuite</code>
	 */
	static public Test suite() {
		return new TestSuite(TestServiceDM.class);
	}
}


