package test.com.bettersoftwarefaster.carserv.domain;

import junit.framework.*;
import junitx.framework.*;
//import junit.util.*;
import java.util.*;
import java.sql.*;
import com.bettersoftwarefaster.carserv.domain.*;
import com.bettersoftwarefaster.carserv.datamgmt.*;


/** @stereotype test */
public class TestCar extends CarTestCase {

	/**
	 * constructor.
	 * @param    aName     a test name
	 */
	public TestCar(String aName) {
		super(aName);
	}

	public void testServices() {
		List actualServices = nigellasCar.services();
		assertEquals(2, actualServices.size());
	}

	public void testGetLicensePlate() {
		assertEquals("X 987 XYZ", nigellasCar.getLicensePlate());
	}

	public void testGetColour() {
		assertEquals(new java.awt.Color(16777215), nigellasCar.getColour());
	}

	public void testGetOwnerKnownAlready() throws DomainException {
		assertEquals(nigella, nigellasCar.getOwner());
	}

	public void testGetOwnerNotKnownAlready() throws DomainException {
		Car car = getCarFactory().getCar( "X 123 ABC" );
		assertEquals(wilbur, car.getOwner());
	}

	public void testRefreshServices() throws DomainException {
		List services = nigellasCar.services();

		nigellasCar.refreshServices();
		List refreshedServices = nigellasCar.services();

		assertEquals(services.size(), refreshedServices.size());
	}

	public void testFillOutService() throws DomainException {

		Service nigellasFirstService = getService(nigellasCar, 7320);
		if (nigellasFirstService == null)
			fail("Unable to locate nigella's first service.");

		assertEquals("labour before fill out", 0, nigellasFirstService.labour().size());
		assertEquals("parts before fill out", 0, nigellasFirstService.chargeableItems().size());
		nigellasCar.fillOutService(nigellasFirstService.getBookedIn());

		nigellasFirstService = getService(nigellasCar, 7320);
		assertEquals("labour after fill out", 1, nigellasFirstService.labour().size());
		assertEquals("parts after fill out", 1, nigellasFirstService.chargeableItems().size());
	}

	private Service getService(Car car, int actualMileage) {
		for(Iterator iter = car.services().iterator(); iter.hasNext(); ) {
			Service eachService = (Service)iter.next();
			if (eachService.getActualMileage() == actualMileage) {
				return eachService;
			}
		}
		return null;
	}

	public void testScheduleRegularService() throws DomainException {
		int origNumberServices = nigellasCar.services().size();
		RegularService service =
			nigellasCar.scheduleRegularService(testHelper.makeDate(2002, 02, 01), 22500);
		assertEquals(nigellasCar, service.getCar());
		assertEquals(22500, service.getServiceMileage());
		assertEquals(testHelper.makeDate(2002, 02, 01), service.getBookedIn());
		int finalNumberServices = nigellasCar.services().size();
		assertEquals(origNumberServices+1, finalNumberServices);
	}

	public void testScheduleRegularServiceInvalidMileage() {
		try {
			nigellasCar.scheduleRegularService(testHelper.makeDate(2001, 04, 23), 22501);
			fail("Should have thrown DomainException, instead nothing.");
		} catch(DomainException ex) {
		}
	}

	public void testScheduleAdhocService() throws DomainException {
		AdhocService service = nigellasCar.scheduleAdhocService(testHelper.makeDate(2001, 06, 04));

		assertEquals(nigellasCar, service.getCar());
		assertEquals(testHelper.makeDate(2001, 06, 04), service.getBookedIn());
	}

	public void testCancelService() throws DomainException {
		AdhocService service = nigellasCar.scheduleAdhocService(testHelper.makeDate(2001, 06, 04));

		int origNumberServices = nigellasCar.services().size();
		nigellasCar.cancelService(service);
		int finalNumberServices = nigellasCar.services().size();

		assertEquals(origNumberServices-1, finalNumberServices);
	}

	protected void tearDown() throws DomainException, TestAccessException {

// can't access Connection directly.
//        Connection connection = getCarServApp().getCurrentDatabaseSession().getConnection();
		Connection connection = (Connection)invoke(getSessionManager().getCurrentDatabaseSession(), "getConnection", new Object[] { } );

		testHelper.removeService(
			connection,
			nigellasCar.getLicensePlate(), testHelper.makeDate(2002, 02, 01));
		testHelper.removeService(
			connection,
			nigellasCar.getLicensePlate(), testHelper.makeDate(2001, 06, 04));
		super.tearDown();
	}

	/**
	 * returns a <code>TestSuite</code> instance which contains this
	 * class as only test.
	 * @return  a <code>TestSuite</code>
	 */
	static public Test suite() {
		return new TestSuite(TestCar.class);
	}
}


