package test.com.bettersoftwarefaster.carserv.domain;

import junit.framework.*;
import junitx.framework.*;
//import junit.util.*;
import java.util.*;
import com.bettersoftwarefaster.carserv.domain.*;
import com.bettersoftwarefaster.carserv.datamgmt.*;
import test.com.bettersoftwarefaster.carserv.*;

/** @stereotype test */
public class TestDomainSessionManager extends DomainTestCase {

	public TestDomainSessionManager(String aName) {
		super(aName);
	}

	public void testSignOnManager() throws DomainException {

		Employee user =
			getSessionManager().
			signOn("Percival", "Lance");

		assertEquals(user.getLastName(), "Percival");
		assertEquals(user.getFirstName(), "Lance");
		assertTrue(user.isManager());
		assertTrue(!user.isMechanic());
	}

	public void testSignOnMechanic() throws DomainException {
		Employee user = getSessionManager().signOn("Jacques", "Hattie");

		assertEquals(user.getLastName(), "Jacques");
		assertEquals(user.getFirstName(), "Hattie");
		assertTrue(!user.isManager());
		assertTrue(user.isMechanic());
	}

	public void testSignOnSessionMadeAvailable() throws DomainException, TestAccessException {
		getSessionManager().signOn("Jacques", "Hattie");

		DataManager dm;
		dm = getSingletonManager().getCustDM();
		assertTrue("session not available for CustDM", dm.isSessionAvailable());
		dm = getSingletonManager().getServiceDM();
		assertTrue("session not available for ServiceDM", dm.isSessionAvailable());
		dm = getSingletonManager().getCarDM();
		assertTrue("session not available for CarDM", dm.isSessionAvailable());
	}

	public void testIsSessionAvailable() throws DomainException {
		Employee user = getSessionManager().signOn("Percival", "Lance");

		assertTrue(getSessionManager().isSessionAvailable());
	}

	public void testLogout() throws DomainException {
		Employee user = getSessionManager().signOn("Jacques", "Hattie");

		getSessionManager().logout();
		assertTrue(getSessionManager().getCurrentUser() == null);
	}

	public void testIsDatabaseConnectionValid() throws DomainException, TestAccessException {
		getSessionManager().signOn("Jacques", "Hattie");

		boolean isValid = ((Boolean)invoke(getSessionManager(), "isDatabaseConnectionValid", new Object[] { } )).booleanValue();
		assertTrue("database connection is valid.", isValid);
	}

	public void testIsDatabaseConnectionValidSignOnTwiceNoLogout() throws DomainException, TestAccessException {
		getSessionManager().signOn("Jacques", "Hattie");
		getSessionManager().signOn("Percival", "Lance");

		boolean isValid = ((Boolean)invoke(getSessionManager(), "isDatabaseConnectionValid", new Object[] { } )).booleanValue();
		assertTrue("database connection is valid.", isValid);
	}

	public void testIsDatabaseConnectionValidNotSignedOn() throws DomainException, TestAccessException {
		boolean isValid = ((Boolean)invoke(getSessionManager(), "isDatabaseConnectionValid", new Object[] { } )).booleanValue();
		assertTrue("database connection should not be valid.", !isValid);
	}

	public void testIsLoggedIn() throws DomainException {
		Employee user = getSessionManager().signOn("Jacques", "Hattie");

		assertTrue(getSessionManager().isLoggedIn());
	}

	public void testGetCurrentUser() throws DomainException {
		// throw away return
		getSessionManager().signOn("Jacques", "Hattie");

		Employee user = getSessionManager().getCurrentUser();

		assertEquals(user.getLastName(), "Jacques");
		assertEquals(user.getFirstName(), "Hattie");
		assertTrue(!user.isManager());
		assertTrue(user.isMechanic());
	}

	protected void tearDown() throws DomainException, TestAccessException {
		getSessionManager().logout();
		super.tearDown();
	}

	/**
	 * returns a <code>TestSuite</code> instance which contains this
	 * class as only test.
	 * @return  a <code>TestSuite</code>
	 */

	static public Test suite() {
		return new TestSuite(TestDomainSessionManager.class);
	}
}


