package test.com.bettersoftwarefaster.carserv.domain;

import junit.framework.*;
import junitx.framework.*;
//import junit.util.*;
import java.util.*;
import com.bettersoftwarefaster.carserv.domain.*;
import test.com.bettersoftwarefaster.carserv.*;

abstract public class TestService extends CarTestCase {

	/**
	 * constructor.
	 * @param    aName     a test name
	 */
	public TestService(String aName) {
		super(aName);
	}

	/**
	 * Template Pattern.  The provided service under test must be for
	 * nigella's Car.
	 */
	protected abstract Service serviceUnderTest();

	protected void setUp() throws DomainException, TestAccessException {
		super.setUp();
		partFactory = getSingletonManager().getPartFactory();
	}
	protected void tearDown() throws DomainException, TestAccessException {
		partFactory = null;
		super.tearDown();
	}


	public void testGetCar() throws DomainException {
		assertEquals(nigellasCar, serviceUnderTest().getCar());
	}

	public void testGetLicensePlate() {
		assertEquals(nigellasCar.getLicensePlate(), serviceUnderTest().getLicensePlate());
	}

	/**
	 * can't test here, depends on exact details of service under test
	 */
	public abstract void testGetBookedIn() throws DomainException;

	public void testBookInCar() throws DomainException {
		serviceUnderTest().bookInCar(22555);
		assertEquals(22555, serviceUnderTest().getActualMileage());
	}

	public void testGetActualMileage() throws DomainException {
		serviceUnderTest().bookInCar(22555);
		assertEquals(22555, serviceUnderTest().getActualMileage());
	}

	public void testGetAndSetLabourAdjustments() throws DomainException {
		serviceUnderTest().setLabourAdjustments(500000L); // 50 quid off!
		assertEquals(500000L, serviceUnderTest().getLabourAdjustments());
	}

	public void testGetAndSetChargeableItemAdjustments() throws DomainException {
		serviceUnderTest().setChargeableItemAdjustments(100000L); // 10 quid off!
		assertEquals(100000L, serviceUnderTest().getChargeableItemAdjustments());
	}

	public void testPickUpCar() throws DomainException {
		serviceUnderTest().pickUpCar(testHelper.makeDate(2002, 02, 02));
		assertEquals(testHelper.makeDate(2002, 02, 02), serviceUnderTest().getPickedUp());
	}

	public void testGetPickedUp() throws DomainException {
		serviceUnderTest().pickUpCar(testHelper.makeDate(2002, 02, 02));
		assertEquals(testHelper.makeDate(2002, 02, 02), serviceUnderTest().getPickedUp());
	}

	public void testUpdateLabour() throws DomainException {
		serviceUnderTest().updateLabour(5, "Half an hour cleaning the undertrim.");

		List labourList = serviceUnderTest().labour();
		assertEquals(1, labourList.size());

		Labour labour = (Labour)labourList.get(0);
		assertEquals(getSessionManager().getCurrentUser(), labour.getMechanic());
		assertEquals(5, labour.getTenthsHours());
		assertEquals("Half an hour cleaning the undertrim.", labour.getNotes());
	}

	public void testUpdateChargeableItem() throws DomainException, TestAccessException {
		Part oil = getPartFactory().getPart(TestData.oil().getNumber());
		serviceUnderTest().updateChargeableItem(oil, 3);

		List chargeableItemList = serviceUnderTest().chargeableItems();
		assertEquals(1, chargeableItemList.size());

		ChargeableItem item = (ChargeableItem)chargeableItemList.get(0);
		assertEquals(oil, item.getPart());
		assertEquals(3, item.getNumberOfItems());
	}

	private PartFactory getPartFactory() {
		return partFactory;
	}
	private PartFactory partFactory;
}


