package test.com.bettersoftwarefaster.carserv.infra;

import junit.framework.*;
import junitx.framework.*;
//import junit.util.*;
import java.util.*;
import java.io.*;
import com.bettersoftwarefaster.carserv.infra.*;


/** @stereotype test */
public class TestCompositeCommand extends ActionHistoryTestCase {

	/**
	 * constructor.
	 * @param    aName     a test name
	 */
	public TestCompositeCommand(String aName) {
		super(aName);
	}

	public void testExecute() throws CommandException {
		command.execute("This is a test");

		ActionHistory expectedActionHistory =
			new ActionHistory( new Object[] {
				"HardWiredActionCommand(cmd1), arg: <This is a test>",
				"HardWiredActionCommand(cmd2), arg: <This is a test>",
				"HardWiredActionCommand(cmd3), arg: <This is a test>"} );
		assertEquals( "action history", expectedActionHistory, getActionHistory() );
	}

	public void testExecuteIntermingled() throws CommandException {
		new UndoableActionCommand(getActionHistory()).execute("undoableActionCommandArg=1");
		command.execute("This is a test");
		new UndoableActionCommand(getActionHistory()).execute("undoableActionCommandArg=2");
		new UndoableActionCommand(getActionHistory()).execute("undoableActionCommandArg=3");

		ActionHistory expectedActionHistory =
			new ActionHistory( new Object[] {
				"undoableActionCommandArg=1",
				"HardWiredActionCommand(cmd1), arg: <This is a test>",
				"HardWiredActionCommand(cmd2), arg: <This is a test>",
				"HardWiredActionCommand(cmd3), arg: <This is a test>",
				"undoableActionCommandArg=2",
				"undoableActionCommandArg=3"  } );
		assertEquals( "action history", expectedActionHistory, getActionHistory() );
	}

	public void testExecuteAndUndoIntermingled() throws CommandException {
		Command undoDummy = new UndoableActionCommand(getActionHistory());

		new UndoableActionCommand(getActionHistory()).execute("undoableActionCommandArg=1");

		command.execute("This is a test");
		undoDummy.execute("undoableActionCommandArg=2");
		undoDummy.undo("undoableActionCommandArg=2");
		command.undo("This is a test");

		ActionHistory expectedActionHistory =
			new ActionHistory( new Object[] {
				"undoableActionCommandArg=1" } );
		assertEquals( "action history", expectedActionHistory, getActionHistory() );
	}

	public void testToString() {
		assertEquals(
			"CompositeCommand:"+
			"\ncommands[0] = HardWiredActionCommand(cmd1)"+
			"\ncommands[1] = HardWiredActionCommand(cmd2)"+
			"\ncommands[2] = HardWiredActionCommand(cmd3)", command.toString());
	}

	protected void setUp() {
		super.setUp();
		List commands = new ArrayList();
		commands.add(new HardWiredActionCommand(getActionHistory(), "cmd1"));
		commands.add(new HardWiredActionCommand(getActionHistory(), "cmd2"));
		commands.add(new HardWiredActionCommand(getActionHistory(), "cmd3"));

		command = new CompositeCommand(commands);
	}

	protected void tearDown() {
		command = null;
		super.tearDown();
	}

	/**
	 * Composite with only undoable's
	 */
	private CompositeCommand command;

	/**
	 * returns a <code>TestSuite</code> instance which contains this
	 * class as only test.
	 * @return  a <code>TestSuite</code>
	 */
	static public Test suite() {
		return new TestSuite(TestCompositeCommand.class);
	}
}
