package test.com.bettersoftwarefaster.carserv.infra;

import junit.framework.*;
import junitx.framework.*;
//import junit.util.*;
import java.util.*;
import java.io.*;
import com.bettersoftwarefaster.carserv.infra.*;


/**
 * The purpose of this test is not to test the test.NotUndoableCommand per se,
 * but rather the template class (SingleLevelUndoCommandAdapter) from which it
 * inherits most of its functionality.
 *
 * @stereotype test
 */
public class TestCompositeCommandInvocation extends ActionHistoryTestCase {

	/**
	 * constructor.
	 * @param    aName     a test name
	 */
	public TestCompositeCommandInvocation(String aName) {
		super(aName);
	}

	/**
	 * Make sure it executes okay
	 */
	public void testExecute() throws CommandException {
		commandInvocation.execute();

		ActionHistory expectedActionHistory =
			new ActionHistory(new Object[] {
				"invocation 1",
				"invocation 2",
				"invocation 3"
		} );
		assertEquals( "action history", expectedActionHistory, getActionHistory() );
	}

	public void testUndo() throws CommandException {
		commandInvocation.execute();
		commandInvocation.undo();

		ActionHistory expectedActionHistory =
			new ActionHistory(new Object[] { } );
		assertEquals( "action history", expectedActionHistory, getActionHistory() );
	}

	protected void setUp() {
		super.setUp();

		List commandInvocations = new ArrayList();

		commandInvocations.add(new SimpleCommandInvocation(new UndoableActionCommand(getActionHistory()), "invocation 1"));
		commandInvocations.add(new SimpleCommandInvocation(new UndoableActionCommand(getActionHistory()), "invocation 2"));
		commandInvocations.add(new SimpleCommandInvocation(new UndoableActionCommand(getActionHistory()), "invocation 3"));

		commandInvocation = new CompositeCommandInvocation(commandInvocations);
	}

	protected void tearDown() {
		commandInvocation = null;
		super.tearDown();
	}

	private CommandInvocation commandInvocation;

	/**
	 * returns a <code>TestSuite</code> instance which contains this
	 * class as only test.
	 * @return  a <code>TestSuite</code>
	 */
	static public Test suite() {
		return new TestSuite(TestCompositeCommandInvocation.class);
	}
}
