package test.com.bettersoftwarefaster.carserv.infra;

import junit.framework.*;
import junitx.framework.*;
//import junit.util.*;
import java.util.*;
import java.io.*;
import com.bettersoftwarefaster.carserv.infra.*;


/**
 * The purpose of this test is not to test the test.NotUndoableCommand per se,
 * but rather the template class (SingleLevelUndoCommandAdapter) from which it
 * inherits most of its functionality.
 *
 * @stereotype test
 */
public class TestNotUndoableCommand extends ActionHistoryTestCase {

	/**
	 * constructor.
	 * @param    aName     a test name
	 */
	public TestNotUndoableCommand(String aName) {
		super(aName);
		systemOut = System.out;  // hold onto for later
	}

	/**
	 * Make sure it executes okay
	 */
	public void testExecute() throws CommandException {
		OutputStream testOut = new ByteArrayOutputStream();
		System.setOut(new PrintStream(testOut));

		command.execute("This is a test");
		assertTrue("tracing, testOut is " + testOut, testOut.toString().startsWith("executing: NotUndoableActionCommand, arg: <This is a test>"));

		ActionHistory expectedActionHistory =
			new ActionHistory(new Object[] { "This is a test" } );
		assertEquals( "action history", expectedActionHistory, getActionHistory() );

		System.setOut(systemOut);
	}

	public void testUndo() {
		// swallow this output
		OutputStream testOut = new ByteArrayOutputStream();
		System.setOut(new PrintStream(testOut));
		try {
			command.execute("This is a test");
		} catch(CommandException ex) {
			fail("preparatory execute failed!");
		}

		// testing starts here
		testOut = new ByteArrayOutputStream();
		System.setOut(new PrintStream(testOut));

		try {
			command.undo("This is a test");
			fail("Expecting to catch CommandException, instead caught nothing");
		} catch(CommandException ex) {
			// expected
		} catch(Exception ex) {
			fail("Expecting to catch CommandException, instead caught ex = " + ex);
		}
//		assertEquals("tracing", 0, testOut.toString().length());

		ActionHistory expectedActionHistory =
			new ActionHistory(new Object[] { "This is a test" } );
		assertEquals( "action history", expectedActionHistory, getActionHistory() );

		System.setOut(systemOut);
	}

	protected void setUp() {
		super.setUp();
		command = new TraceCommand(new NotUndoableActionCommand(getActionHistory()));
	}

	protected void tearDown() {
		command = null;
		System.setOut(systemOut); // in case any problems during test runs
		super.tearDown();
	}

	private OutputStream testOut;
	private Command command;

	private final PrintStream systemOut;

	/**
	 * returns a <code>TestSuite</code> instance which contains this
	 * class as only test.
	 * @return  a <code>TestSuite</code>
	 */
	static public Test suite() {
		return new TestSuite(TestNotUndoableCommand.class);
	}
}
