package test.com.bettersoftwarefaster.carserv.infra;

import junit.framework.*;
import junitx.framework.*;
//import junit.util.*;
import java.util.*;
import java.io.*;
import com.bettersoftwarefaster.carserv.infra.*;


/**
 * The purpose of this test is not to test the test.NotUndoableCommand per se,
 * but rather the template class (SingleLevelUndoCommandAdapter) from which it
 * inherits most of its functionality.
 *
 * @stereotype test
 */
public class TestNotUndoableCommandInvocation extends ActionHistoryTestCase {

	/**
	 * constructor.
	 * @param    aName     a test name
	 */
	public TestNotUndoableCommandInvocation(String aName) {
		super(aName);
	}

	/**
	 * Make sure it executes okay
	 */
	public void testExecute() throws CommandException {
		commandInvocation.execute();

		ActionHistory expectedActionHistory =
			new ActionHistory(new Object[] { "invocation 1" } );
		assertEquals( "action history", expectedActionHistory, getActionHistory() );
	}

	public void testUndo() throws CommandException {
		Command.ExecReturn execReturn = commandInvocation.execute();

		assertTrue("not undoable", (execReturn != Command.ExecReturn.UNDOABLE) );

		try  {
			commandInvocation.undo();
			fail("Should have thrown CommandException, instead nothing.");
		} catch(CommandException ex) {
			// expected
		}

		ActionHistory expectedActionHistory =
			new ActionHistory(new Object[] { "invocation 1" } );
		assertEquals( "action history", expectedActionHistory, getActionHistory() );
	}

	protected void setUp() {
		super.setUp();
		commandInvocation = new SimpleCommandInvocation(
			new NotUndoableActionCommand(getActionHistory()), "invocation 1");
	}

	protected void tearDown() {
		commandInvocation = null;
		super.tearDown();
	}

	private CommandInvocation commandInvocation;

	/**
	 * returns a <code>TestSuite</code> instance which contains this
	 * class as only test.
	 * @return  a <code>TestSuite</code>
	 */
	static public Test suite() {
		return new TestSuite(TestNotUndoableCommandInvocation.class);
	}
}
