package test.com.bettersoftwarefaster.carserv.infra;

import junit.framework.*;
import junitx.framework.*;
//import junit.util.*;
import java.util.*;
import java.io.*;
import com.bettersoftwarefaster.carserv.infra.*;


/** @stereotype test */
public class TestNotUndoableCompositeCommand extends ActionHistoryTestCase {

	/**
	 * constructor.
	 * @param    aName     a test name
	 */
	public TestNotUndoableCompositeCommand(String aName) {
		super(aName);
	}

	public void testUndo() throws CommandException {

		Command.ExecReturn execReturn = command.execute("This is a test");

		assertTrue("not undoable", (execReturn != Command.ExecReturn.UNDOABLE) );

		try {
			command.undo(null);
			fail("Expected to catch CommandException, instead caught nothing");
		} catch(CommandException ex) {
			assertEquals("Caught CommandException", "Not undoable.", ex.getMessage());
		} catch(Exception ex) {
			fail("Expected to catch CommandException, instead caught ex = " + ex);
		}

		ActionHistory expectedActionHistory =
			new ActionHistory( new Object[] {
				"HardWiredActionCommand(cmd1), arg: <This is a test>",
				"HardWiredActionCommand(cmd2), arg: <This is a test>",
				"This is a test",
		} );
		assertEquals( "action history", expectedActionHistory, getActionHistory() );
	}

	protected void setUp() {
		super.setUp();
		List commands = new ArrayList();
		commands.add(new HardWiredActionCommand(getActionHistory(), "cmd1"));
		commands.add(new HardWiredActionCommand(getActionHistory(), "cmd2"));
 		commands.add(new NotUndoableActionCommand(getActionHistory()));

		command = new CompositeCommand(commands);
	}

	protected void tearDown() {
		command = null;
		super.tearDown();
	}

	/**
	 * Composite with some not undoable's
	 */
	private CompositeCommand command;

	/**
	 * returns a <code>TestSuite</code> instance which contains this
	 * class as only test.
	 * @return  a <code>TestSuite</code>
	 */
	static public Test suite() {
		return new TestSuite(TestNotUndoableCompositeCommand.class);
	}
}
