package test.com.bettersoftwarefaster.carserv.infra;

import junit.framework.*;
import junitx.framework.*;
//import junit.util.*;
import java.util.*;
import java.io.*;
import com.bettersoftwarefaster.carserv.infra.*;


/** @stereotype test */
public class TestTraceCommand extends ActionHistoryTestCase {

	/**
	 * constructor.
	 * @param    aName     a test name
	 */
	public TestTraceCommand(String aName) {
		super(aName);
		systemOut = System.out;  // hold onto for later
	}

	public void testExecute() throws CommandException {
		OutputStream testOut = new ByteArrayOutputStream();
		System.setOut(new PrintStream(testOut));

		command.execute("This is a test");
		assertTrue("tracing, testOut is " + testOut, testOut.toString().startsWith("executing: UndoableActionCommand, arg: <This is a test>"));

		ActionHistory expectedActionHistory =
			new ActionHistory(new Object[] { "This is a test" } );
		assertEquals( "action history", expectedActionHistory, getActionHistory() );

		System.setOut(systemOut);
	}

	public void testToString() {
		assertEquals("TraceCommand; logger = PrintLogger, command = UndoableActionCommand", command.toString());
	}

	protected void setUp() {
		super.setUp();
		command = new TraceCommand(new UndoableActionCommand(getActionHistory()));
	}

	protected void tearDown() {
		command = null;
		System.setOut(systemOut); // in case any problems during test runs
		super.tearDown();
	}

	private OutputStream testOut;
	private Command command;

	private final PrintStream systemOut;

	/**
	 * returns a <code>TestSuite</code> instance which contains this
	 * class as only test.
	 * @return  a <code>TestSuite</code>
	 */
	static public Test suite() {
		return new TestSuite(TestTraceCommand.class);
	}
}
