package test.com.bettersoftwarefaster.carserv.infra;

import junit.framework.*;
import junitx.framework.*;
//import junit.util.*;
import java.util.*;
import java.io.*;
import com.bettersoftwarefaster.carserv.infra.*;


/** @stereotype test */
public class TestTraceCommandInvocation extends ActionHistoryTestCase {

	/**
	 * constructor.
	 * @param    aName     a test name
	 */
	public TestTraceCommandInvocation(String aName) {
		super(aName);
		systemOut = System.out;  // hold onto for later
	}

	public void testExecute() throws CommandException {
		OutputStream testOut = new ByteArrayOutputStream();
		System.setOut(new PrintStream(testOut));

		commandInvocation.execute();
		assertTrue("tracing, testOut is " + testOut, testOut.toString().startsWith("executing: CommandInvocation: command=<UndoableActionCommand> arg=<this is a test>"));

		ActionHistory expectedActionHistory =
			new ActionHistory(new Object[] { "this is a test" } );
		assertEquals( "action history", expectedActionHistory, getActionHistory() );

		System.setOut(systemOut);
	}

	public void testUndo() throws CommandException {
		// swallow this output
		OutputStream testOut = new ByteArrayOutputStream();
		System.setOut(new PrintStream(testOut));
		commandInvocation.execute();

		// testing starts here
		testOut = new ByteArrayOutputStream();
		System.setOut(new PrintStream(testOut));

		commandInvocation.undo();
		assertTrue("tracing, testOut is " + testOut, testOut.toString().startsWith("undoing: CommandInvocation: command=<UndoableActionCommand> arg=<this is a test>"));

		ActionHistory expectedActionHistory =
			new ActionHistory(new Object[] { } );
		assertEquals( "action history", expectedActionHistory, getActionHistory() );

		System.setOut(systemOut);
	}

	public void testToString() {
		assertEquals("TraceCommandInvocation; logger = PrintLogger, commandInvocation = CommandInvocation: command=<UndoableActionCommand> arg=<this is a test>", commandInvocation.toString());
	}

	protected void setUp() {
		super.setUp();
		commandInvocation = new TraceCommandInvocation(
			new SimpleCommandInvocation(
			new UndoableActionCommand(getActionHistory()), "this is a test") );
	}

	protected void tearDown() {
		commandInvocation = null;
		System.setOut(systemOut); // in case any problems during test runs
		super.tearDown();
	}

	private OutputStream testOut;
	private CommandInvocation commandInvocation;

	private final PrintStream systemOut;

	/**
	 * returns a <code>TestSuite</code> instance which contains this
	 * class as only test.
	 * @return  a <code>TestSuite</code>
	 */
	static public Test suite() {
		return new TestSuite(TestTraceCommandInvocation.class);
	}
}
