package com.togethersoft.modules.qa.audit.HUNGARIAN;

import java.util.*;

import com.togethersoft.modules.qa.api.*;
import com.togethersoft.modules.qa.utils.*;

import com.togethersoft.openapi.sci.*;
import com.togethersoft.openapi.sci.enum.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class HUNGARIAN extends AuditPluginImpl{

    private final static String[] typeNames = {
        "boolean", "byte", "short", "int", "long", "float", "double", "char" };
    private final static String[] typeDefaultPrefix = {
        "z",       "b",    "s",     "i",   "l",   "f",      "d",      "c" };

	public HUNGARIAN() {
        typeCheckers = new TypeChecker[typeNames.length];
        for(int i=0; i<typeCheckers.length;i++) {
            typeCheckers[i] = new TypeChecker(typeNames[i]);
        }
        restoreOptions();
    }

    public boolean storeOptions() {
        for(int i=0; i<typeCheckers.length;i++) {
            putProperty(typeCheckers[i].getTypeName()+".enabled",
                typeCheckers[i].isEnabled());
            putProperty(typeCheckers[i].getTypeName()+".prefix",
                typeCheckers[i].getPrefix());
        }
        return true;
    }

    public void restoreOptions() {
        for(int i=0; i<typeCheckers.length;i++) {
            typeCheckers[i].setEnabled(
                getProperty(typeCheckers[i].getTypeName()+".enabled", true));
            typeCheckers[i].setPrefix(
                getProperty(typeCheckers[i].getTypeName()+".prefix", typeDefaultPrefix[i]));
        }
    }

    public boolean run(SciObject obj) {
        procGoal = PROC_OPERS | PROC_ATTRS;
        process(obj);
        return true;
    }
	protected void processConceptVar(SciVariable var) {
    	super.processConceptVar(var);
		for(int i=0; i<typeCheckers.length;i++) {
            if (! typeCheckers[i].isOk(var)) {
                AuditViolations.add(this,currMbr,var,
                    "'" + typeCheckers[i].getTypeName() + "' " +
                    "variables start with '" +
                    typeCheckers[i].getPrefix() + "'");
            }
        }
	}
    public java.lang.String getTitle() {
        return "Hungarian Notation";
    }
    public java.lang.String getGroupName() {
        return "Better Software Faster";
    }
	public boolean getDefaultChoice() {
        return true;
    }
    public java.lang.String getDefaultSeverity() {
        return SEVERITY_NORMAL;
    }
	public JComponent createOptionsPane() {
        optionsPane = new JPanel();
        for (int i=0; i<typeCheckers.length; i++) {
			optionsPane.add(typeCheckers[i].toComponent());
        }
        return optionsPane;
    }
    public JComponent getOptionsPane() {
        return optionsPane;
    }

	/**
	 * @supplierRole optionsPane 
	 */
    private JPanel optionsPane;

	/**
     * @associates <{com.togethersoft.modules.qa.audit.HUNGARIAN.TypeChecker}>
     * @supplierCardinality 1..*
     * @constraints {frozen}
     * @link aggregationByValue
     */
    private final TypeChecker[] typeCheckers;

    private void forTestingOnly() {
		boolean xBadlyNamedBoolean;
		boolean zCorrectlyNamedBoolean;
        byte xBadlyNamedByte;
        byte bCorrectlyNamedByte;
        short xBadlyNamedShort;
        short sCorrectlyNamedShort;
        int xBadlyNamedInt;
        int iCorrectlyNamedInt;
        long xBadlyNamedLong;
        long lCorrectlyNamedLong;
        float xBadlyNamedFloat;
        float fCorrectlyNamedFloat;
        double xBadlyNamedDouble;
        double dCorrectlyNamedDouble;
        char xBadlyNamedChar;
        char cCorrectlyNamedChar;
    }
}
