package com.togethersoft.modules.bettersoftwarefaster.patternInstance;

import com.togethersoft.modules.bettersoftwarefaster.rwisupport.*;
import com.togethersoft.openapi.ide.*;
import com.togethersoft.openapi.ide.message.*;
import com.togethersoft.openapi.rwi.*;
import com.togethersoft.openapi.rwi.enum.*;
import com.togethersoft.openapi.sci.*;

import java.util.*;

/**
 * Iterates over the RWI model, and applies the <<patternInstance>>
 * stereotype to any objects that are instantiated from a class
 * with <<pattern>> stereotype.
 */
public class PatternInstanceModule extends RwiVisitorSupport implements IdeScript {
    public PatternInstanceModule() {
        super("Pattern Instance");
    }

    public void run(IdeContext ideContext) {
        getMessagePage().printMessage("Started.");
        if (!isActiveProject())
            return;
		getRwiVisitorDispatcher().visitAndDispatch();
        getMessagePage().setVisible(true);
        getMessagePage().printMessage("Finished.");
    }

    public Object visitPackage(RwiPackage rwiPackage) {
        return null;
    }

    public boolean visitDiagrams() {
        return false;
    }
    public Object visitDiagram(RwiDiagram rwiDiagram) {
        return null;
    }

    public boolean visitNodes() {
        return true;
    }
    public Object visitNode(RwiNode rwiNode) {

        if (rwiNode.getProperty(RwiProperty.SHAPE_TYPE).equals(RwiShapeType.OBJECT)) {
            RwiElement nodeClass = RwiSupport.elementNamed(
                rwiNode.getProperty("$instantiatedClass"));

			if (nodeClass == null) {
                return null;
            }

			if (nodeClass.hasProperty("stereotype") &&
                nodeClass.getProperty("stereotype").equals("pattern")) {

				if (rwiNode.hasProperty("stereotype") &&
					rwiNode.getProperty("stereotype").equals("patternInstance")) {

                    return null; // nothing to do
                }
                rwiNode.setProperty("stereotype", "patternInstance");
		        getMessagePage().printMessage("<<patternInstance>> stereotype applied", rwiNode);
            }
        }
        return null;
    }

    public boolean visitMembers() { return false; }
    public Object visitMember(RwiMember rwiMember) {
        return null;
    }

    public boolean visitLinks() { return false; }
    public Object visitLink(RwiLink rwiLink) {
        return null;
    }

    /**
     * Maps rwiNodes (representing top-level classes) to a list of diagrams in which they appear as references
     * (that is, as a shortcut)
     * @supplierRole nodeDiagramList
     * @clientQualifier node
     * @link aggregationByValue
     * @supplierCardinality 1
     */
    private Map rwiNodeDiagramListMap = new HashMap();
}
