package com.togethersoft.modules.bettersoftwarefaster.rwisupport;

import com.togethersoft.openapi.ide.diagram.*;
import com.togethersoft.openapi.ide.editor.*;
import com.togethersoft.openapi.util.text.*;
import com.togethersoft.openapi.rwi.*;

import java.util.*;

/**
* Wraps up a message in the JList contained within the RwiMessagePage.  Provides
* support for jumping to either an RwiElement or an RwiReference in the model.
*
* @see RwiMessagePage
*/
public class RwiMessage {

    RwiMessage(String message) {
        this.message = message;
        this.rwiModel = RwiModelAccess.getModel();
        this.ideDiagramManager = IdeDiagramManagerAccess.getDiagramManager();
    }

    public RwiMessage(String message, RwiElement rwiElement) {
		this(message);
        this.rwiReference = null;
        this.rwiElement = rwiElement;
    }

    public RwiMessage(String message, RwiReference rwiReference) {
		this(message);
        this.rwiReference = rwiReference;
		this.rwiElement = rwiReference.getElement();
    }

    public void jumpToItem() {
        RwiDiagram rwiDiagram = null;
        IdeDiagram ideDiagram = null;
		IdeDiagramElement ideDiagramElement = null;

		// use rwiReference in preference to rwiElement
        if (rwiReference != null) {
            rwiDiagram = rwiReference.getContainingDiagram();
			if (rwiDiagram == null) {
                return; // shouldn't happen
            }
			ideDiagram = ideDiagramManager.openDiagram(rwiDiagram,true);
            ideDiagramElement = ideDiagram.findElement(rwiReference);
        } else
        if (rwiElement != null) {
			rwiDiagram = rwiModel.findDiagramFor(this.rwiElement);
			if (rwiDiagram == null) {
                return; // could happen ?
            }
			ideDiagram = ideDiagramManager.openDiagram(rwiDiagram,true);
            ideDiagramElement = ideDiagram.findElement(rwiElement);
		}
		ideDiagramManager.setActiveDiagram(ideDiagram);
        if (ideDiagramElement != null) {
			ideDiagram.unselectAll();
            ideDiagram.select(ideDiagramElement);
            ideDiagram.scrollTo(ideDiagramElement);
        }

		// from here on, use the underlying rwiElement if item held is a reference,
        // or the actual rwiElement if item held is an element
        if (rwiElement != null) {
            // get file and location of element within file
            String fileName = this.rwiElement.getProperty(RwiProperty.FILE);
            if (fileName != null) {
	            try {
	                int startPosition = Integer.parseInt(this.rwiElement.getProperty(RwiProperty.START_POSITION));
		            // get ideEditor for the file containing this element
		            IdeEditorManager ideEditorManager = IdeEditorManagerAccess.getEditorManager();
		            IdeEditor ideEditor = ideEditorManager.findEditor(fileName);
		            if (ideEditor == null) {
		                ideEditor = ideEditorManager.getDefaultEditor();
			            ideEditor.setFile(fileName);
		            }
		            ideEditorManager.setActiveEditor(ideEditor);
					// jump to element and select
					TextPosition startTextPosition = ideEditor.getPositionConverter().offsetToCell(startPosition);
					TextPosition endTextPosition = new TextPosition(startTextPosition.getLine()+1,0);
					IdeEditorVisibleArea viewArea = ideEditor.getVisibleArea();
					viewArea.setFirstLine(startTextPosition.getLine());
					IdeEditorSelection selection = ideEditor.getSelection(IdeEditorSelectionType.PRIMARY);
					selection.setSelectionStart(startTextPosition);
					selection.setSelectionEnd(endTextPosition);
	            } catch(Exception ex) {
	            }
			}
        }
    }

    public String toString() {
        return message;
    }

    final private String message;
    private RwiElement rwiElement = null;
    private RwiReference rwiReference = null;

    final private RwiModel rwiModel;
    final private IdeDiagramManager ideDiagramManager;
}
