package com.togethersoft.modules.bettersoftwarefaster.rwisupport;

import com.togethersoft.openapi.ide.*;
import com.togethersoft.openapi.ide.window.*;
import com.togethersoft.openapi.ide.project.*;
import com.togethersoft.openapi.ide.message.*;

import com.togethersoft.openapi.rwi.*;
import com.togethersoft.openapi.ide.message.*;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

/**
 * Support for printing out messages on a module-specific message tab. Messages
 * are printed out in a JList; single-clicking on an entry jumps to that node
 * in the package diagram in which it belongs (delegates to RwiMessage to
 * accomplish this).  Overloaded versions of <pre>printMessage()</pre> allow
 * just a simple String message to be printed, or a ValidationException, or a
 * collection of ValidationExceptions.
 *
 * @see ValidationException
 * @see ValidationExceptionCollector
 * @see RwiMessage
 */
public class RwiMessagePage {

	/** @stereotype factoryMethod */
    public static RwiMessagePage getMessagePage(String messagePageId, String messagePageTitle) {
       	RwiMessagePage rwiMessagePage = (RwiMessagePage)rwiMessagePageMap.get(messagePageId);
        if (rwiMessagePage == null) {
            rwiMessagePage = new RwiMessagePage(messagePageId, messagePageTitle);
            rwiMessagePageMap.put(messagePageId, rwiMessagePage);
        }
        return rwiMessagePage;
    }

    private RwiMessagePage(String messagePageId, String messagePageTitle ) {

        this.messagePageId = messagePageId;
        this.messagePageTitle = messagePageTitle;

		openPage(); // build our page, and open
	}

	/**
    * Opens page, and sets active.
    * Also, checks messagePage has not been closed on us, and rebuilds if necessary
    * (populating this.ideMessagePage, this.jlist and this.listModel in the process)
    */
	private IdeMessagePage openPage() {
		// make sure that we have our message page
		ideMessagePage = IdeMessageManagerAccess.getMessageManager().findPage(messagePageId);

        if (ideMessagePage != null) {
			IdeMessageManagerAccess.getMessageManager().setActivePage(ideMessagePage);
            return ideMessagePage;
        }

        listModel = new DefaultListModel();
		jlist = new JList(listModel);

        jlist.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
	    jlist.addListSelectionListener(
		new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent e) {
			    if (e.getValueIsAdjusting())
			        return;
			    if (!jlist.isSelectionEmpty()) {
			        int index = jlist.getSelectedIndex();
					RwiMessage rwiMessage = (RwiMessage)listModel.getElementAt(index);
                    rwiMessage.jumpToItem();
					setMessagePageActive();
			    }
            }
		});
        final JPopupMenu popup = new JPopupMenu();
        popup.add(
            new AbstractAction("Clear") {
            public void actionPerformed(ActionEvent ev) {
                listModel.clear();
            }
        } );

		class PopupListener extends MouseAdapter {
		    public void mousePressed(MouseEvent e) {
		        maybeShowPopup(e);
		    }
		    public void mouseReleased(MouseEvent e) {
		        maybeShowPopup(e);
		    }
		    private void maybeShowPopup(MouseEvent e) {
		        if (e.isPopupTrigger()) {
		            popup.show(e.getComponent(),
		                       e.getX(), e.getY());
		        }
		    }
		}
        jlist.addMouseListener(new PopupListener());

        JPanel jpanel = new JPanel(new GridLayout(1, 1));
        jpanel.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
        jpanel.add(new JScrollPane(jlist));

        ideMessagePage = IdeMessageManagerAccess.
            getMessageManager().
            openPage(
	            messagePageId,
	            jpanel,
	            null); // IdeMessageView - not used

		ideMessagePage.setTitle( messagePageTitle );

        return ideMessagePage;
    }

	private JList getJList() {
		openPage();
        return jlist;
    }

	private DefaultListModel getListModel() {
		openPage();
        return listModel;
    }

	public void printMessage(String message) {
		getListModel().addElement(new RwiMessage(message));
        scrollToLastRow();
    }
	public void printMessage(String message, RwiReference rwiReference) {
        getListModel().addElement(new RwiMessage(message, rwiReference));
        scrollToLastRow();
    }
	public void printMessage(String message, RwiElement rwiElement) {
        getListModel().addElement(new RwiMessage(message, rwiElement));
        scrollToLastRow();
    }
	public void printMessage(ValidationException ex) {
        printMessage(ex.getMessage(), ex.getRwiElement());
    }
	public void printMessage(ValidationExceptionCollector valExColl) {
        for (Iterator iter = valExColl.iterator(); iter.hasNext(); ) {
            ValidationException ex = (ValidationException)iter.next();
	        printMessage(ex);
        }
    }
	private void scrollToLastRow() {
        int lastRow = getListModel().getSize()-1;
		getJList().setSelectedIndex(lastRow);
		getJList().ensureIndexIsVisible(lastRow);
    }

	public void clear() {
        ((DefaultListModel)getJList().getModel()).clear();
    }
    public void setVisible(boolean visibility) {
        setMessagePageActive();
        IdeMessageManagerAccess.getMessageManager().setPaneVisible(visibility);
    }
    private void setMessagePageActive() {
        IdeMessageManagerAccess.getMessageManager().setActivePage(this.ideMessagePage);
    }

	final private String messagePageId;
	final private String messagePageTitle;

	/**
	 * @link aggregationByValue
	 *   @associates <{com.togethersoft.modules.rwi.RwiMessage}>
	 * @supplierCardinality 1
	 */
	private JList jlist;
	private DefaultListModel listModel;

    /**
     * @supplierCardinality 1
     * @link aggregationByValue 
     */
    private IdeMessagePage ideMessagePage;

	/**
     * Maps each messagePageId to an instance of RwiMessagePage
     * Multiple invocations of any given module will always pick up the
	 * same RwiMessagePage
	 */
	final static private Map rwiMessagePageMap = new HashMap();
}
