package com.togethersoft.modules.bettersoftwarefaster.rwisupport;

import com.togethersoft.openapi.ide.*;
import com.togethersoft.openapi.rwi.*;
import com.togethersoft.openapi.rwi.enum.*;

/**
 * Static support methods to help with inspecting a model.  Typically will be used by module implementations, only used sparingly by the framework itself. 
 */
public class RwiSupport {

    /** undocumented RwiShapeType */
	private final static String MESSAGE_LINK_SHAPE_TYPE = "MessageLink";
    /** undocumented RwiShapeType */
	private final static String NOTE_LINK_SHAPE_TYPE = "NoteLink";

	////////////////////////////////////////////////////////////////////

	static final public RwiModel getRwiModel() {
		return RwiModelAccess.getModel();
	}

	static final public RwiPackageEnumeration getModelRootPackages() {
		return getRwiModel().rootPackages(RwiProperty.MODEL);
	}

	static final private RwiPackageEnumeration getImportedRootPackages() {
		return getRwiModel().rootPackages(RwiProperty.IMPORT);
	}

	static final public RwiElement elementNamed(String uniqueName) {
		return getRwiModel().findElement(uniqueName);
	}

	////////////////////////////////////////////////////////////////////

	static final public String nameOf(RwiElement rwiElement) {
		return (rwiElement!=null)?rwiElement.getProperty(RwiProperty.NAME):"<null element>";
	}

	static final public String fullNameOf(RwiElement rwiElement) {
		return (rwiElement!=null)?rwiElement.getProperty(RwiProperty.FULL_NAME):"<null element>";
	}

	static final public String uniqueNameOf(RwiElement rwiElement) {
		return (rwiElement!=null)?rwiElement.getProperty(RwiProperty.UNIQUE_NAME):"<null element>";
	}

	static final public String shapeTypeOf(RwiElement rwiElement) {
		return (rwiElement!=null)?rwiElement.getProperty(RwiProperty.SHAPE_TYPE):"<null element>";
	}

	static final public RwiDiagram primaryDiagramFor(RwiElement rwiElement) {
		return getRwiModel().findDiagramFor(rwiElement);
	}

	static final public RwiPackage containingPackageFor(RwiElement rwiElement) {
		RwiDiagram rwiDiagram = primaryDiagramFor(rwiElement);
		RwiPackage designPackage = rwiDiagram.getDesignPackage();  // file package (*.dfXXX file)
		return designPackage.getContainingPackage();
	}

	////////////////////////////////////////////////////////////////////

	static final public boolean hasRwiNodeContainingNode(RwiNode rwiNode) {
 		return (rwiNode.getContainingNode() != null);
	}

	static final public RwiMember getNamedRwiMemberinRwiNode(RwiNode rwiNode, String rwiMemberName) {
		RwiMemberEnumeration rwiMemberEnumeration = rwiNode.members();

		RwiMember rwiMember;
		for( ; rwiMemberEnumeration.hasMoreElements(); ) {
			rwiMember = rwiMemberEnumeration.nextRwiMember();
			if (rwiMember.getProperty(RwiProperty.NAME).equals(rwiMemberName)) {
				return rwiMember;
			}
		}
		return null;
	}

   	////////////////////////////////////////////////////////////////////

	static final public boolean hasRwiNodeMemberNamed(RwiNode rwiNode, String rwiMemberName) {
		return (getNamedRwiMemberinRwiNode(rwiNode, rwiMemberName) != null);
	}


	static final public boolean isRwiNodeClassOrInterface(RwiNode rwiNode) {
		return rwiNode.getProperty(RwiProperty.SHAPE_TYPE).equals(RwiShapeType.CLASS);
	}

	static final public boolean isRwiNodeStrictlyClass(RwiNode rwiNode) {
		if (!isRwiNodeClassOrInterface(rwiNode))
			return false;
		return (!isRwiNodeStrictlyInterface(rwiNode));
	}

	static final public boolean isRwiNodeTopLevelClass(RwiNode rwiNode) {
		if (!isRwiNodeStrictlyClass(rwiNode))
			return false;
        return (rwiNode.getContainingNode() == null);
	}

	static final public boolean isRwiNodeStrictlyInterface(RwiNode rwiNode) {
		if (!isRwiNodeClassOrInterface(rwiNode))
			return false;
		return (rwiNode.hasProperty(RwiProperty.INTERFACE));
	}

	////////////////////////////////////////////////////////////////////

	static final public boolean isRwiMemberOperationOrConstructor(RwiMember rwiMember) {
		return rwiMember.getProperty(RwiProperty.SHAPE_TYPE).equals(RwiShapeType.OPERATION);
	}

	static final public boolean isRwiMemberStrictlyOperation(RwiMember rwiMember) {
		if (!isRwiMemberOperationOrConstructor(rwiMember))
			return false;
		return (!isRwiMemberStrictlyConstructor(rwiMember));
	}

	static final public boolean isRwiMemberStrictlyConstructor(RwiMember rwiMember) {
		if (!isRwiMemberOperationOrConstructor(rwiMember))
			return false;
		return (rwiMember.hasProperty(RwiProperty.CONSTRUCTOR));
	}

	static final public boolean isRwiMemberAttribute(RwiMember rwiMember) {
		return rwiMember.getProperty(RwiProperty.SHAPE_TYPE).equals(RwiShapeType.ATTRIBUTE);
	}

	static final public boolean isRwiMemberRepresentingLink(RwiMember rwiMember) {
		return rwiMember.getProperty(RwiProperty.MEMBER_LINK) != null;
	}

	static final public boolean isRwiMemberPublic(RwiMember rwiMember) {
 		return (rwiMember.hasProperty(RwiProperty.PUBLIC));
	}

	static final public boolean isRwiMemberProtected(RwiMember rwiMember) {
 		return (rwiMember.hasProperty(RwiProperty.PROTECTED));
	}

	static final public boolean isRwiMemberPackageLocal(RwiMember rwiMember) {
 		return (rwiMember.hasProperty(RwiProperty.PACKAGE_LOCAL));
	}

	static final public boolean isRwiMemberPrivate(RwiMember rwiMember) {
 		return (rwiMember.hasProperty(RwiProperty.PRIVATE));
	}

	////////////////////////////////////////////////////////////////////

	////////////////////////////////////////////////////////////////////

    static final public boolean isRwiNodeNote(RwiNode rwiNode) {
		return rwiNode.getProperty(RwiProperty.SHAPE_TYPE).equals(RwiShapeType.NOTE);
    }

	////////////////////////////////////////////////////////////////////

    static final public boolean isRwiLinkMessageLink(RwiLink rwiLink) {
		return rwiLink.getProperty(RwiProperty.SHAPE_TYPE).equals(MESSAGE_LINK_SHAPE_TYPE);
	}

	static final public boolean isRwiLinkNoteLink(RwiLink rwiLink) {
		return rwiLink.getProperty(RwiProperty.SHAPE_TYPE).equals(NOTE_LINK_SHAPE_TYPE);
	}

}


