package com.togethersoft.modules.bettersoftwarefaster.rwisupport;

import com.togethersoft.openapi.ide.*;
import com.togethersoft.openapi.ide.window.*;
import com.togethersoft.openapi.ide.project.*;
import com.togethersoft.openapi.ide.message.*;

import com.togethersoft.openapi.rwi.*;


/**
 * Support class for for visiting selective nodes within the RWI model.
 * Can either subclass from it, or delegate to it.
 * <p> If visitor and filter are not explicitly specified, then will use instance
 *     of self.
 * <li> Automatically instantiates a visitorDispatcher so that the RWI model
 *      can be iterated over simply using <pre>getRwiVisitorDispatcher().visitAndDispatch()</pre>
 * <li> Also has support for a module-specific message page, and some other helper methods.
 * <p>Not an IdeScript in its own right, but typically would be subclassed by a module that
 * would implement IdeScript, and would call the visitAndDispatch() method as described above.
 */
public class RwiVisitorSupport implements RwiVisitor, RwiVisitorFilter {

	public RwiVisitorSupport(
        	String messageTitle,
            RwiVisitor rwiVisitor, RwiVisitorFilter rwiVisitorFilter) {
        initialize(messageTitle, rwiVisitor, rwiVisitorFilter);
    }

	public RwiVisitorSupport(
        	String messageTitle) {
        initialize(messageTitle, this, this);
    }

    private void initialize(String messageTitle, RwiVisitor rwiVisitor, RwiVisitorFilter rwiVisitorFilter) {
		messagePage = RwiMessagePage.getMessagePage(this.getClass().getName(), messageTitle);
        rwiVisitorDispatcher = new RwiVisitorDispatcher(rwiVisitor, rwiVisitorFilter);
    }

	////////////////////////////////////////////////////////////////////

    protected RwiVisitorDispatcher getRwiVisitorDispatcher() {
        return rwiVisitorDispatcher;
    }

	/**
	 * @clientCardinality 1
	 * @supplierCardinality 1
	 * @link aggregationByValue
	 */
	private RwiVisitorDispatcher rwiVisitorDispatcher;

	////////////////////////////////////////////////////////////////////


    /**
     * @supplierCardinality 1
     * @link aggregationByValue 
     */
    private RwiMessagePage messagePage;

	////////////////////////////////////////////////////////////////////

	public Object visitPackage(RwiPackage rwiPackage){
        return null;
    }

    public boolean visitDiagrams(){ return true; }
	public Object visitDiagram(RwiDiagram rwiDiagram){
        return null;
	}

    public boolean visitNodes(){ return true; }
	public Object visitNode(RwiNode rwiNode) {
		return null;
    }

    public boolean visitMembers(){ return true; }
	public Object visitMember(RwiMember rwiMember){
        return null;
    }

    public boolean visitLinks(){ return true; }
	public Object visitLink(RwiLink rwiLink){
        return null;
    }

	////////////////////////////////////////////////////////////////////

    protected boolean isActiveProject() {
		if (IdeProjectManagerAccess.getProjectManager().getActiveProject() == null) {
            getMessagePage().printMessage("No open project");
			return false;
		}
        return true;
    }

	////////////////////////////////////////////////////////////////////

    protected RwiMessagePage getMessagePage() {
        return messagePage;
    }

	final protected void showMessageDialog(
							String title,
							int    dialogType,
							Object messageOrComponent) {
		IdeWindowManagerAccess.getWindowManager().showMessageDialog(
									title, dialogType, messageOrComponent);
	}
}


