package com.bettersoftwarefaster.carserv.infra;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.io.*;

/**
 * A panel that acts as a status bar.  Any messages posted to the status bar
 * will be removed after some specified time (by default, 20 seconds).
 * Exceptions can also be posted to the status bar; the status bar will
 * display ex.getMessage(), and clicking on the status bar will display a
 * dialogue box showing the toString() of that exception.  (This works well
 * with exceptions that subclass CarServException, exploiting the stack trace
 * unravelling functionality provided therein).
 * @see CarServException
 */
public class StatusBar extends JPanel {
    public StatusBar() {
        this(10);
    }

    public StatusBar(long clearMessageTimeout) {
        super(new FlowLayout(FlowLayout.LEFT));
        this.clearMessageTimeout = clearMessageTimeout;
        messageLabel = new JLabel(" ");
        add(messageLabel);
        addMouseListener(
        new MouseAdapter() {
            public void mouseReleased(MouseEvent ev) {
                if (exception != null) {
                    JTextArea messageTextArea =
                    new JTextArea(exception.getMessage() +
                    unravelled(exception));
                    messageTextArea.setEditable(false);
                    JScrollPane scrollPane = new JScrollPane(messageTextArea);
                    scrollPane.setPreferredSize(new Dimension(400, 200));
                    JOptionPane.showMessageDialog(StatusBar.this, scrollPane,
                    "Details", JOptionPane.INFORMATION_MESSAGE);
                }
            }
            private String unravelled(Exception ex) {
                if (ex == null) {
                    return " (no embedded exception).";
                }
                else {
                    StringWriter writer = new StringWriter();
                    ex.printStackTrace(new PrintWriter(writer));
                    return "\n>> embedded exception: " + ex.getMessage() +
                    "\n>> stack trace: " + writer.toString();
                }
            }
        });
    }

    public String getMessage() { return message; }

    public void setMessage(Exception ex) {
        this.exception = ex;
        if (ex.getMessage().length() == 0) {
            setMessage("Exception occurred.");
        }
        else {
            setMessage(ex.getMessage());
        }
    }

    public void setMessage(String message) {
        this.message = message;
        if (message.length() == 0) {
            messageLabel.setText(" ");
        }
        else {
            messageLabel.setText(message);
        }
        if (clearMessageTimeout > 0) {
            this.thread = new Thread() {
                public void run() {
                    try {
                        Thread.sleep(clearMessageTimeout * 1000);
                    }
                    catch (InterruptedException ex) {
                    }
                    finally {
                        if (thread == this) {
                            clearMessage();
                        }
                        else {
                            // do nothing, another more recent thread must
                            // now be running
                        }
                    }
                }
            };
            thread.start();
        }
    }

    public void clearMessage() {
        exception = null;
        message = null;
        messageLabel.setText(" ");
    }

    public long getClearMessageTimeout() {
        return clearMessageTimeout;
    }

    public void setClearMessageTimeout(long clearMessageTimeout) {
        this.clearMessageTimeout = clearMessageTimeout;
    }

    private Thread thread;
    private final JLabel messageLabel;
    private String message;
    private Exception exception;
    private long clearMessageTimeout;
}
